#include "cxt_mgr.h"
#include "gpio_model.h"
#include "board_gpio.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "aver_xilinx.h"

enum {
	GPIO_IC_AVER311,
	GPIO_IC_NUM,
};

typedef struct {
	const char *const *pin_name;
	U8_T pin_no;
} pin_desc_t;

typedef struct {
	const char *ic_name;
	pin_desc_t *pin_desc;
} gpio_config_t;

static const char *const pin_name[BOARD_GPIO_NUM] = {
    [BOARD_GPIO_RESERVED] = "PIN_RESERVED",
    [BOARD_GPIO_EDID_WP_D] = "PIN_EDID_WP_DIGITAL",
    [BOARD_GPIO_HOT_PLUG] = "PIN_HOT_PLUG",
    [BOARD_GPIO_RESET_ADV7604] = "PIN_RESET_ADV7604",
    [BOARD_GPIO_PWD_ADV7604] = "PIN_PWD_ADV7604",
    [BOARD_GPIO_RESET_AIC3101] = "PIN_RESET_AIC3101",
    [BOARD_GPIO_RESET_ADV7180] = "PIN_RESET_ADV7180",
    [BOARD_GPIO_PWD_ADV7180] = "PIN_PWD_ADV7180",
    [BOARD_GPIO_EDID_W1] = "PIN_EDID_W1_ENABLE",
	[BOARD_GPIO_EDID_W2] = "PIN_EDID_W2_ENABLE",
	[BOARD_GPIO_LOCK_ADV7604] = "PIN_LOCK_ADV7604",
	[BOARD_GPIO_LOCK_ADV7180] = "PIN_LOCK_ADV7180",
	//[BOARD_GPIO_RESET_GV7601] = "PIN_RESET_GV7601",
	//[BOARD_GPIO_HDSD_SET_GV7601] = "PIN_HDSD_SET_GV7601",
};

static pin_desc_t aver_xilinx_pin_desc[] = {
	{
		.pin_name = &pin_name[BOARD_GPIO_RESERVED],
		.pin_no = AVER_XILINX_GPIO_0,
	}, {
		.pin_name = &pin_name[BOARD_GPIO_EDID_WP_D],
		.pin_no = AVER_XILINX_GPIO_1,
	}, {
		.pin_name = &pin_name[BOARD_GPIO_HOT_PLUG],
		.pin_no = AVER_XILINX_GPIO_2,
	}, {
		.pin_name = &pin_name[BOARD_GPIO_RESET_ADV7604],
		.pin_no = AVER_XILINX_GPIO_3,
	}, {
		.pin_name = &pin_name[BOARD_GPIO_PWD_ADV7604],
		.pin_no = AVER_XILINX_GPIO_4,
	}, {
                .pin_name = &pin_name[BOARD_GPIO_RESET_AIC3101],
		.pin_no = AVER_XILINX_GPIO_5,
	}, {
                .pin_name = &pin_name[BOARD_GPIO_RESET_ADV7180],
		.pin_no = AVER_XILINX_GPIO_6,
	}, {	
                .pin_name = &pin_name[BOARD_GPIO_PWD_ADV7180],
		.pin_no = AVER_XILINX_GPIO_7,
	}, {
                .pin_name = &pin_name[BOARD_GPIO_EDID_W1],
		.pin_no = AVER_XILINX_GPIO_8,
	}, {
                .pin_name = &pin_name[BOARD_GPIO_EDID_W2],
		.pin_no = AVER_XILINX_GPIO_9,
	}, {
                .pin_name = &pin_name[BOARD_GPIO_LOCK_ADV7604],
		.pin_no = AVER_XILINX_GPIO_10,
	}, {
                .pin_name = &pin_name[BOARD_GPIO_LOCK_ADV7180],
		.pin_no = AVER_XILINX_GPIO_11,
	}, {
		NULL,
	},
};

static gpio_config_t gpio_config[GPIO_IC_NUM] = {
	{
		.ic_name = AVER_XILINX_GPIODRV_NAME,
		.pin_desc = aver_xilinx_pin_desc,
	},
};

int board_gpio_init(cxt_mgr_handle_t cxt_mgr)
{
	gpio_model_handle_t gpio_mgr;
	int i;

	gpio_mgr = cxt_manager_get_context(cxt_mgr, GPIO_CXT_ID, 0);
	if (gpio_mgr) {
		for (i = 0; i < GPIO_IC_NUM; i++) {
			pin_desc_t *pin_desc;

			pin_desc = gpio_config[i].pin_desc;
			while (pin_desc->pin_name) {
				gpio_model_register_pin(gpio_mgr, gpio_config[i].ic_name, *pin_desc->pin_name, pin_desc->pin_no);
				pin_desc++;
			}
		}
#if 0
		for (i = 0; i < BOARD_GPIO_NUM; i++)
		{
			board_gpio_cxt->gpio_pin_handle[i] = gpio_model_request_pin_handle(gpio_mgr, pin_name[i]);
			if (board_gpio_cxt->gpio_pin_handle[i] == GPIO_MODEL_BAD_PIN_HANDLE)
			{
				printk(" request %s failed\n", pin_name[i]);
			}
		}
#endif
	}

	return 0;
}

void board_set_gpio(cxt_mgr_handle_t cxt_mgr, board_gpio_e no, gpio_model_pin_value_t value)
{
	gpio_model_handle_t gpio_mgr;

	do {
		gpio_model_pin_handle_t pin_handle;
		if(!cxt_mgr)
			break;

		gpio_mgr = cxt_manager_get_context(cxt_mgr, GPIO_CXT_ID, 0);
		if (!gpio_mgr)
			break;

		if (no < 0 || no > BOARD_GPIO_NUM)
			break;

		pin_handle = gpio_model_request_pin_handle(gpio_mgr, pin_name[no]);
		if (!pin_handle)
			break;

		gpio_model_pin_set(pin_handle, value);
		gpio_model_pin_release(pin_handle);
	} while (0);
}

gpio_model_pin_value_t board_get_gpio(cxt_mgr_handle_t cxt_mgr, board_gpio_e no)
{
	gpio_model_handle_t gpio_mgr;
	gpio_model_pin_value_t value = GPIOIC_VALUE_UNKNOWN;
	
	do {
		gpio_model_pin_handle_t pin_handle;
		if (!cxt_mgr)
			break;

		gpio_mgr = cxt_manager_get_context(cxt_mgr, GPIO_CXT_ID, 0);
		if (!gpio_mgr)
			break;

		if (no < 0 || no > BOARD_GPIO_NUM)
			break;

		pin_handle = gpio_model_request_pin_handle(gpio_mgr, pin_name[no]);
		if (!pin_handle)
			break;

		value = gpio_model_pin_get(pin_handle);
		gpio_model_pin_release(pin_handle);
	} while (0);
	
	return value;
}

