#include <linux/kernel.h>
#include "cxt_mgr.h"
#include "framegrabber.h"
#include "mem_model.h"
#include "v4l2_model.h"
#include "board_v4l2.h"
#include "debug.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "task_model.h"
#include "gpio_model.h"
#include "aver_xilinx.h"
#include "adv7604.h"
#include "m24c02.h"
#include "board_alsa.h"
#include "board_gpio.h"
#include "adv7180.h"
#include "aic3101.h"

//#define adv7604_event
static int cnt_retry=0;
enum cl311m2_input_e {
	CL311M2_HDMI_INPUT = 0,
    CL311M2_YPBPR_INPUT,
    CL311M2_CVBS_INPUT,
    CL311M2_SVIDEO_INPUT,
	//CL311M2_VGA_INPUT,
	//CL311M2_SDI_INPUT,
	CL311M2_INPUT_COUNT,
};

enum cl311m2_i2c_chip_e {
	CL311M2_I2C_CHIP_M24C02,
	CL311M2_I2C_CHIP_ADV7604, 
    CL311M2_I2C_CHIP_ADV7180,
    CL311M2_I2C_CHIP_AIC3101,
	CL311M2_I2C_CHIP_COUNT,
};

typedef struct {
	BASIC_CXT_HANDLE_DECLARE;
	framegrabber_handle_t fg_handle;
	v4l2_model_handle_t v4l2_handle;
	task_model_handle_t task_model_handle;
	handle_t aver_xilinx_handle;
	handle_t i2c_chip_handle[CL311M2_I2C_CHIP_COUNT];
	task_handle_t check_signal_task_handle;
	adv7604_frameinfo_t adv7604_cur_frameinfo;
	adv7604_frameinfo_t adv7604_last_frameinfo;
	adv7180_frameinfo_t adv7180_cur_frameinfo;
	adv7180_frameinfo_t adv7180_last_frameinfo;
} board_v4l2_context_t;

typedef struct {
	const char *name;
	int index;
} board_chip_desc_t;

static board_chip_desc_t cl311m2_chip_desc[CL311M2_I2C_CHIP_COUNT] = {
	[CL311M2_I2C_CHIP_M24C02] = {
		.name = M24C02_DRVNAME,
		.index = 1,
	},
	
	[CL311M2_I2C_CHIP_ADV7604] = {
		.name = ADV7604_DRVNAME,
		.index = 1,
	},
	
	[CL311M2_I2C_CHIP_ADV7180] = {
		.name = ADV7180_DRVNAME,
		.index = 1,
	},

        [CL311M2_I2C_CHIP_AIC3101] = {
		.name = AIC3101_DRVNAME,
		.index = 1,
	},
};

static framegrabber_setup_input_info_t cl311m2_input_info[] = {
	[CL311M2_HDMI_INPUT] = {
		.name = "HDMI",
		.support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			//[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	#if 0
	[CL311M2_VGA_INPUT] = {
		.name = "VGA",
		.support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	#endif
        [CL311M2_YPBPR_INPUT] = {
		.name = "YPBPR",
		.support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
    [CL311M2_CVBS_INPUT] = {
		.name = "CVBS",
		.support_framesize_info = {
			//[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			//[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			//[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			//[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	[CL311M2_SVIDEO_INPUT] = {
		.name = "S-Video",
		.support_framesize_info = {
			//[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			//[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			//[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			//[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	#if 0
	[CL311M2_SDI_INPUT] = {
		.name = "SDI",
		.support_framesize_info = {
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	#endif
	[CL311M2_INPUT_COUNT] = {
		.name = NULL,
	},
};

static framegrabber_property_t cl311m2_property = {
	.name = "CL311M2",
	.input_setup_info = cl311m2_input_info,
	.support_out_pixfmt_mask = FRAMEGRABBER_PIXFMT_YUYV_BIT,
	//.max_supported_line_width = 3840,
	.max_frame_size = 1920 * 1080,
};

static v4l2_model_device_setup_t device_info =
{
	.type = DEVICE_TYPE_GRABBER,
	.driver_name = "CL311M2",
	.card_name = "AVerMedia CL311M2",
	.capabilities = V4L2_MODEL_CAPS_CAPTURE_BIT | V4L2_MODEL_CAPS_READWRITE_BIT | V4L2_MODEL_CAPS_STREAMING_BIT,
	.buffer_type = V4L2_MODEL_BUF_TYPE_DMA_SG,
};

static void *board_v4l2_alloc(void);
static void board_v4l2_release(void *cxt);

static void fg_s_input(framegrabber_handle_t handle, int input);
static void fg_stream_on(framegrabber_handle_t handle);
static void fg_stream_off(framegrabber_handle_t handle);
static void fg_wr_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid);
static void fg_rd_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid);

//static void cl311m2_adv7604_event(void *cxt, adv7604_event_e event);
static void cl311m2_adv7604_digital_event(void *cxt, adv7604_event_e event);
static void cl311m2_adv7604_analog_event(void *cxt, adv7604_event_e event);
static void cl311m2_adv7180_event(void *cxt, adv7180_event_e event);

static void video_buffer_done(void *data);

static framegrabber_interface_t cl311m2_ops = {
	.s_input      = fg_s_input,
	.stream_on    = fg_stream_on,
	.stream_off   = fg_stream_off,
	.wr_edid      = fg_wr_edid,
	.rd_edid      = fg_rd_edid,
};

static void *board_v4l2_alloc()
{
	board_v4l2_context_t *cxt;

	cxt = mem_model_alloc_buffer(sizeof(*cxt));

	return cxt;
}

static void board_v4l2_release(void *cxt)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;

	if (board_v4l2_cxt) {
		if (board_v4l2_cxt->check_signal_task_handle) {
			task_model_release_task(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle);
			board_v4l2_cxt->check_signal_task_handle = NULL;
		}
		cxt_manager_unref_context(board_v4l2_cxt->task_model_handle);
		cxt_manager_unref_context(board_v4l2_cxt->aver_xilinx_handle);
		mem_model_free_buffer(board_v4l2_cxt);
	}
}

static void v4l2_stream_on(v4l2_model_callback_parameter_t *cb_info)
{
	board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	debug_msg("%s %p\n", __func__, board_v4l2_cxt);
}

static void v4l2_stream_off(v4l2_model_callback_parameter_t *cb_info)
{
	board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	debug_msg("%s %p\n", __func__, board_v4l2_cxt);
}

static void v4l2_buffer_prepare(v4l2_model_callback_parameter_t *cb_info)
{
	board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	v4l2_model_buffer_info_t *buffer_info = cb_info->u.buffer_prepare_info.buffer_info;

	debug_msg("%s %p buffer_info %p\n", __func__, board_v4l2_cxt, buffer_info);
	if (buffer_info) {
		int i;
		v4l2_model_buf_desc_t *desc;
		unsigned framebufsize, remain;
		int width, height;
		unsigned bytesperline;

		framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle, &width, &height);
		bytesperline = framegrabber_g_out_bytesperline(board_v4l2_cxt->fg_handle);
		framebufsize = bytesperline * height;
		debug_msg("%s %dx%d framesize %u\n", __func__, width, height, framebufsize);
		debug_msg("buf type %d count %d\n", buffer_info->buf_type, buffer_info->buf_count[0]);
		for (i = 0, desc = buffer_info->buf_info[0], remain = framebufsize; i < buffer_info->buf_count[0]; i++) {
			debug_msg("addr %08x size %x\n", desc[i].addr, desc[i].size);
			if (remain >= desc[i].size) {
				aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle, desc[i].addr, desc[i].size, 0);
				remain -= desc[i].size;
			} else {
				//aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle, desc[i].addr, remain);
				//remain = 0;
				if (remain !=0) // in order to avoid video stop as size 0 set to FPGA //20190225
				{
					aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle,desc[i].addr,remain, 0);
					remain =0;
				}
				break;
			}
		}

		aver_xilinx_active_current_desclist(board_v4l2_cxt->aver_xilinx_handle, video_buffer_done, board_v4l2_cxt, 0);
	}
}

static void v4l2_buffer_init(v4l2_model_callback_parameter_t *cb_info)
{
	//board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	v4l2_model_buffer_info_t *buffer_info = cb_info->u.buffer_prepare_info.buffer_info;

	debug_msg("%s buffer_info %p\n", __func__,buffer_info);
	if (buffer_info) {
		int i;
		v4l2_model_buf_desc_t *desc;
		debug_msg("buf type %d count %d\n", buffer_info->buf_type, buffer_info->buf_count[0]);
		for (i = 0, desc = buffer_info->buf_info[0]; i < buffer_info->buf_count[0]; i++) {
			debug_msg("addr %08x size %x\n", desc[i].addr,desc[i].size);
		}
	}
}

static void fg_s_input(framegrabber_handle_t handle, int input)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	//cxt_mgr_handle_t cxt_mgr = get_cxt_manager_from_context(board_v4l2_cxt);
	
	handle_t adv7604_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7604];
	handle_t adv7180_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7180];
	handle_t aic3101_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_AIC3101];
	
	printk("========%s input=%d\n", __func__, input);
	
	framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
	
	//input =CL311M2_YPBPR_INPUT;
	//aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,input);
#if 1
	if (input == CL311M2_HDMI_INPUT /*|| input == CL311M2_YPBPR_INPUT*/) {
		adv7180_unregister_callback(adv7180_handle);
		
		adv7604_unregister_callback(adv7604_handle);
		
		//adv7604_register_callback(adv7604_handle, cl311m2_adv7604_digital_event, board_v4l2_cxt);
		
		//adv7604_set_sync_ch(adv7604_handle,ADV7604_SYNC_CH1);
		
		//aver_xilinx_adv7604_reset(board_v4l2_cxt->aver_xilinx_handle); //internal edid wil be cleaned
		
		//aic3101_audio_select(aic3101_handle,AIC3101_INPUT_NONE);
		
		adv7604_set_input(adv7604_handle,ADV7604_MODE_HDMI);
		
	    aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_VIDEO_INPUT_0);
	    
	    aver_xilinx_select_audio_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_AUDIO_INPUT_0);
	    
	    printk("%s =====callback CL311M2_HDMI_INPUT\n", __func__);
	     
	    adv7604_register_callback(adv7604_handle, cl311m2_adv7604_digital_event, board_v4l2_cxt);
	    
		//adv7604_register_callback(adv7604_handle, cl311m2_adv7604_event, board_v4l2_cxt);
       
	} 
	#if 1
	else if (input == CL311M2_YPBPR_INPUT) {
		 adv7180_unregister_callback(adv7180_handle);
		 
		 adv7604_unregister_callback(adv7604_handle);
		 
		 //adv7604_register_callback(adv7604_handle, cl311m2_adv7604_analog_event, board_v4l2_cxt);
		 
		 //adv7604_set_sync_ch(adv7604_handle,ADV7604_SYNC_CH2);
		 
		 //aver_xilinx_adv7604_reset(board_v4l2_cxt->aver_xilinx_handle);	 
		  
		 adv7604_set_input(adv7604_handle,ADV7604_MODE_COMP);
		 
		 aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_VIDEO_INPUT_0);
		 
		 aver_xilinx_select_audio_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_AUDIO_INPUT_1);
		 
		 aic3101_audio_select(aic3101_handle,AIC3101_COMP);
		 
		 printk("%s =====callback CL311M2_YPBPR_INPUT\n", __func__);
		 
		 adv7604_register_callback(adv7604_handle, cl311m2_adv7604_analog_event, board_v4l2_cxt);
		//gv7601_unregister_callback(gv7601_handle);
        //board_set_gpio(cxt_mgr, BOARD_GPIO_ADV7604_I2S_EN, GPIOIC_VALUE_HIGH);
		//board_set_gpio(cxt_mgr, BOARD_GPIO_AIC3101_I2S_EN, GPIOIC_VALUE_HIGH);
		
		//adv7604_register_callback(adv7604_handle, cl311m2_adv7604_event, board_v4l2_cxt);
        
	}
	#endif
	else if (input == CL311M2_CVBS_INPUT) {
		adv7604_unregister_callback(adv7604_handle);
		
		adv7180_set_input(adv7180_handle,ADV7180_INPUT_CVBS);
		
        aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_VIDEO_INPUT_1);
        
        aver_xilinx_select_audio_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_AUDIO_INPUT_1);
        
        aic3101_audio_select(aic3101_handle,AIC3101_CVBS);
      
        printk("%s callback CL311M2_CVBS_INPUT\n", __func__);
        
		adv7180_register_callback(adv7180_handle, cl311m2_adv7180_event, board_v4l2_cxt);
		
	}
	else if (input == CL311M2_SVIDEO_INPUT) {
		adv7604_unregister_callback(adv7604_handle);
		
		adv7180_set_input(adv7180_handle,ADV7180_INPUT_S_VIDEO);
		
        aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_VIDEO_INPUT_1);
        
        aver_xilinx_select_audio_input(board_v4l2_cxt->aver_xilinx_handle,AVER_XILINX_AUDIO_INPUT_1);
        
        aic3101_audio_select(aic3101_handle,AIC3101_SVIDEO);
		
		printk("%s CL311M2_SVIDEO_INPUT\n", __func__);
      
		adv7180_register_callback(adv7180_handle, cl311m2_adv7180_event, board_v4l2_cxt);
	}
#endif	
}

static void fg_stream_on(framegrabber_handle_t handle)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	//cxt_mgr_handle_t cxt_mgr = get_cxt_manager_from_context(board_v4l2_cxt);
	handle_t adv7604_handle=board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7604]; 
	aver_xilinx_video_process_cfg_t vip_cfg;
	aver_xilinx_audio_process_cfg_t aup_cfg;
	int width, height, out_width, out_height, out_framerate;
	int crop_left, crop_top, crop_width, crop_height;
	framegrabber_framemode_e framemode;
	const framegrabber_pixfmt_t *pixfmt=framegrabber_g_out_pixelfmt(handle);
	int input;
	int audio_rate=0;
	enum adv7604_audio_sample fe_audioinfo=0;
	
	printk(">>>>>>%s \n", __func__);
	
	adv7604_get_audioinfo(adv7604_handle,&fe_audioinfo);
	
	audio_rate = fe_audioinfo;
	
	//framegrabber_s_input_audioinfo(board_v4l2_cxt->fg_handle,audio_rate);
	
	switch (audio_rate)
    {
		case ADV7604_AUDIO_RATE_32000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_32000;
		break;
		case ADV7604_AUDIO_RATE_44100:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_44100;
		break;
		case ADV7604_AUDIO_RATE_48000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		break;
		case ADV7604_AUDIO_RATE_88200:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_88200;
		break;
		case ADV7604_AUDIO_RATE_96000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_96000;
		break;
		case ADV7604_AUDIO_RATE_176400:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_176400;
		break;
		case ADV7604_AUDIO_RATE_192000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_192000;
		break;
		default:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		break;
	}
    printk("%s...vip_cfg.audio_rate_index=%d\n",__func__,vip_cfg.audio_rate);
		
	framemode = framegrabber_g_input_framemode(board_v4l2_cxt->fg_handle);
	//framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle, &width, &height);
	framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle, &out_width, &out_height);
	out_framerate = framegrabber_g_out_framerate(board_v4l2_cxt->fg_handle);
	framegrabber_g_crop_rect(board_v4l2_cxt->fg_handle, &crop_left, &crop_top, &crop_width, &crop_height);
	
	input = framegrabber_g_input(board_v4l2_cxt->fg_handle);

    printk("%s input=%d\n", __func__, input);
	vip_cfg.is_interlace = (FRAMEGRABBER_FRAMEMODE_INTERLACED == framemode) ? AVER_TRUE : AVER_FALSE;
	
#if 0
	vip_cfg.is_interlace = (FRAMEGRABBER_FRAMEMODE_INTERLACED == framemode) ? AVER_TRUE : AVER_FALSE;
	vip_cfg.in_framesize.width = width;
	vip_cfg.in_framesize.height = height;
	vip_cfg.out_framesize.width = out_width;
	vip_cfg.out_framesize.height = out_height;
	vip_cfg.out_framerate = out_framerate;
#endif	
	//vip_cfg.clip_start.x = crop_left;
	//vip_cfg.clip_start.y = crop_top;
	//vip_cfg.clip_size.width = crop_width;
	//vip_cfg.clip_size.height = crop_height;
	vip_cfg.clip_start.x = 0;
	vip_cfg.clip_start.y = 0;
	vip_cfg.clip_size.width = width;
	vip_cfg.clip_size.height = height;
	//printk("%s in %dx%d out %dx%d %d\n", __func__, width, height, out_width, out_height, out_framerate);

	//vip_cfg.valid_mask = FRAMERATE_CFG_VALID_MASK | CLIP_CFG_VALID_MASK | SCALER_CFG_VALID_MASK;
	//vip_cfg.valid_mask = FRAMERATE_CFG_VALID_MASK | SCALER_CFG_VALID_MASK;
	//vip_cfg.pixel_clock = 0;
	
    vip_cfg.valid_mask=SCALER_CFG_VALID_MASK|FRAMERATE_CFG_VALID_MASK;
    
    vip_cfg.in_framerate = framegrabber_g_input_framerate(board_v4l2_cxt->fg_handle);
#if 0    
    if (vip_cfg.is_interlace==AVER_TRUE) 
        vip_cfg.in_framerate = vip_cfg.in_framerate *2; //same as fpga detect and move to adv7604
    else
        vip_cfg.in_framerate = vip_cfg.in_framerate;
#endif	
#if 1 //audio and video path
	if (input == CL311M2_HDMI_INPUT)
	{
	    aup_cfg.input = AVER_XILINX_AUDIO_INPUT_0; 
	    aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,0);
	} 
	else if (input == CL311M2_YPBPR_INPUT)
	{
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		aup_cfg.input = AVER_XILINX_AUDIO_INPUT_1; 
		aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,0);
	} 
	else if (input == CL311M2_CVBS_INPUT)
	{
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
        aup_cfg.input = AVER_XILINX_AUDIO_INPUT_1;
        aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,1);
	} 
	else if (input == CL311M2_SVIDEO_INPUT)
	{
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		aup_cfg.input = AVER_XILINX_AUDIO_INPUT_1;
		aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,1); 
	}
#endif	

	aver_xilinx_config_audio_process(board_v4l2_cxt->aver_xilinx_handle, &aup_cfg);
	
    framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle, &width, &height);
    
    if ((input == CL311M2_CVBS_INPUT) && (height ==480))
	{
	    vip_cfg.clip_start.y = 2*(0x0c); //NTSC VBI offset
	    printk("vbi offset\n");
	} 
    
    printk("==%s in %dx%d out %dx%d %d/100\n", __func__, width, height, out_width, out_height, out_framerate);
    
#if 1
	vip_cfg.is_interlace = (FRAMEGRABBER_FRAMEMODE_INTERLACED == framemode) ? AVER_TRUE : AVER_FALSE;
	vip_cfg.in_framesize.width = width;
	vip_cfg.in_framesize.height = height;
	vip_cfg.out_framesize.width = out_width;
	vip_cfg.out_framesize.height = out_height;
	vip_cfg.out_framerate = out_framerate/100;
#endif

    switch(pixfmt->pixfmt_out)
    {
        case AVER_XILINX_FMT_YUYV:
            vip_cfg.pixel_format=AVER_XILINX_FMT_YUYV;
            mesg("%s...vip_cfg.pixel_format_output=aver_xilinx_FMT_YUYV\n",__func__);
            break;
        case AVER_XILINX_FMT_UYVY:
        case AVER_XILINX_FMT_YVYU:
        case AVER_XILINX_FMT_VYUY:
            vip_cfg.pixel_format=AVER_XILINX_FMT_UYVY;
            mesg("%s...vip_cfg.pixel_format_output=aver_xilinx_FMT_UYVY\n",__func__);
            break;
        case AVER_XILINX_FMT_RGBP: //RGB565
        case AVER_XILINX_FMT_RGBR: //RGB565X
        case AVER_XILINX_FMT_RGBO:
        case AVER_XILINX_FMT_RGBQ:
        case AVER_XILINX_FMT_RGB3: //RGB24 4:4:4
            vip_cfg.pixel_format=AVER_XILINX_FMT_RGB3;
            mesg("%s...vip_cfg.pixel_format=aver_xilinx_FMT_RGB3\n",__func__);
            break; 
        case AVER_XILINX_FMT_BGR3: 
        case AVER_XILINX_FMT_RGB4:
        case AVER_XILINX_FMT_BGR4:
            vip_cfg.pixel_format=AVER_XILINX_FMT_BGR3;
            mesg("%s...vip_cfg.pixel_format=aver_xilinx_FMT_BGR3\n",__func__);
            break;
        default:
            vip_cfg.pixel_format=AVER_XILINX_FMT_YUYV;
            break;
    }       
    
    vip_cfg.pixel_format_out = pixfmt->pixfmt_out;
	aver_xilinx_config_video_process(board_v4l2_cxt->aver_xilinx_handle, &vip_cfg, 0);
	aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_TRUE, 0);
    //printk("vip_cfg.in_framesize.width=%d and height=%d\n",vip_cfg.in_framesize.width,vip_cfg.in_framesize.height);
	//task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 100000);
}

static void fg_stream_off(framegrabber_handle_t handle)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
        printk(">>>>>>%s \n", __func__);	
	aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_FALSE, 0);
}

static void fg_wr_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	handle_t m24c02_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_M24C02];

	m24c02_wr_edid(m24c02_handle, start_block, blocks, edid);
}

static void fg_rd_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	handle_t m24c02_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_M24C02];

	m24c02_rd_edid(m24c02_handle, start_block, blocks, edid);
}

static void video_buffer_done(void *data)
{
	board_v4l2_context_t *board_v4l2_cxt = data;

	debug_msg("%s board_v4l2_cxt %p\n", __func__, board_v4l2_cxt);
	if (board_v4l2_cxt) {
		v4l2_model_buffer_done(board_v4l2_cxt->v4l2_handle);
	}
}

static void check_signal_stable_task(void *data)
{
    board_v4l2_context_t *board_v4l2_cxt=data;
    int width,height;
    
    aver_xilinx_frame_info_t detected_frameinfo;
    //adv7619_frameinfo_t *fe_frameinfo=&board_v4l2_cxt->cur_fe_frameinfo; 
    BOOL_T is_interlace;
    
    debug_msg("%s\n",__func__);
    
    //adv7619_get_frameinfo(adv7619_handle,fe_frameinfo);
    //mesg("%s adv7619 detected size %dx%d;framerate %d \n",__func__,fe_frameinfo->width,fe_frameinfo->height,fe_frameinfo->framerate);
        
    //aver_xilinx_get_frameinfo(board_v4l2_cxt->aver_xilinx_handle,&detected_frameinfo,fe_frameinfo->pixel_clock/*0*/); 
    aver_xilinx_get_frameinfo(board_v4l2_cxt->aver_xilinx_handle, &detected_frameinfo, AVER_XILINX_VIDEO_INPUT_0, 0);
    //framemode=framegrabber_g_input_framemode(board_v4l2_cxt->fg_handle);
    //is_interlace=(framemode==FRAMEGRABBER_FRAMEMODE_INTERLACED) ? AVER_TRUE : AVER_FALSE;
    is_interlace = framegrabber_g_input_interlace(board_v4l2_cxt->fg_handle);
    framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle,&width,&height);
	
	if ((detected_frameinfo.width == 720) && (detected_frameinfo.height ==483))
    {
		detected_frameinfo.height =480;
	}
	
    if((((detected_frameinfo.width<320 || detected_frameinfo.height<240)) || (width !=detected_frameinfo.width)|| (height !=detected_frameinfo.height)) && (cnt_retry<1))
    {
		cnt_retry++;
		printk("%s retry...in %dx%d  get %dx%d\n",__func__,width,height,detected_frameinfo.width,detected_frameinfo.height); //test
        task_model_run_task_after(board_v4l2_cxt->task_model_handle,board_v4l2_cxt->check_signal_task_handle,10000);
        return;
    }
    
    if(width==detected_frameinfo.width && height==detected_frameinfo.height && (is_interlace==detected_frameinfo.is_interlace))
    {
		cnt_retry=0;
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
    }else
    {
        printk("===============%s detected input %dx%d not match to fpga %dx%d\n",__func__,width,height,detected_frameinfo.width,detected_frameinfo.height);
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); 
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);  
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0); 
        //printk("set mask as LOCKED_BIT=0\n");
    }      
}

static void cl311m2_adv7604_digital_event(void *cxt, adv7604_event_e event)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;
	handle_t adv7604_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7604];
	//aver_xilinx_video_cfg_t cfg;
	adv7604_frameinfo_t *cur_frameinfo=&board_v4l2_cxt->adv7604_cur_frameinfo; 
	//adv7604_frameinfo_t *last_frameinfo=&board_v4l2_cxt->adv7604_last_frameinfo; 
	unsigned int hdcp_flag;
	
	//printk("%s event=%d\n",__func__,event);
	switch(event) {
		case ADV7604_HDMI_LOCK:	
			//printk("ADV7604_HDMI_LOCK...\n");
           
            //adv7604_get_frameinfo_last(adv7604_handle,last_frameinfo);
			
			//framegrabber_s_input_frameinfo_last(board_v4l2_cxt->fg_handle,last_frameinfo->width,last_frameinfo->height,last_frameinfo->is_interlace);
			
			adv7604_get_frameinfo(adv7604_handle,cur_frameinfo);
			
			//aver_xilinx_hdmi_hotplug(board_v4l2_cxt->aver_xilinx_handle);
			//printk("%s current wxh=%dx%d; framerate=%d; is interlace=%d\n",__func__,(int)cur_frameinfo->width,
            //(int)cur_frameinfo->height,(int)cur_frameinfo->framerate,cur_frameinfo->is_interlace);	
			
            framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle, cur_frameinfo->is_interlace);
            //adv7604_get_audioinfo(adv7604_handle,fe_audioinfo);
			
			//framegrabber_s_input_audioinfo(board_v4l2_cxt->fg_handle,*fe_audioinfo);
			 
            framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,cur_frameinfo->framerate, 0);
            
            framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,cur_frameinfo->is_interlace);

            if(cur_frameinfo->is_interlace == 0)
            {
            	framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,cur_frameinfo->width,cur_frameinfo->height);
            }
            else
            {
            	framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,cur_frameinfo->width,cur_frameinfo->height / 2);
            }
			#if 1
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
			
			task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 10000); //us
			#endif
			//task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 50000);
			//printk("ADV7604_LOCK_OUT\n");
			break;

		case ADV7604_HDMI_UNLOCK:
		//case ADV7604_NOTIFY_NOSIGNAL:
		//case ADV7604_NOTIFY_NOSYNC:
		    //adv7604_enable_streaming(adv7604_handle,AVER_FALSE);
			printk("ADV7604_HDMI_UNLOCK\n");
			//aver_xilinx_hdmi_hotplug(board_v4l2_cxt->aver_xilinx_handle);
			
			//framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); 
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
			break;
			
		case ADV7604_HDCP:
            adv7604_get_frameinfo(adv7604_handle,cur_frameinfo);
            hdcp_flag = cur_frameinfo->adv7604_hdcp_flag;
		    //framegrabber_s_hdcp_flag(board_v4l2_cxt->fg_handle,hdcp_flag);
		    #if 0
		    if (hdcp_flag==1)
		    {
		        framegrabber_s_hdcp_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_HDCP_STATUS_ENABLE);
		        framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_OK);
			}
		    else
		    {
		        framegrabber_s_hdcp_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_HDCP_STATUS_DISABLE);
			}
			#endif
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_OK);
		    
		    
		    //framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SYNC_BIT);
		    break;  	

		case ADV7604_HDCP_ENABLE:
		    printk("=====================ADV7604_HDCP_ENABLE=====\n");
			framegrabber_s_hdcp_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_HDCP_STATUS_ENABLE);
			break;
		case ADV7604_HDCP_DISABLE:
		    printk("=====================ADV7604_HDCP_DISABLE=====\n");
			framegrabber_s_hdcp_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_HDCP_STATUS_DISABLE);
			break;	
		default: 
		    printk("ADV7604_HDMI_OTHER_EVENT\n");
		    break;	
	}
}

static void cl311m2_adv7604_analog_event(void *cxt, adv7604_event_e event)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;
	handle_t adv7604_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7604];
	//aver_xilinx_video_cfg_t cfg;
	adv7604_frameinfo_t *cur_frameinfo=&board_v4l2_cxt->adv7604_cur_frameinfo; 
	//adv7604_frameinfo_t *last_frameinfo=&board_v4l2_cxt->adv7604_last_frameinfo; 
	#ifdef adv7604_event
	printk("%s event=%d\n",__func__,event);
	#endif
	switch(event) {
		case ADV7604_YPBPR_LOCK:
		
		    #ifdef adv7604_event	
			printk("ADV7604_YPBPR_LOCK\n");
			#endif
			
			//adv7604_get_frameinfo_last(adv7604_handle,last_frameinfo);
			
			//framegrabber_s_input_frameinfo_last(board_v4l2_cxt->fg_handle,last_frameinfo->width,last_frameinfo->height,last_frameinfo->is_interlace);
            framegrabber_s_hdcp_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_HDCP_STATUS_DISABLE);
            
			adv7604_get_frameinfo(adv7604_handle,cur_frameinfo);
			//aver_xilinx_hdmi_hotplug(board_v4l2_cxt->aver_xilinx_handle);
			#ifdef adv7604_event	
			printk("%s current wxh=%dx%d; framerate=%d; is interlace=%d\n",__func__,(int)cur_frameinfo->width,
            (int)cur_frameinfo->height,(int)cur_frameinfo->framerate,cur_frameinfo->is_interlace);	
			#endif
            framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle, cur_frameinfo->is_interlace);
            //adv7604_get_audioinfo(adv7604_handle,fe_audioinfo);
			
			//framegrabber_s_input_audioinfo(board_v4l2_cxt->fg_handle,*fe_audioinfo);
			 
            framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,cur_frameinfo->framerate, 0);
            
            //framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,cur_frameinfo->is_interlace);
            
            framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,cur_frameinfo->width,cur_frameinfo->height); 

			#if 1
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
			
			task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 10000);
			#endif
			//task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 50000);
			//printk("ADV7604_LOCK_OUT\n");
			break;

		case ADV7604_YPBPR_UNLOCK:
		//case ADV7604_NOTIFY_NOSIGNAL:
		//case ADV7604_NOTIFY_NOSYNC:
		    //adv7604_enable_streaming(adv7604_handle,AVER_FALSE);
			printk("ADV7604_YPBPR_UNLOCK\n");
			
			//framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); 
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0);
			break;

		default: 
		    printk("ADV7604_YPBPR_OTHER_EVENT\n");
		    break;	
	}
}

static void cl311m2_adv7180_event(void *cxt, adv7180_event_e event)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;
	//handle_t aver_xilinx_handle = board_v4l2_cxt->aver_xilinx_handle;
	handle_t adv7180_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7180];
	adv7180_frameinfo_t *fe_frameinfo=&board_v4l2_cxt->adv7180_cur_frameinfo; 
	int macrovision_flag;
	
	//printk("%s event %d\n", __func__, event);
	switch(event) {
		case ADV7180_LOCK:
			printk("ADV7180_LOCK\n");
			framegrabber_s_hdcp_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_HDCP_STATUS_DISABLE);
		    adv7180_get_frameinfo(adv7180_handle,fe_frameinfo);
		
			printk("%s wxh=%dx%d; framerate=%d; is interlace=%d\n",__func__,(int)fe_frameinfo->width,
            (int)fe_frameinfo->height,(int)fe_frameinfo->framerate,fe_frameinfo->is_interlace);	
			
            framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle, fe_frameinfo->is_interlace);
            //adv7604_get_audioinfo(adv7604_handle,fe_audioinfo);
			
			//framegrabber_s_input_audioinfo(board_v4l2_cxt->fg_handle,*fe_audioinfo);
			 
            framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,fe_frameinfo->framerate, 0);
            
            framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,fe_frameinfo->is_interlace);
            
            framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,fe_frameinfo->width,fe_frameinfo->height); //tt 0602
            
			#if 1
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
			framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,
				FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
			//task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 10);
			#endif
			break;
			
		case ADV7180_MACROVISION:
            //debug_msg("ADV7180_MACROVISION\n");
            
            adv7180_get_frameinfo(adv7180_handle,fe_frameinfo);
            macrovision_flag = fe_frameinfo->macrovision_level;
            //printk("ADV7180_MACROVISION control(%d)\n",macrovision_flag);
		    //framegrabber_s_hdcp_flag(board_v4l2_cxt->fg_handle,hdcp_flag);
		    if (macrovision_flag !=0)
		    {
		        framegrabber_s_macrovision_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_MACROVISION_STATUS_ENABLE);
		        framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SYNC_BIT);
			}
		    else
		    {
		        framegrabber_s_macrovision_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_MACROVISION_STATUS_DISABLE);
			}
		    
		    
		    //framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SYNC_BIT);
		    break;  	

		case ADV7180_UNLOCK:
			printk("ADV7180_UNLOCK\n");

			//framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); 
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0);
			break;
	}
}

#if 0
static void cl311m2_gv7601_event(void *cxt, gv7601_event_e event)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;
	//handle_t aver_xilinx_handle = board_v4l2_cxt->aver_xilinx_handle;
	handle_t gv7601_handle = board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_GV7601];
	//gv7601_frameinfo_t frameinfo;
	aver_xilinx_video_cfg_t cfg;
	gv7601_frameinfo_t *fe_frameinfo=&board_v4l2_cxt->gv7601_cur_frameinfo; 
	//gv7601_frameinfo_t *fe_frameinfo; 
	//adv7601_system_type_e system_type; 
	//GV7601_eResolutionType_t system_type; //tbd 0415 for HD/SD ?

	debug_msg("%s event %d\n", __func__, event);
	switch(event) {
		case GV7601_LOCK:
			debug_msg("GV7601_LOCK\n");
			printk("GV7601_LOCK\n");
			//gv7601_get_frameinfo(gv7601_handle, &frameinfo);
			gv7601_get_frameinfo(gv7601_handle,fe_frameinfo);
			
			//cfg.width = frameinfo.width;
			//cfg.height = frameinfo.height;
#if 1
			cfg.width = fe_frameinfo->width;
			cfg.height = fe_frameinfo->height;
			cfg.htotal = fe_frameinfo->htotal; 
			///cfg.hbackporch = frameinfo.hbackporch;
			///cfg.vbackporch = frameinfo.vbackporch;
			cfg.framerate = fe_frameinfo->framerate;
			cfg.is_interlace = fe_frameinfo->is_interlace;
			//cfg.colorspace = (fe_frameinfo->colorspace == CS_YUV) ? aver_xilinx_CS_YUV444 : aver_xilinx_CS_RGB_FULL; //tbd 0415
#endif

			//system_type = gv7601_get_system_type(gv7601_handle);
			//debug_msg("%s system_type %d\n", __func__, system_type);
			//aver_xilinx_config_gv7601_input(board_v4l2_cxt->aver_xilinx_handle, system_type); //tbd 0415
			framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,fe_frameinfo->framerate);
            framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,fe_frameinfo->width,fe_frameinfo->height); 
			//gv7601_enable_dark_screen(mdin400_handle, AVER_FALSE);
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_OK);
			framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,
				FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
			break;

		case GV7601_UNLOCK:
			debug_msg("GV7601_UNLOCK\n");

			//gv7601_enable_dark_screen(mdin400_handle, AVER_TRUE);
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			//framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0);
			break;
	}
}
#endif
void board_v4l2_init(cxt_mgr_handle_t cxt_mgr)
{
	framegrabber_handle_t framegrabber_handle;
	board_v4l2_context_t *board_v4l2_cxt = NULL;
	v4l2_model_handle_t v4l2_handle;
	task_model_handle_t task_model_handle = NULL;
	handle_t i2c_mgr = NULL;
	handle_t aver_xilinx_handle = NULL;
	task_handle_t task_handle = NULL;
	int i;
	enum {
		BOARD_V4L2_OK = 0,
		BOARD_V4L2_ERROR_CXT_MGR,
		BOARD_V4L2_ERROR_FRAMEGRABBER_INIT,
		BOARD_V4L2_ERROR_V4L2_MODEL_INIT,
		BOARD_V4L2_ERROR_ALLOC_CXT,
		BOARD_V4L2_ERROR_GET_MAIN_CHIP_HANDLE,
		BOARD_V4L2_ERROR_GET_I2C_MGR,
		BOARD_V4L2_ERROR_GET_TASK_MGR,
		BOARD_V4L2_ERROR_ALLOC_TASK,
	} err = BOARD_V4L2_OK;

	do {
		if (!cxt_mgr) {
			err = BOARD_V4L2_ERROR_CXT_MGR;
			break;
		}

		framegrabber_handle = framegrabber_init(cxt_mgr, &cl311m2_property, &cl311m2_ops);
		if (framegrabber_handle == NULL) {
			err = BOARD_V4L2_ERROR_FRAMEGRABBER_INIT;
			break;
		}

		board_v4l2_cxt = cxt_manager_add_cxt(cxt_mgr, BOARD_V4L2_CXT_ID, board_v4l2_alloc, board_v4l2_release);
		if (!board_v4l2_cxt) {
			err = BOARD_V4L2_ERROR_ALLOC_CXT;
			break;
		}

		board_v4l2_cxt->fg_handle = framegrabber_handle;
		framegrabber_set_data(framegrabber_handle, board_v4l2_cxt);
		
		v4l2_handle = v4l2_model_init(cxt_mgr, &device_info, framegrabber_handle, 0);
		if (v4l2_handle == NULL) {
			err = BOARD_V4L2_ERROR_V4L2_MODEL_INIT;
			break;
		}
		board_v4l2_cxt->v4l2_handle = v4l2_handle;

		aver_xilinx_handle = cxt_manager_get_context(cxt_mgr, AVER_XILINX_CXT_ID, 0);
		if (aver_xilinx_handle == NULL) {
			err = BOARD_V4L2_ERROR_GET_MAIN_CHIP_HANDLE;
			break;
		}
		board_v4l2_cxt->aver_xilinx_handle = aver_xilinx_handle;

		i2c_mgr = cxt_manager_get_context(cxt_mgr, I2C_CXT_ID, 0);
		if (i2c_mgr == NULL) {
			err = BOARD_V4L2_ERROR_GET_I2C_MGR;
			break;
		}

		for (i = 0; i < CL311M2_I2C_CHIP_COUNT; i++) {
			board_v4l2_cxt->i2c_chip_handle[i] = i2c_model_get_nth_driver_handle(i2c_mgr, 
				 cl311m2_chip_desc[i].name, cl311m2_chip_desc[i].index);

			printk("i2c_chip_handle[%d]=%p\n", i, board_v4l2_cxt->i2c_chip_handle[i]);
		}

		task_model_handle = cxt_manager_get_context(cxt_mgr, TASK_MODEL_CXT_ID, 0);
		if (task_model_handle == NULL) {
			err = BOARD_V4L2_ERROR_GET_TASK_MGR;
			break;
		}
		board_v4l2_cxt->task_model_handle = task_model_handle;
		cxt_manager_ref_context(task_model_handle);

		task_handle = task_model_create_task(task_model_handle, check_signal_stable_task, board_v4l2_cxt, "check_signal_task");
		if (task_handle == NULL) {
			err = BOARD_V4L2_ERROR_ALLOC_TASK;
			break;
		}
		board_v4l2_cxt->check_signal_task_handle = task_handle;

		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_STREAMON, &v4l2_stream_on, board_v4l2_cxt);
		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_STREAMOFF, &v4l2_stream_off, board_v4l2_cxt);
		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_BUFFER_PREPARE, &v4l2_buffer_prepare, board_v4l2_cxt);
		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_BUFFER_INIT, &v4l2_buffer_init, board_v4l2_cxt);
        
        //adv7604_register_callback(board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_ADV7604], cl311m2_adv7604_event, board_v4l2_cxt);
        //gv7601_register_callback(board_v4l2_cxt->i2c_chip_handle[CL311M2_I2C_CHIP_GV7601], cl311m2_gv7601_event, board_v4l2_cxt);
        
		framegrabber_start(framegrabber_handle);

		cxt_manager_ref_context(aver_xilinx_handle);
	} while (0);

	if (err != BOARD_V4L2_OK) {
		switch (err) {
			case BOARD_V4L2_ERROR_ALLOC_TASK:
				cxt_manager_unref_context(task_model_handle);
			case BOARD_V4L2_ERROR_GET_TASK_MGR:
			case BOARD_V4L2_ERROR_GET_I2C_MGR:
			case BOARD_V4L2_ERROR_V4L2_MODEL_INIT:
			case BOARD_V4L2_ERROR_ALLOC_CXT:
			case BOARD_V4L2_ERROR_FRAMEGRABBER_INIT:
			case BOARD_V4L2_ERROR_CXT_MGR:
			default:
				break;
		}
		debug_msg("%s err %d\n", __func__, err);
	}
}

