/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * board_config.c
 *
 *  Created on: Jan 25, 2018
 *      Author: 
 *      Version:
 * =================================================================
 */
 
#include <linux/kernel.h>
#include "board.h"
#include "cxt_mgr.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "mem_model.h"
#include "gpio_model.h"
#include "task_model.h"
#include "trace.h"
#include "debug.h"
#include "aver_xilinx.h"
#include "board_gpio.h"
#include "board_i2c.h"
#include "board_alsa.h"
#include "board_v4l2.h"

int board_init(void);
void board_exit(void);


//#ifdef CONFIG_CL311M2
//const char *BOARD_NAME = "cl311m2";
//pci_model_id_t id_table[] = {  //need to be updated
//	{
//		.vendor = 0x1461,
//		.device = 0x0054,
//		.sub_vendor = 0x1461,
//		.sub_device = 0x3114,
//		.driver_data = CL311M2,
//	}, {
//		0
//	}
//};
//#else
const char *BOARD_NAME = "cl311mn";
pci_model_id_t id_table[] = {  //need to be updated
	{
		.vendor = 0x1461,
		.device = 0x0054,
		.sub_vendor = 0x1461,
		.sub_device = 0x3111,
		.driver_data = CL311MN,
	}, {
		0
	}
};
//#endif

static aver_xilinx_cfg_t aver_xilinx_cfg = {
	.gpio_mask = Bit(23) | Bit(25) | Bit(26) | Bit(27),
	.gpio_value = Bit(25) | Bit(27),
	.i2c_bus_speed = {
    	[AVER_XILINX_I2C_BUS_0] = 125 * 1000,
		//[aver_xilinx_I2C_BUS_1] = 125 * 1000,
		//[aver_xilinx_I2C_BUS_2] = 400 * 1000,
		//[aver_xilinx_I2C_BUS_3] = 125 * 1000,
	},
	.audio_buffer_size = 12 * 1024,
};

int board_probe(struct device *dev, unsigned long driver_info)
{
	cxt_mgr_handle_t cxt_mgr = get_cxt_manager(dev);
	handle_t pci_handle = NULL;
	//handle_t trace_handle = NULL;
	handle_t i2c_mgr = NULL;
	handle_t gpio_mgr = NULL;
	handle_t task_handle = NULL;
	handle_t mem_handle = NULL;
	handle_t aver_xilinx_handle = NULL;
	int i;
	int ret;
	enum {
		NO_ERROR = 0,
		NO_PCI_HANDLE,
		ERROR_TRACE_HANDLE,
		ERROR_I2C_MGR,
		ERROR_GPIO_MGR,
		ERROR_TASK_HANDLE,
		ERROR_MEM_HANDLE,
		ERROR_AVER311,
		ERROR_BOARD_I2C_INIT,
		ERROR_BOARD_GPIO_INIT,
	} err = NO_ERROR;

	printk("%s\n", __func__);
	do {
		pci_handle = pci_model_get_handle(cxt_mgr);
		if (!pci_handle) {
			err = NO_PCI_HANDLE;
			break;
		}

		//trace_handle = trace_model_init(cxt_mgr);
		//if (!trace_handle) {
		//	err = ERROR_TRACE_HANDLE;
		//	break;
		//}

		i2c_mgr = i2c_model_init(cxt_mgr);
		if (!i2c_mgr) {
			err = ERROR_I2C_MGR;
			break;
		}

		gpio_mgr = gpio_model_init(cxt_mgr);
		if (!gpio_mgr) {
			err = ERROR_GPIO_MGR;
			break;
		}

		mem_handle = mem_model_init(cxt_mgr);
		if (!mem_handle) {
			err = ERROR_MEM_HANDLE;
			break;
		}

		task_handle = task_model_init(cxt_mgr);
		if (!task_handle) {
			err = ERROR_TASK_HANDLE;
			break;
		}

		aver_xilinx_handle = aver_xilinx_init(cxt_mgr, pci_handle, &aver_xilinx_cfg);
		if (!aver_xilinx_handle) {
			err = ERROR_AVER311;
			break;
		}
		
		for (i = 0; i < BOARD_I2C_BUS_NUM; i++) {
			i2c_model_bus_cfg_t i2c_bus_cfg;
			const char *i2c_bus_name;

			i2c_bus_name = board_get_i2c_bus_name(i);
			switch (i) {
				case I2C_BUS_COM_I2C0:
					aver_xilinx_get_i2c_bus_cfg(aver_xilinx_handle, AVER_XILINX_I2C_BUS_0, &i2c_bus_cfg);
					i2c_model_new_bus(i2c_mgr, i2c_bus_name, &i2c_bus_cfg);
					break;

				//case I2C_BUS_COM_I2C1:
				//	aver_xilinx_get_i2c_bus_cfg(aver_xilinx_handle, aver_xilinx_I2C_BUS_1, &i2c_bus_cfg);
				//	i2c_model_new_bus(i2c_mgr, i2c_bus_name, &i2c_bus_cfg);
				//	break;

				//case I2C_BUS_COM_I2C3:
				//	aver_xilinx_get_i2c_bus_cfg(aver_xilinx_handle, aver_xilinx_I2C_BUS_3, &i2c_bus_cfg);
				//	i2c_model_new_bus(i2c_mgr, i2c_bus_name, &i2c_bus_cfg);
				//	break;
			}
		}
		
		//printk("%s break\n",__func__);
        //break;
        
		ret = board_gpio_init(cxt_mgr);
		if (ret != 0) {
			err = ERROR_BOARD_GPIO_INIT;
			break;
		}

		ret = board_i2c_init(cxt_mgr, driver_info);
		if (ret != 0) {
			err = ERROR_BOARD_I2C_INIT;
			break;
		}

		board_alsa_init(cxt_mgr);
		board_v4l2_init(cxt_mgr);
		
	} while (0);
    //err = ERROR_AVER311;
	if (err != NO_ERROR) {
		debug_msg("%s err\n", __func__, err);
		switch (err) {
			case ERROR_AVER311:
				cxt_manager_unref_context(i2c_mgr);
			case ERROR_I2C_MGR:
				cxt_manager_unref_context(gpio_mgr);
			case ERROR_GPIO_MGR:
				//cxt_manager_unref_context(trace_handle);
			case ERROR_TRACE_HANDLE:
			case NO_PCI_HANDLE:
				break;
			default:
				break;
		}
		return err;
	}

	return 0;
}

void board_remove(struct device *dev)
{
	debug_msg("%s\n",__func__);
}

int board_init()
{
	pci_model_driver_setup_t pci_setup;

	pci_setup.name = BOARD_NAME;
	pci_setup.id_table = id_table;
	pci_setup.prob_func = board_probe;
	pci_setup.remove_func = board_remove;
	pci_setup.flags = PCI_MODEL_DISABLE_MSI | PCI_MODEL_FORCE_DMA_32;
	pci_model_driver_init(&pci_setup);

	return 0;
}

void board_exit()
{
	pci_model_driver_exit();
}

