/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * board_config.c
 *
 *  Created on: Jan 25, 2018
 *      Author: 
 *      Version:
 * =================================================================
 */
 
#include <linux/kernel.h>
#include "cxt_mgr.h"
#include "i2c_model.h"
#include "gpio_model.h"
#include "board.h"
#include "board_gpio.h"
#include "board_i2c.h"
#include "debug.h"
#include "sys.h"
#include "m24c02.h"
#include "aic3101.h"
#include "adv7604.h"
#include "gv7601.h"
#include "pca9539a.h"

static const char *i2c_bus_name[BOARD_I2C_BUS_NUM] = {
	[I2C_BUS_COM_I2C0] = "I2C_BUS_COM_I2C0",
	//[I2C_BUS_COM_I2C1] = "I2C_BUS_COM_I2C1",
	//[I2C_BUS_COM_I2C3] = "I2C_BUS_COM_I2C3",
};

static void m24c02_wr_enable(handle_t cxt_mgr_handle, BOOL_T enable);
static void aic3101_hw_reset(handle_t cxt_mgr_handle);
static void pca9539a_hw_reset(handle_t cxt_mgr_handle);

static m24c02_cfg_t m24c02_cfg = {
	.wr_enable_func = m24c02_wr_enable,
};

static pca9539a_cfg_t pca9539a_cfg = {
	.hw_reset_func = pca9539a_hw_reset,
};

static aic3101_cfg_t aic3101_cfg = {
	.hw_reset_func = aic3101_hw_reset,
};

static adv7604_cfg_t adv7604_cfg = {
	.out_ctrl = {
		.out_fmt = ADV7604_OP_FORMAT_SEL_SDR_444_36_MODE0,
		.out_colorspace = ADV7604_OUT_YUV709,
	},
	//.connector_hdmi = 1,
	.ADC={AIN4,AIN5,AIN6,NC},
	.SOG={SYNC2,SYNC2},
    .op_ch_sel = ADV7604_OP_CH_SEL_RGB,
    .op_format_sel = ADV7604_OP_FORMAT_SEL_SDR_444_36_MODE0,
	.sync_ch_sel ={ ADV7604_SYNC_CH_SEL_HDMI, ADV7604_SYNC_CH_SEL_AIN2_GR},

	/* IO register 0x02 */
    .alt_gamma = 1,
    .op_656_range = 1,
    .rgb_out = 0,
    .alt_data_sat = 0,

	/* IO register 0x05 */
    .blank_data = 1,
    .insert_av_codes = 1,
    .replicate_av_codes = 0,
    .invert_cbcr = 0, //cb cr swap

    .output_bus_lsb_to_msb = 0,
    .hdmi_free_run_mode = 0,
    
    //.hw_reset_func = adv7604_hw_reset,
};

static gv7601_cfg_t gv7601_cfg = {
	.out_ctrl = {
		.out_fmt = 0,
		.out_colorspace = GV7601_OUT_YUV709,
	},
	
	//.hw_reset_func = gv7601_hw_reset,
};

static i2c_dev_info_t i2c_bus0_devices[] = {
	
	{
		.name = AIC3101_DRVNAME,
		.addr = 0x30 >> 1,
		.context = &aic3101_cfg,
	}, 
	{
		.name = PCA9539A_DRVNAME,
		.addr = 0xE8 >> 1,
		.context = &pca9539a_cfg,
	}, 
	{
		.name = ADV7604_DRVNAME,
		.addr = 0x40 >> 1,
		.context = &adv7604_cfg,
	},
	{
		.name = M24C02_DRVNAME, //reserved 0205
		.addr = 0xa0 >> 1,
		.context = &m24c02_cfg,
	}, 
	{
		.name = GV7601_DRVNAME,
		.addr = 0xFF >> 1, //pseudo address
		.context = &gv7601_cfg,
	}, {
		NULL,
	}
};

#if 0
static i2c_dev_info_t i2c_bus1_devices[] = { //tbd 0117
	{
		.name = GV7601_DRVNAME,
		.addr = 0xFF >> 1, //pseudo address
		.context = &gv7601_cfg,
	}, {
		NULL,
	}
};
#endif

static i2c_dev_info_t *board_i2c_bus_devices[SUPPORT_BOARD_NUM][BOARD_I2C_BUS_NUM] = {
	[CL311MN][I2C_BUS_COM_I2C0] = &i2c_bus0_devices[0],
	//[CL311MN][I2C_BUS_COM_I2C1] = &i2c_bus1_devices[0],
};

static void m24c02_wr_enable(handle_t cxt_mgr_handle, BOOL_T enable)
{
	debug_msg("%s %d\n", __func__, enable);

	sys_msleep(5);
	//board_set_gpio(cxt_mgr_handle, BOARD_GPIO_ANA_EDID_UPDATE, (enable == AVER_TRUE) ? GPIOIC_VALUE_HIGH : GPIOIC_VALUE_LOW);
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_EDID_WP_A, (enable == AVER_TRUE) ? GPIOIC_VALUE_LOW : GPIOIC_VALUE_HIGH);
	sys_msleep(5);
}

static void pca9539a_hw_reset(handle_t cxt_mgr_handle) //check 0122 need to add for pca9539a
{
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_PCA9539A, GPIOIC_VALUE_LOW);
	sys_msleep(5);
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_PCA9539A, GPIOIC_VALUE_HIGH);
	sys_msleep(5);
}


static void aic3101_hw_reset(handle_t cxt_mgr_handle)
{
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_AIC3101, GPIOIC_VALUE_LOW);
	sys_msleep(5);
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_AIC3101, GPIOIC_VALUE_HIGH);
	sys_msleep(5);
	
#if 0	
	DWORD temp = ReadRegisterDWORD(REG_AVER_BRIDGE_XILINX_GPIO);
        temp &= ~BIT(5);
        WriteRegisterDWORD(REG_AVER_BRIDGE_XILINX_GPIO, temp);

        WaitMS(10);

        // high
        temp = ReadRegisterDWORD(REG_AVER_BRIDGE_XILINX_GPIO);
        temp |= BIT(5);
        WriteRegisterDWORD(REG_AVER_BRIDGE_XILINX_GPIO, temp);
#endif	
}

#if 0
static void gv7601_hw_reset(handle_t cxt_mgr_handle) //check 0205
{
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_GV7601, GPIOIC_VALUE_LOW);
	sys_msleep(5);
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_GV7601, GPIOIC_VALUE_HIGH);
	sys_msleep(5);
}

static void adv7604_hw_reset(handle_t cxt_mgr_handle) //check 0205
{
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_ADV7604, GPIOIC_VALUE_LOW);
	sys_msleep(5);
	board_set_gpio(cxt_mgr_handle, BOARD_GPIO_RESET_ADV7604, GPIOIC_VALUE_HIGH);
}
#endif

const char *board_get_i2c_bus_name(int no)
{
	if (no < 0 || no >= BOARD_I2C_BUS_NUM)
		return NULL;

	return i2c_bus_name[no];
}

int board_i2c_init(cxt_mgr_handle_t cxt_mgr, unsigned long dev_id)
{
	i2c_model_handle_t i2c_mgr = cxt_manager_get_context(cxt_mgr, I2C_CXT_ID, 0);
	int err = 0;
	int ret = 0;

	if (i2c_mgr) {
		int i;
		i2c_dev_info_t *dev_info;
		
		pca9539a_hw_reset(cxt_mgr);
		aic3101_hw_reset(cxt_mgr);

		for (i = 0; i < BOARD_I2C_BUS_NUM; i++) {
			if (board_i2c_bus_devices[dev_id][i]) {
				dev_info = &board_i2c_bus_devices[dev_id][i][0];

				while (err == 0 && dev_info->name != NULL) {
					ret = i2c_model_bus_attach_device(i2c_mgr, i2c_bus_name[i], dev_info->name, dev_info->addr, dev_info->context);
					if (ret == 0) {
						dev_info++;
					} else {
						err = -1;
						//debug_msg("error in attach %s %d\n", dev_info->name,ret);
						printk("error in attach %s %d\n", dev_info->name,ret);
						break;
					}
				}

				if (err) {
					break;
				}
			}
		}
	}

	return err;
}

