/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * board_v4l2.c
 *
 *  Created on: Apr 15, 2017
 *      Author: 
 *      Version:
 * =================================================================
 */
 
#include <linux/kernel.h>
#include "cxt_mgr.h"
#include "framegrabber.h"
#include "mem_model.h"
#include "v4l2_model.h"
#include "board_v4l2.h"
#include "debug.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "task_model.h"
#include "aver_xilinx.h"
#include "cn312_sdi.h"
#include "cn312_hdmi.h" //rr
#include "board_alsa.h"

static int cnt_retry=0;
enum cn312_input_e
{
	CN312_SDI_INPUT=0,
	CN312_SDI_INPUT1=1,
	CN312_INPUT_COUNT,
};

enum cn312_dual_stream_e
{
	CN312S_SDI_STREAM0=0,
	CN312S_SDI_STREAM1,
	CN312S_STREAM_COUNT,
};

//enum cn312_i2c_chip_e
//{
//    CN312_I2C_CHIP_SDI_0,
//    CN312_I2C_CHIP_SDI_1,
//    CN312_I2C_CHIP_COUNT,
//};

enum cn312_i2c_device_e
{
    CN312_I2C_DEVICE_0,
    CN312_I2C_DEVICE_1,
    CN312_I2C_DEVICE_COUNT,
};

typedef struct
{
	BASIC_CXT_HANDLE_DECLARE;
    framegrabber_handle_t fg_handle;
    v4l2_model_handle_t v4l2_handle;
    task_model_handle_t task_model_handle;
    handle_t aver_xilinx_handle;
    //handle_t i2c_chip_handle[CN312_I2C_CHIP_COUNT];
    handle_t i2c_device_handle[CN312_I2C_DEVICE_COUNT];
    sdi_frameinfo_t cur_sdi_frameinfo;
    sdi_packetinfo_t cur_sdi_packetinfo;
    enum sdi_audio_sample cur_sdi_audiorate;
    hdmi_frameinfo_t cur_hdmi_frameinfo;
    hdmi_packetinfo_t cur_hdmi_packetinfo;
    enum hdmi_audio_sample cur_hdmi_audiorate;
    int cur_bchs_value;
    int cur_bchs_selection;
    task_handle_t check_signal_task_handle;
    int device_index;
    int card_id;
}board_v4l2_context_t;

typedef struct
{
    const char *name;
    int index;
}board_chip_desc_t;

typedef struct
{
    const char *name;
    int index;
}board_device_desc_t;

static board_device_desc_t cn312_device_desc[CN312_I2C_DEVICE_COUNT]=
{
    [CN312_I2C_DEVICE_0]=
    {
        .name=HDMI_DRVNAME,
        .index=1,
    },
    [CN312_I2C_DEVICE_1]=
    {
        .name=SDI_DRVNAME,
        .index=1,
    },  
};

static framegrabber_setup_input_info_t cn312_input_info[] =
{
    [CN312_SDI_INPUT] =
    {
        .name = "HDMI",
        .support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			//[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_800x600]   = REFRESHRATE_120_BIT | REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			//[FRAMESIZE_800x600]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_1024x768]  = REFRESHRATE_120_BIT | REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			//[FRAMESIZE_1024x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x864]  = REFRESHRATE_75_BIT,
			//[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_120_BIT | REFRESHRATE_100_BIT | REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x768]  = REFRESHRATE_120_BIT | REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x800]  = REFRESHRATE_120_BIT | REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x960]  = REFRESHRATE_85_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x1024] = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1360x768]  = REFRESHRATE_120_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1366x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x900]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		    //[FRAMESIZE_3840x2160] = REFRESHRATE_30_BIT, 
            //[FRAMESIZE_4096x2160] = REFRESHRATE_30_BIT,   
		},                         
    },
    [CN312_SDI_INPUT1] = //stream B 
    {
        .name = "SDI",
        .support_framesize_info = {
			//[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			//[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			//[FRAMESIZE_800x600]   = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			//[FRAMESIZE_1024x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			//[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			//[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		    //[FRAMESIZE_3840x2160] = REFRESHRATE_30_BIT, 
            //[FRAMESIZE_4096x2160] = REFRESHRATE_30_BIT,   
		},                         
    },
    [CN312_INPUT_COUNT] =
    {
		.name = NULL,
    },
};

static framegrabber_property_t  cn312_property={
		.name="CN312M",
		.input_setup_info=cn312_input_info,
		.support_out_pixfmt_mask=FRAMEGRABBER_PIXFMT_BITMSK,//FRAMEGRABBER_PIXFMT_YUYV_BIT
        .max_frame_size= 1920*1080, //4096 *2160
};

static v4l2_model_device_setup_t device_info=
{
    .type = DEVICE_TYPE_GRABBER,
    .driver_name="CN312M",
    .card_name="AVerMedia CN312M_",
    .capabilities= V4L2_MODEL_CAPS_CAPTURE_BIT | V4L2_MODEL_CAPS_READWRITE_BIT | V4L2_MODEL_CAPS_STREAMING_BIT,
    .buffer_type=V4L2_MODEL_BUF_TYPE_DMA_SG,
};

static void *board_v4l2_alloc(void);
static void board_v4l2_release(void *cxt);
static void cn312_v4l2_stream_on(v4l2_model_callback_parameter_t *cb_info);
static void cn312_v4l2_stream_off(v4l2_model_callback_parameter_t *cb_info);
static void cn312_stream_on(framegrabber_handle_t handle);
static void cn312_stream_off(framegrabber_handle_t handle);

static void cn312s_stream_input(framegrabber_handle_t handle, int input_stream);

static void cn312_bchs_write(framegrabber_handle_t handle);
static void cn312_brightness_read(framegrabber_handle_t handle,int *brightness);
static void cn312_contrast_read(framegrabber_handle_t handle,int *contrast);
static void cn312_hue_read(framegrabber_handle_t handle,int *hue);
static void cn312_saturation_read(framegrabber_handle_t handle,int *saturation);

static int cn312_flash_dump(framegrabber_handle_t handle,int start_block, int blocks, U8_T *flash_dump); //
static int cn312_flash_update(framegrabber_handle_t handle,int start_block, int blocks, U8_T *flash_dump); //

static framegrabber_interface_t cn312_ops={
    .stream_on      = cn312_stream_on,
    .stream_off     = cn312_stream_off,
    //.bchs_get     = cn312_bchs_read,
    .brightness_get = cn312_brightness_read,
    .contrast_get   = cn312_contrast_read,
    .hue_get        = cn312_hue_read,
    .saturation_get = cn312_saturation_read,
    .bchs_set       = cn312_bchs_write,
    .flash_read     = cn312_flash_dump,
    .flash_update   = cn312_flash_update,
    .s_input        = cn312s_stream_input,
};

static void *board_v4l2_alloc()
{
	board_v4l2_context_t *cxt;
	cxt=mem_model_alloc_buffer(sizeof(*cxt));

	return cxt;
}

static void board_v4l2_release(void *cxt)
{
	board_v4l2_context_t *board_v4l2_cxt=cxt;

	if(board_v4l2_cxt)
	{
        if(board_v4l2_cxt->check_signal_task_handle)
        {
            task_model_release_task(board_v4l2_cxt->task_model_handle,board_v4l2_cxt->check_signal_task_handle);
            board_v4l2_cxt->check_signal_task_handle=NULL;
        }
        cxt_manager_unref_context(board_v4l2_cxt->task_model_handle);
        cxt_manager_unref_context(board_v4l2_cxt->aver_xilinx_handle);
        mem_model_free_buffer(board_v4l2_cxt);
	}
}

static void cn312s_stream_input(framegrabber_handle_t handle, int input_stream)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	//cxt_mgr_handle_t cxt_mgr = get_cxt_manager_from_context(board_v4l2_cxt);
	handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 

	mesg("==========%s input=%d\n", __func__, input_stream);
	
	//framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);

	if (input_stream == CN312S_SDI_STREAM0) {
		
	    sdi_set_stream(sdi_handle, SDI_STREAM_A);	

	} else if (input_stream == CN312S_SDI_STREAM1) {
		
       sdi_set_stream(sdi_handle, SDI_STREAM_B);
       
	} 
}

static void cn312_v4l2_stream_on(v4l2_model_callback_parameter_t *cb_info)
{
    board_v4l2_context_t *board_v4l2_cxt=cb_info->asso_data;
    
    debug_msg("%s %p\n",__func__,board_v4l2_cxt);

}

static void cn312_v4l2_stream_off(v4l2_model_callback_parameter_t *cb_info)
{
    board_v4l2_context_t *board_v4l2_cxt=cb_info->asso_data;
    debug_msg("%s %p\n",__func__,board_v4l2_cxt);
    
}
static void cn312_stream_on(framegrabber_handle_t handle)
{
    board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    aver_xilinx_video_process_cfg_t vip_cfg;
    int width,height;
    int out_width,out_height;
    int frameinterval=0;
    int audio_rate=0;
    int dual_pixel=0;
    int ioctl_bypass_disable=0;
    int yuv422=0;
    framegrabber_framemode_e framemode;
    sdi_packetinfo_t *sdi_packetinfo=&board_v4l2_cxt->cur_sdi_packetinfo;  
    handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
    const framegrabber_pixfmt_t *pixfmt=framegrabber_g_out_pixelfmt(handle);
    //enum sdi_audio_sample sdi_audiorate=0;
    //sdi_frameinfo_t *sdi_frameinfo=&board_v4l2_cxt->cur_sdi_frameinfo; 
    
    mesg(">>>>%s device_index=%d\n",__func__,board_v4l2_cxt->device_index);
    
 
    framemode=framegrabber_g_input_interlace(board_v4l2_cxt->fg_handle);
    framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle,&width,&height);

    //frameinterval=framegrabber_g_input_framerate(board_v4l2_cxt->fg_handle);
    frameinterval=framegrabber_g_out_framerate(board_v4l2_cxt->fg_handle);
    if (frameinterval ==0)
    {
        frameinterval=framegrabber_g_input_framerate(board_v4l2_cxt->fg_handle);
    }
    
    framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle,&out_width,&out_height);
    mesg("%s in %dx%d out %dx%d\n",__func__,width,height,out_width,out_height);
    
    hdmi_get_packetinfo(sdi_handle,(hdmi_packetinfo_t*)sdi_packetinfo);
    
    //mesg("%s sdi_packetinfo->m_WorkingMode=%d\n",__func__,sdi_packetinfo->m_WorkingMode);          
    
    if (out_width >= 1920 && out_height >= 2160) // 4k mode could be 4k2k/2k2k
        sdi_packetinfo->m_WorkingMode_out = DEVICE_4k2k_MODE;
    else if ((out_width <= 720) && (out_height <= 576))
        sdi_packetinfo->m_WorkingMode_out = DEVICE_480_MODE;
    else
        sdi_packetinfo->m_WorkingMode_out = DEVICE_720_MODE;	
    //mesg("%s sdi_packetinfo->m_WorkingMode_out=%d\n",__func__,sdi_packetinfo->m_WorkingMode_out);    
    
    sdi_packetinfo->m_WorkingMode_in = sdi_packetinfo->m_WorkingMode; 
           
    sdi_packetinfo->m_PixelFmt_out = pixfmt->pixfmt_out; //0: YUYV 1:UYVY 8:RGB3

    mesg("%s pixfmt->pixfmt_out=%d\n",__func__,pixfmt->pixfmt_out);

    vip_cfg.workingmode = sdi_packetinfo->m_WorkingMode; 
    vip_cfg.colorspacemode = sdi_packetinfo->m_ColorSpaceMode;
    vip_cfg.samplingmode = sdi_packetinfo->m_SamplingMode;

    if(framemode==FRAMEGRABBER_FRAMEMODE_INTERLACED)
    {
        vip_cfg.is_interlace=AVER_TRUE;
    }
    else
    {
        vip_cfg.is_interlace=AVER_FALSE;
    }
	
    mesg("%s %s\n", __func__,(vip_cfg.is_interlace) ? "interlace":"progress");
    
    //sdi_get_audiorate(sdi_handle,&sdi_audiorate);
    
    //framegrabber_g_input_audiorate(board_v4l2_cxt->fg_handle);
    
    //sdi_get_yuvmode(sdi_handle,&yuv422);
    
    dual_pixel = framegrabber_g_input_dualmode(board_v4l2_cxt->fg_handle);
    
    if (dual_pixel ==1)
        vip_cfg.dual_pixel = 1;
    else
        vip_cfg.dual_pixel = 0;   
    
    audio_rate = framegrabber_g_input_audiorate(board_v4l2_cxt->fg_handle);
    
    //framegrabber_s_input_audiorate(board_v4l2_cxt->fg_handle,audio_rate);
    
    switch (audio_rate)
    {
		case SDI_AUDIO_RATE_32000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_32000;
		break;
		case SDI_AUDIO_RATE_44100:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_44100;
		break;
		case SDI_AUDIO_RATE_48000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		break;
		case SDI_AUDIO_RATE_88200:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_88200;
		break;
		case SDI_AUDIO_RATE_96000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_96000;
		break;
		case SDI_AUDIO_RATE_176400:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_176400;
		break;
		case SDI_AUDIO_RATE_192000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_192000;
		break;
		default:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_44100;
		break;
	}
    printk("%s...vip_cfg.audio_rate_index=%d\n",__func__,vip_cfg.audio_rate);
    //sdi_get_frameinfo(sdi_handle,sdi_frameinfo);
    
    //mesg("%s========sdi_frameinfo->packet_colorspace=%d\n",__func__,sdi_frameinfo->packet_colorspace);
    
    vip_cfg.in_framerate = framegrabber_g_input_framerate(board_v4l2_cxt->fg_handle);
    
    if (vip_cfg.is_interlace==AVER_TRUE)
        vip_cfg.in_framerate = vip_cfg.in_framerate *2; //same as fpga detect
    else
        vip_cfg.in_framerate = vip_cfg.in_framerate;
    
    vip_cfg.YUV422_flag = yuv422;    
    //vip_cfg.packet_colorspace = sdi_frameinfo->packet_colorspace;    
    vip_cfg.in_framesize.width=width;
    vip_cfg.in_framesize.height=height;
    vip_cfg.out_framesize.width=out_width;
    vip_cfg.out_framesize.height=out_height;
    vip_cfg.clip_start.x=0;
    vip_cfg.clip_start.y=0;
    vip_cfg.clip_size.width=width;
    vip_cfg.clip_size.height=height;

    vip_cfg.valid_mask=SCALER_CFG_VALID_MASK|FRAMERATE_CFG_VALID_MASK;
    //vip_cfg.pixel_clock=board_v4l2_cxt->cur_sdi_frameinfo.pixel_clock;
    
    vip_cfg.out_framerate=frameinterval;
    
    //mesg("%s...vip_cfg.in_framerate=%d  vip_cfg.out_framerate x 100=%d\n",__func__,vip_cfg.in_framerate,vip_cfg.out_framerate);
    
    ioctl_bypass_disable = framegrabber_g_fpga_bypassmode(board_v4l2_cxt->fg_handle);
    
    vip_cfg.ioctl_YUYV = framegrabber_g_output_pixelformat(board_v4l2_cxt->fg_handle);
    
    #ifndef CONFIG_SONY
    ioctl_bypass_disable =1;
    vip_cfg.ioctl_YUYV =1;
    #endif
	
    //vip_cfg.video_bypass = 1; //about low latency mode that is auto-detected by FPGA, enabled as in framerate = out framerate

    vip_cfg.pixel_format_out = pixfmt->pixfmt_out;
#if 0
    switch(/*pixfmt->pixfmt_out*/0) //unused for CN312S
    {
        case AVER_XILINX_FMT_YUYV:
        case AVER_XILINX_FMT_UYVY:
        case AVER_XILINX_FMT_YVYU:
        case AVER_XILINX_FMT_VYUY:
            vip_cfg.pixel_format_out=AVER_XILINX_FMT_YUYV;
            mesg("%s...vip_cfg.pixel_format_output=AVER_XILINX_FMT_YUYV\n",__func__);
            break;
        case AVER_XILINX_FMT_RGBP: //RGB565
        case AVER_XILINX_FMT_RGBR: //RGB565X
        case AVER_XILINX_FMT_RGBO:
        case AVER_XILINX_FMT_RGBQ:
        case AVER_XILINX_FMT_RGB3: //RGB24 4:4:4
        case AVER_XILINX_FMT_BGR3: 
        case AVER_XILINX_FMT_RGB4:
        case AVER_XILINX_FMT_BGR4:
            vip_cfg.pixel_format_out=AVER_XILINX_FMT_BGR3;
            mesg("%s...vip_cfg.pixel_format=AVER_XILINX_FMT_RGB3\n",__func__);
            break;
        default:
            vip_cfg.pixel_format_out=AVER_XILINX_FMT_YUYV;
            break;
    }  
#endif

	vip_cfg.pixel_format_out = pixfmt->pixfmt_out;
    aver_xilinx_config_video_process(board_v4l2_cxt->aver_xilinx_handle, &vip_cfg,board_v4l2_cxt->device_index);

    aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_TRUE,board_v4l2_cxt->device_index);
    if(board_v4l2_cxt->device_index == 0)
        hdmi_set_color_space(sdi_handle,(hdmi_packetinfo_t*)sdi_packetinfo);
    #if 0
    else
        sdi_set_color_space(sdi_handle,sdi_packetinfo);
    #endif
    //sdi_set_freerun_screen(sdi_handle,AVER_FALSE);    
}

static void cn312_stream_off(framegrabber_handle_t handle)
{
    board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    //handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[CN312_I2C_DEVICE_0]; 
	
    mesg(">>>>>>>>>>>%s device_index=%d\n",__func__,board_v4l2_cxt->device_index);
    //sdi_set_freerun_screen(sdi_handle,AVER_TRUE);
	
    aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_FALSE,board_v4l2_cxt->device_index);
}

static int cn312_flash_dump(framegrabber_handle_t handle,int start_block, int blocks, U8_T *flash_dump) //
{
    board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    //char version[10];
    int ret=0;
		
    ret = aver_xilinx_flash_dump(board_v4l2_cxt->aver_xilinx_handle,start_block,blocks,flash_dump);
    
    return ret;
}

static int cn312_flash_update(framegrabber_handle_t handle,int start_block, int blocks, U8_T *flash_update)
{
    board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    //char version[10];
    int ret=0;
	
    ret = aver_xilinx_flash_update(board_v4l2_cxt->aver_xilinx_handle,start_block,blocks,flash_update);
    
    return ret;
}

static void cn312_brightness_read(framegrabber_handle_t handle,int *brightness)
{
	board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);//
    //handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
   
	//sdi_get_brightness(sdi_handle,&board_v4l2_cxt->cur_bchs_value); 
	
	framegrabber_set_data(handle,board_v4l2_cxt);
	brightness = &board_v4l2_cxt->cur_bchs_value;
	handle->fg_bchs_value = *brightness;
	//mesg("%s get brightness=%x\n",__func__,board_v4l2_cxt->cur_bchs_value);
}
static void cn312_contrast_read(framegrabber_handle_t handle,int *contrast)
{
	board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    //handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
   
	//sdi_get_contrast(sdi_handle,&board_v4l2_cxt->cur_bchs_value); 
	
	framegrabber_set_data(handle,board_v4l2_cxt);
	contrast = &board_v4l2_cxt->cur_bchs_value;
	handle->fg_bchs_value = *contrast;
	//mesg("%s get contrast=%x\n",__func__,board_v4l2_cxt->cur_bchs_value);
}

static void cn312_hue_read(framegrabber_handle_t handle,int *hue)
{
	board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    //handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
   
	//sdi_get_hue(sdi_handle,&board_v4l2_cxt->cur_bchs_value); 
	
	framegrabber_set_data(handle,board_v4l2_cxt);
	hue = &board_v4l2_cxt->cur_bchs_value;
	handle->fg_bchs_value = *hue;
	//mesg("%s get hue=%x\n",__func__,board_v4l2_cxt->cur_bchs_value);
}

static void cn312_saturation_read(framegrabber_handle_t handle,int *saturation)
{
	board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
    //handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
   
	//sdi_get_saturation(sdi_handle,&board_v4l2_cxt->cur_bchs_value); 
	
	framegrabber_set_data(handle,board_v4l2_cxt);
	saturation = &board_v4l2_cxt->cur_bchs_value;
	handle->fg_bchs_value = *saturation;
	//mesg("%s get saturation=%x\n",__func__,board_v4l2_cxt->cur_bchs_value);
}

static void cn312_bchs_write(framegrabber_handle_t handle)
{
	//board_v4l2_context_t *board_v4l2_cxt=framegrabber_get_data(handle);
	//handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
   
	//int bchs_value = handle->fg_bchs_value;
	//int bchs_select = handle->fg_bchs_selection;	
		
	//mesg("%s...bchs_select=%d\n",__func__,bchs_select);
		
	{
		//framegrabber_s_input_bchs(board_v4l2_cxt->fg_handle,board_v4l2_cxt->cur_bchs_value,board_v4l2_cxt->cur_bchs_selection);
		//sdi_set_bchs(sdi_handle,&bchs_value,&bchs_select); 
		//if (bchs_select ==0) mesg("set brightness=%d\n",bchs_value);
		//if (bchs_select ==1) mesg("set contrast=%d\n",bchs_value);
		//if (bchs_select ==2) mesg("set hue=%d\n",bchs_value);
		//if (bchs_select ==3) mesg("set saturation=%d\n",bchs_value);
	}
}

static void cn312_video_buffer_done(void *data)
{
    board_v4l2_context_t *board_v4l2_cxt=data;
    //mesg("%s board_v4l2_cxt %p\n",__func__,board_v4l2_cxt);
    if(board_v4l2_cxt)
    {
        v4l2_model_buffer_done(board_v4l2_cxt->v4l2_handle);
    }
}

static void cn312_v4l2_buffer_prepare(v4l2_model_callback_parameter_t *cb_info)
{
    board_v4l2_context_t *board_v4l2_cxt=cb_info->asso_data;
    v4l2_model_buffer_info_t *buffer_info=cb_info->u.buffer_prepare_info.buffer_info;
    //mesg("%s %p buffer_info %p\n",__func__,board_v4l2_cxt,buffer_info);
    if(buffer_info)
    {
        int i;
        v4l2_model_buf_desc_t *desc;
        unsigned framebufsize,remain, s2;
        int width,height;
        unsigned bytesperline;
        
        framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle,&width,&height);
        bytesperline=framegrabber_g_out_bytesperline(board_v4l2_cxt->fg_handle);
        s2 = 0;
        framebufsize=bytesperline*height; 
        for(i=0,desc=buffer_info->buf_info[0],remain=(framebufsize - s2);i<buffer_info->buf_count[0];i++)
        {
            if(remain >= desc[i].size)
            {
                aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle,desc[i].addr,desc[i].size,board_v4l2_cxt->device_index);
                remain -= desc[i].size;
            }else
            {
                if (remain !=0) // in order to avoid video stop as size 0 set to FPGA //sync sony project
                {
                    aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle,desc[i].addr,remain,board_v4l2_cxt->device_index);
                    remain =0;
                }
                if((s2 == 0) && (remain == 0))
                    break;
            }

            //mesg("addr %08x size %x\n",desc[i].addr,desc[i].size);
        }
        aver_xilinx_active_current_desclist(board_v4l2_cxt->aver_xilinx_handle,cn312_video_buffer_done,board_v4l2_cxt,board_v4l2_cxt->device_index);   
    }  
}

static void cn312_v4l2_buffer_init(v4l2_model_callback_parameter_t *cb_info)
{
//    board_v4l2_context_t *board_v4l2_cxt=cb_info->asso_data;
    v4l2_model_buffer_info_t *buffer_info=cb_info->u.buffer_prepare_info.buffer_info;
    //mesg("%s %p buffer_info %p\n",__func__,board_v4l2_cxt,buffer_info);
    if(buffer_info)
    {
        int i;
        v4l2_model_buf_desc_t *desc;
        //mesg("buf type %d count %d\n",buffer_info->buf_type,buffer_info->buf_count[0]);
        for(i=0,desc=buffer_info->buf_info[0];i<buffer_info->buf_count[0];i++)
        {
            //mesg("addr %08x size %x\n",desc[i].addr,desc[i].size);
        } 
    }  
}

static void check_signal_stable_task(void *data)
{
    board_v4l2_context_t *board_v4l2_cxt=data;
    //sdi_frameinfo_t *sdi_frameinfo=&board_v4l2_cxt->cur_sdi_frameinfo; 
    //handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; //check 20170511  
    int width,height;
    int dual_pixel;
    int dual_pixel_set;
    framegrabber_framemode_e framemode;
    aver_xilinx_frame_info_t detected_frameinfo;
    sdi_frameinfo_t *sdi_frameinfo=&board_v4l2_cxt->cur_sdi_frameinfo; 
    BOOL_T is_interlace;
    int stable_check;
    
    mesg("%s\n",__func__);
    
    //sdi_get_frameinfo(sdi_handle,sdi_frameinfo);
    //mesg("%s sdi detected size %dx%d;framerate %d \n",__func__,sdi_frameinfo->width,sdi_frameinfo->height,sdi_frameinfo->framerate);
        
    aver_xilinx_get_frameinfo(board_v4l2_cxt->aver_xilinx_handle,&detected_frameinfo,&stable_check,board_v4l2_cxt->device_index); 
    framemode=framegrabber_g_input_framemode(board_v4l2_cxt->fg_handle);
    is_interlace=(framemode==FRAMEGRABBER_FRAMEMODE_INTERLACED) ? AVER_TRUE : AVER_FALSE;
    framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle,&width,&height);
    dual_pixel = framegrabber_g_input_dualmode(board_v4l2_cxt->fg_handle);
    
    if (dual_pixel ==1)
        dual_pixel_set = detected_frameinfo.width*2;
    else
        dual_pixel_set = detected_frameinfo.width; 
   
    mesg("%s stable_check=%d\n",__func__,stable_check);

    if ((stable_check == 0) && (cnt_retry<6))
    {
		cnt_retry++;
		task_model_run_task_after(board_v4l2_cxt->task_model_handle,board_v4l2_cxt->check_signal_task_handle,50000);
        return;
	} 
    printk("%s fpga detected size %dx%d, is_interlace = %d, sdi_frameinfo->is_interlace = %d\n",__func__,detected_frameinfo.width,detected_frameinfo.height, is_interlace, sdi_frameinfo->is_interlace);

 
    if (sdi_frameinfo->is_interlace)
    {
		detected_frameinfo.width = sdi_frameinfo->width;
		detected_frameinfo.height = sdi_frameinfo->height/2;
	}
	else
	{
		detected_frameinfo.width = sdi_frameinfo->width;
		detected_frameinfo.height = sdi_frameinfo->height;
	}
	
	framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,detected_frameinfo.width,detected_frameinfo.height); //rr
      
#if 0      
    if(width==detected_frameinfo.width && height==detected_frameinfo.height && (is_interlace==detected_frameinfo.is_interlace))
    {
		mesg("%s fix input %dx%d to fpga %dx%d\n",__func__,width,height,detected_frameinfo.width,detected_frameinfo.height);
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
        cnt_retry=0;
    }else
    {
        mesg("%s input %dx%d match to fpga %dx%d\n",__func__,width,height,detected_frameinfo.width,detected_frameinfo.height);
      
        width=detected_frameinfo.width;
        height=detected_frameinfo.height;
        
        framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,width,height);
        if(is_interlace!=detected_frameinfo.is_interlace)
        {
            framemode=(detected_frameinfo.is_interlace) ? FRAMEGRABBER_FRAMEMODE_INTERLACED:FRAMEGRABBER_FRAMEMODE_PROGRESS;
            framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle,framemode);
        }
        task_model_run_task_after(board_v4l2_cxt->task_model_handle,board_v4l2_cxt->check_signal_task_handle,50000); // run again to comfirm
    }
#else
	printk("%s fix input %dx%d to fpga %dx%d\n",__func__,width,height,detected_frameinfo.width,detected_frameinfo.height);
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,width,height);
    //framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);    
    framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
    cnt_retry=0;

#endif       
}

static void cn312_sdi_event(void *cxt,sdi_event_e event) 
{
    board_v4l2_context_t *board_v4l2_cxt=cxt;
    sdi_frameinfo_t *sdi_frameinfo=&board_v4l2_cxt->cur_sdi_frameinfo;
    sdi_packetinfo_t *sdi_packetinfo=&board_v4l2_cxt->cur_sdi_packetinfo;  
    enum sdi_audio_sample sdi_audiorate=board_v4l2_cxt->cur_sdi_audiorate; 
    handle_t sdi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
    //sdi_out_format_e out_format;
    //unsigned int hdcp_flag;
    //int ITU_flag;
    aver_xilinx_frame_info_t detected_frameinfo;
    int stable_check;
    //const framegrabber_pixfmt_t *pixfmt=framegrabber_g_out_pixelfmt(handle);
    
    printk("%s device_index %d\n",__func__,board_v4l2_cxt->device_index);
    
    if (board_v4l2_cxt->device_index ==1)
    {
    
    switch(event)
    {
    case SDI_LOCK:
    {
        printk("Device%d SDI_LOCK\n",board_v4l2_cxt->device_index);	
        
        {
            aver_xilinx_get_frameinfo(board_v4l2_cxt->aver_xilinx_handle,&detected_frameinfo,&stable_check,board_v4l2_cxt->device_index); 
            printk("%s fpga detect %dx%d\n",__func__,detected_frameinfo.width,detected_frameinfo.height);
			
            sdi_get_frameinfo(sdi_handle,sdi_frameinfo);
            
            //aver_xilinx_hdmi_hotplug(board_v4l2_cxt->fg_handle);
        
            //mesg("%s locked fe %dx%d%c\n",__func__,sdi_frameinfo->width,sdi_frameinfo->height,(sdi_frameinfo->is_interlace)?'i':'p'/*,sdi_frameinfo->framerate*/);
            
            sdi_get_packetinfo(sdi_handle,sdi_packetinfo);
          
            //sdi_packetinfo->m_PixelFmt_out = pixfmt->pixfmt_out; //0: yuv 1:uyvy 8:RGB3
            
            //sdi_set_color_space(sdi_handle,sdi_packetinfo);
            sdi_audiorate = SDI_AUDIO_RATE_48000; //SDI standard
            framegrabber_s_input_audiorate(board_v4l2_cxt->fg_handle,sdi_audiorate);
            
            //mesg("sdi_frameinfo->denominator = %d\n",sdi_frameinfo->denominator);
            
            framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,sdi_frameinfo->framerate,sdi_frameinfo->denominator);
            
            framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,sdi_frameinfo->is_interlace);
            
            framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,sdi_frameinfo->width,sdi_frameinfo->height); //tt 0602
            
            //framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle,framemode);
            
            //framegrabber_s_input_dualmode(board_v4l2_cxt->fg_handle,sdi_frameinfo->dual_pixel); //tt 1003
            
            //framegrabber_s_hdcp_flag(board_v4l2_cxt->fg_handle,hdcp_flag);
            #if 0
            if ((sdi_frameinfo->is_ITU601==1) && (sdi_frameinfo->is_ITU709==0))
                ITU_flag =1;
            else if ((sdi_frameinfo->is_ITU601==0) && (sdi_frameinfo->is_ITU709==1))
                ITU_flag =2;
            else  
                ITU_flag =0;  
            
            framegrabber_s_input_ITU(board_v4l2_cxt->fg_handle,ITU_flag);
                    
            #endif
            
            //aver_xilinx_dual_pixel(board_v4l2_cxt->aver_xilinx_handle,sdi_frameinfo->dual_pixel);
            #if 0
            if((sdi_frameinfo->pixel_clock>170000000/*150000*/) || (sdi_frameinfo->dual_pixel_like ==1))//rr1012
            {
                //sdi_out_format_e out_format;
                if((sdi_frameinfo->is_yuv422==1) && (sdi_frameinfo->is_yuv444==0))
                    out_format=SDI_OUT_FORMAT_SDR_422_2X24_INTERLEAVE_MODE0;
                else if((sdi_frameinfo->is_yuv444==1) && (sdi_frameinfo->is_yuv422==0))
                    out_format=SDI_OUT_FORMAT_SDR_444_2X24_INTERLEAVE_MODE0;
                else
                    out_format=SDI_OUT_FORMAT_SDR_422_2X24_INTERLEAVE_MODE0;   
                sdi_set_out_format(sdi_handle,out_format);
                //sdi_set_out_format(board_v4l2_cxt->i2c_device_handle[CN312_I2C_CHIP_SDI_1],out_format);
            }
            else
            {
				if((sdi_frameinfo->is_yuv422==1) && (sdi_frameinfo->is_yuv444==0))
                    out_format=SDI_OUT_FORMAT_SDR_ITU656_24_MODE0;
                else if((sdi_frameinfo->is_yuv444==1) && (sdi_frameinfo->is_yuv422==0))
                    out_format=SDI_OUT_FORMAT_SDR_444_36_MODE0;
                else
                    out_format=SDI_OUT_FORMAT_SDR_ITU656_24_MODE0;   
                sdi_set_out_format(sdi_handle,out_format);
            }
            //mesg("=========== sdi_set_out_format=%02x\n",out_format);
            #endif
        }
        
        framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle,sdi_frameinfo->is_interlace);
        
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
        
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,
					FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
        
        //sdi_set_audio_out_enable(sdi_handle,AVER_TRUE,board_v4l2_cxt->device_index);
        //task_model_run_task_after(board_v4l2_cxt->task_model_handle,board_v4l2_cxt->check_signal_task_handle,100000);
    }
        break;
    case SDI_UNLOCK:
        printk("SDI_UNLOCK\n");
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
        framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,0);
        break;
    #if 0    
    case SDI_UNLOCK_ENHANCE:
        mesg("SDI_UNLOCK_ENHANCE\n");
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL);
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
        //framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,0);
        break;  
        
    case SDI_FRAME_CHANGE:
        mesg(">>>SDI_FRAME_CHANGE\n");
        //framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL);
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); //tt 0615
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,0);
        break;
     #endif         
     default:
		break;
    }
     
    }
    if (board_v4l2_cxt->device_index ==0)
    {
		printk("%s device%d event\n",__func__,board_v4l2_cxt->device_index);
	}
    
}

static void cn312_hdmi_event(void *cxt,hdmi_event_e event) 
{
    board_v4l2_context_t *board_v4l2_cxt=cxt;
    hdmi_frameinfo_t *hdmi_frameinfo=&board_v4l2_cxt->cur_hdmi_frameinfo;
    hdmi_packetinfo_t *hdmi_packetinfo=&board_v4l2_cxt->cur_hdmi_packetinfo;  
    enum hdmi_audio_sample hdmi_audiorate=board_v4l2_cxt->cur_hdmi_audiorate; 
    handle_t hdmi_handle=board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index]; 
    //hdmi_out_format_e out_format;
    //unsigned int hdcp_flag;
    //int ITU_flag;
    aver_xilinx_frame_info_t detected_frameinfo;
    int stable_check;
    //const framegrabber_pixfmt_t *pixfmt=framegrabber_g_out_pixelfmt(handle);
    
    mesg("%s event %d\n",__func__,event);
    
    switch(event)
    {
    case HDMI_LOCK:
    {
        printk("CH[%d] HDMI_LOCK\n",board_v4l2_cxt->device_index);	
        
        {
            aver_xilinx_get_frameinfo(board_v4l2_cxt->aver_xilinx_handle,&detected_frameinfo,&stable_check,board_v4l2_cxt->device_index); 

            if(board_v4l2_cxt->device_index == 0)
            {
                hdmi_get_frameinfo(hdmi_handle,hdmi_frameinfo);
                hdmi_get_packetinfo(hdmi_handle,hdmi_packetinfo);
                hdmi_audiorate = hdmi_get_audio_sample(hdmi_handle); //SDI standard
                framegrabber_s_input_audiorate(board_v4l2_cxt->fg_handle,hdmi_audiorate);
                framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,hdmi_frameinfo->framerate,hdmi_frameinfo->denominator);
                framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,hdmi_frameinfo->is_interlace);
                framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,hdmi_frameinfo->width,hdmi_frameinfo->height); //tt 0602
            }else{
                hdmi_get_frameinfo(hdmi_handle,hdmi_frameinfo);
                hdmi_get_packetinfo(hdmi_handle,hdmi_packetinfo);
                hdmi_audiorate = hdmi_AUDIO_RATE_48000; //SDI standard
                framegrabber_s_input_audiorate(board_v4l2_cxt->fg_handle,hdmi_audiorate);
                framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,hdmi_frameinfo->framerate,hdmi_frameinfo->denominator);
                framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,hdmi_frameinfo->is_interlace);
                framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,hdmi_frameinfo->width,hdmi_frameinfo->height); //tt 0602
            }
            
            //aver_xilinx_hdmi_hotplug(board_v4l2_cxt->fg_handle);
            //mesg("%s locked fe %dx%d%c\n",__func__,hdmi_frameinfo->width,hdmi_frameinfo->height,(hdmi_frameinfo->is_interlace)?'i':'p'/*,hdmi_frameinfo->framerate*/);
          
            //hdmi_packetinfo->m_PixelFmt_out = pixfmt->pixfmt_out; //0: yuv 1:uyvy 8:RGB3
            
            //framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle,framemode);
            
            //framegrabber_s_input_dualmode(board_v4l2_cxt->fg_handle,hdmi_frameinfo->dual_pixel); //tt 1003
            
            //framegrabber_s_hdcp_flag(board_v4l2_cxt->fg_handle,hdcp_flag);
            #if 0
            if ((hdmi_frameinfo->is_ITU601==1) && (hdmi_frameinfo->is_ITU709==0))
                ITU_flag =1;
            else if ((hdmi_frameinfo->is_ITU601==0) && (hdmi_frameinfo->is_ITU709==1))
                ITU_flag =2;
            else  
                ITU_flag =0;  
            
            framegrabber_s_input_ITU(board_v4l2_cxt->fg_handle,ITU_flag);
                    
            #endif
            
            //aver_xilinx_dual_pixel(board_v4l2_cxt->aver_xilinx_handle,hdmi_frameinfo->dual_pixel);
            #if 0
            if((hdmi_frameinfo->pixel_clock>170000000/*150000*/) || (hdmi_frameinfo->dual_pixel_like ==1))//rr1012
            {
                //hdmi_out_format_e out_format;
                if((hdmi_frameinfo->is_yuv422==1) && (hdmi_frameinfo->is_yuv444==0))
                    out_format=SDI_OUT_FORMAT_SDR_422_2X24_INTERLEAVE_MODE0;
                else if((hdmi_frameinfo->is_yuv444==1) && (hdmi_frameinfo->is_yuv422==0))
                    out_format=SDI_OUT_FORMAT_SDR_444_2X24_INTERLEAVE_MODE0;
                else
                    out_format=SDI_OUT_FORMAT_SDR_422_2X24_INTERLEAVE_MODE0;   
                hdmi_set_out_format(hdmi_handle,out_format);
                //hdmi_set_out_format(board_v4l2_cxt->i2c_device_handle[CN312_I2C_CHIP_SDI_1],out_format);
            }
            else
            {
				if((hdmi_frameinfo->is_yuv422==1) && (hdmi_frameinfo->is_yuv444==0))
                    out_format=SDI_OUT_FORMAT_SDR_ITU656_24_MODE0;
                else if((hdmi_frameinfo->is_yuv444==1) && (hdmi_frameinfo->is_yuv422==0))
                    out_format=SDI_OUT_FORMAT_SDR_444_36_MODE0;
                else
                    out_format=SDI_OUT_FORMAT_SDR_ITU656_24_MODE0;   
                hdmi_set_out_format(hdmi_handle,out_format);
            }
            //mesg("=========== hdmi_set_out_format=%02x\n",out_format);
            #endif
        }
        
        framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle,hdmi_frameinfo->is_interlace);
        
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
        
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,
					FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
        
        //hdmi_set_audio_out_enable(hdmi_handle,AVER_TRUE,board_v4l2_cxt->device_index);
        //task_model_run_task_after(board_v4l2_cxt->task_model_handle,board_v4l2_cxt->check_signal_task_handle,100000);
    }
        break;
    case HDMI_UNLOCK:
        printk("HDMI_UNLOCK\n");
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
        framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,0);
        //hdmi_set_audio_out_enable(hdmi_handle,AVER_FALSE,board_v4l2_cxt->device_index);
        break;
    case HDMI_AUDIO_CHG:
        printk("Audio change \n");
        hdmi_audiorate = hdmi_get_audio_sample(hdmi_handle);
        framegrabber_s_input_audiorate(board_v4l2_cxt->fg_handle,hdmi_audiorate);
    #if 0    
    case HDMI_UNLOCK_ENHANCE:
        mesg("HDMI_UNLOCK_ENHANCE\n");
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL);
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
        //framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,0);
        break;  
        
    case HDMI_FRAME_CHANGE:
        mesg(">>>HDMI_FRAME_CHANGE\n");
        //framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL);
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); //tt 0615
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_INPUT_STATUS_OK);
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,0);
        break;
     #endif         
     default:
		break;
    } 
}

#if 0
static void cn312_adv7619_get_cardid(void *cxt, U32_T* cardid) 
{
	board_v4l2_context_t *board_v4l2_cxt = (board_v4l2_context_t*)cxt;

	if((board_v4l2_cxt == NULL) || (cardid == NULL))
	{
		return;
	}
	
	aver_xilinx_get_cardid(board_v4l2_cxt->aver_xilinx_handle, cardid);
}
#endif

void board_v4l2_init(cxt_mgr_handle_t cxt_mgr,int device_index, unsigned long card_id)
{
    framegrabber_handle_t framegrabber_handle;
    board_v4l2_context_t *board_v4l2_cxt=NULL;
    v4l2_model_handle_t v4l2_handle;
    task_model_handle_t task_model_handle=NULL;
    handle_t i2c_mgr=NULL;
    handle_t aver_xilinx_handle=NULL;
    task_handle_t task_handle=NULL;
    int i;
    
    enum
    {
        BOARD_V4L2_OK = 0,
        BOARD_V4L2_ERROR_CXT_MGR,
        BOARD_V4L2_ERROR_FRAMEGRABBER_INIT,
        BOARD_V4L2_ERROR_V4L2_MODEL_INIT,
        BOARD_V4L2_ERROR_ALLOC_CXT,
        BOARD_V4L2_ERROR_GET_MAIN_CHIP_HANDLE,
        BOARD_V4L2_ERROR_GET_I2C_MGR,
        BOARD_V4L2_ERROR_GET_TASK_MGR,
        BOARD_V4L2_ERROR_ALLOC_TASK,
    } err = BOARD_V4L2_OK;

    do
    {
        if (!cxt_mgr)
        {
            err = BOARD_V4L2_ERROR_CXT_MGR;
            break;
        }

        framegrabber_handle = framegrabber_init(cxt_mgr, &cn312_property, &cn312_ops);
        if (framegrabber_handle == NULL)
        {
            err = BOARD_V4L2_ERROR_FRAMEGRABBER_INIT;
            break;
        }
        board_v4l2_cxt = cxt_manager_add_cxt(cxt_mgr, BOARD_V4L2_CXT_ID, board_v4l2_alloc, board_v4l2_release);
        if (!board_v4l2_cxt)
        {
            err = BOARD_V4L2_ERROR_ALLOC_CXT;
            break;
        }
        board_v4l2_cxt->fg_handle = framegrabber_handle;
        framegrabber_set_data(framegrabber_handle,board_v4l2_cxt);
        
        v4l2_handle = v4l2_model_init(cxt_mgr, &device_info,framegrabber_handle,card_id); //////////sony
        if(v4l2_handle==NULL)
	    {
            err=BOARD_V4L2_ERROR_V4L2_MODEL_INIT;
		    break;
	    }
	    board_v4l2_cxt->v4l2_handle=v4l2_handle;
        aver_xilinx_handle=cxt_manager_get_context(cxt_mgr,AVER_XILINX_CXT_ID,0);
	    if(aver_xilinx_handle==NULL)
	    {
            err=BOARD_V4L2_ERROR_GET_MAIN_CHIP_HANDLE;
            break;
	    }
            
        board_v4l2_cxt->aver_xilinx_handle=aver_xilinx_handle;
        i2c_mgr=cxt_manager_get_context(cxt_mgr,I2C_CXT_ID,0);
	    if(i2c_mgr==NULL)
	    {
            err=BOARD_V4L2_ERROR_GET_I2C_MGR;
            break;
	    }
	    //i = device_index;
        for(i=0;i<CN312_I2C_DEVICE_COUNT;i++)
	    {
            board_v4l2_cxt->i2c_device_handle[i]=i2c_model_get_nth_driver_handle(i2c_mgr,cn312_device_desc[i].name,cn312_device_desc[i].index);
            mesg("...i2c_device_handle[%d] %p\n",i,board_v4l2_cxt->i2c_device_handle[i]);
	    }
	    
	    board_v4l2_cxt->device_index = device_index;
	    
	    board_v4l2_cxt->card_id = card_id;
	    
        task_model_handle=cxt_manager_get_context(cxt_mgr,TASK_MODEL_CXT_ID,0);
        if(task_model_handle==NULL)
        {
            err=BOARD_V4L2_ERROR_GET_TASK_MGR;
            break;
        }
        board_v4l2_cxt->task_model_handle=task_model_handle;
        cxt_manager_ref_context(task_model_handle);
        
        task_handle=task_model_create_task(task_model_handle,check_signal_stable_task,board_v4l2_cxt,"check_signal_task");
        if(task_handle==NULL)
        {
            err=BOARD_V4L2_ERROR_ALLOC_TASK;
            break;
        }
        
        board_v4l2_cxt->check_signal_task_handle=task_handle;
        
        v4l2_model_register_callback(v4l2_handle,V4L2_MODEL_CALLBACK_STREAMON ,&cn312_v4l2_stream_on, board_v4l2_cxt);
        v4l2_model_register_callback(v4l2_handle,V4L2_MODEL_CALLBACK_STREAMOFF,&cn312_v4l2_stream_off, board_v4l2_cxt);
        v4l2_model_register_callback(v4l2_handle,V4L2_MODEL_CALLBACK_BUFFER_PREPARE,&cn312_v4l2_buffer_prepare, board_v4l2_cxt);
        v4l2_model_register_callback(v4l2_handle,V4L2_MODEL_CALLBACK_BUFFER_INIT,&cn312_v4l2_buffer_init, board_v4l2_cxt);
        if (board_v4l2_cxt->device_index ==1)
            sdi_register_callback(board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index/*1*/],cn312_sdi_event,board_v4l2_cxt);
            
        if (board_v4l2_cxt->device_index ==0)    
            hdmi_register_callback(board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index/*0*/],cn312_hdmi_event,board_v4l2_cxt);
        //sdi_register_callback_for_getcardid(board_v4l2_cxt->i2c_device_handle[board_v4l2_cxt->device_index],cn312_sdi_get_cardid, board_v4l2_cxt);
        framegrabber_start(framegrabber_handle);
        cxt_manager_ref_context(aver_xilinx_handle);
        
        framegrabber_s_card_id(board_v4l2_cxt->fg_handle,board_v4l2_cxt->card_id);
       
        //sdi_get_board_id(board_v4l2_cxt->i2c_device_handle[CN312_I2C_CHIP_SDI_0],&board_v4l2_cxt->board_id);    
    }
    while (0);
    if (err != BOARD_V4L2_OK)
    {
        switch (err)
        {
        case BOARD_V4L2_ERROR_ALLOC_TASK:
            cxt_manager_unref_context(task_model_handle);
        case BOARD_V4L2_ERROR_GET_TASK_MGR:
        case BOARD_V4L2_ERROR_GET_I2C_MGR:
        case BOARD_V4L2_ERROR_V4L2_MODEL_INIT:
        case BOARD_V4L2_ERROR_ALLOC_CXT:
        case BOARD_V4L2_ERROR_FRAMEGRABBER_INIT:
        case BOARD_V4L2_ERROR_CXT_MGR:
        default:
            break;
        }
        debug_msg("%s err %d\n", __func__, err);
    }
}

void board_v4l2_suspend(cxt_mgr_handle_t cxt_mgr)
{
    board_v4l2_context_t *board_v4l2_cxt;
    handle_t cn312m_handle;
    int i;

    board_v4l2_cxt = cxt_manager_get_context(cxt_mgr,BOARD_V4L2_CXT_ID,0);
    if (!board_v4l2_cxt)
    {
        debug_msg("Error: cannot get board_v4l2_cxt");
        return;
    }
    
    for(i=0;i<CN312_I2C_DEVICE_COUNT;i++)
    {
        cn312m_handle=board_v4l2_cxt->i2c_device_handle[i];
        if (!cn312m_handle)
        {
            mesg("Error: cannot get sdi_handle");
            return;
        }
        mesg("%s(%d) %p\n",__func__,i,cn312m_handle);
        if (i==0) 
            hdmi_power_off(cn312m_handle);
        if (i==1) 
            sdi_power_off(cn312m_handle);    
	} 
    
}

void board_v4l2_resume(cxt_mgr_handle_t cxt_mgr)
{
    board_v4l2_context_t *board_v4l2_cxt;
    handle_t cn312m_handle;
    int i;

    board_v4l2_cxt = cxt_manager_get_context(cxt_mgr,BOARD_V4L2_CXT_ID,0);
    if (!board_v4l2_cxt)
    {
        debug_msg("Error: cannot get board_v4l2_cxt");
        return;
    }

    for(i=0;i<CN312_I2C_DEVICE_COUNT;i++)
    {
        cn312m_handle=board_v4l2_cxt->i2c_device_handle[i];
        if (!cn312m_handle)
        {
            mesg("Error: cannot get sdi_handle");
            return;
        }
        mesg("%s(%d) %p\n",__func__,i,cn312m_handle);
        
        if (i==0) 
            hdmi_power_on(cn312m_handle);
        if (i==1) 
            sdi_power_on(cn312m_handle);   
	} 

}

