/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * board_config.c
 *
 *  Created on: Apr 15, 2017
 *      Author: 
 *      Version:
 * =================================================================
 */
 
//#include <linux/kernel.h>
//#include <linux/module.h>
#include "board.h"
#include "cxt_mgr.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "mem_model.h"
#include "gpio_model.h"
#include "task_model.h"
#include "trace.h"
#include "debug.h"
#include "aver_xilinx.h"
#include "board_i2c.h"
#include "board_gpio.h"
#include "board_alsa.h"
#include "adv7619.h"
#include "board_v4l2.h"

//#include "pic_bmp.h"

static char *no_signal_pic = NULL;

int board_init(void);
void board_exit(void);

const char *BOARD_NAME="CX311H";
//pci_model_driver_setup_t pci_setup;
extern int subsystem_id;
//extern U32_T g_card_id;
//static unsigned int board_enable_notification_image = 0;


pci_model_id_t id_table[]={
	{
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3117,
      .driver_data=CL312H1,        
    },
    {
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3115,
      .driver_data=CL314H1,        
    },
    {
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3110,
      .driver_data=CM311H,        
    },
    {
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3112,
      .driver_data=CM311H,        
    },
    {
      .vendor=0x1461,
      .device=0x0054, //according to hw DEVICE ID config
      .sub_vendor = 0x1461,
	  .sub_device = 0x3116,
      .driver_data=CM311H,        
    },
    {
      0
    }
    
};

static aver_xilinx_cfg_t aver_xilinx_cfg=
{
    .gpio_mask = Bit(1),//Bit(0) | Bit(2), //reserved for test GPIO W/R
    .gpio_value = Bit(0),//Bit(0) ,        
    .i2c_bus_speed={
        [AVER_XILINX_I2C_BUS_0]=400*1000,
   
    },
    .audio_buffer_size=12*1024,//12*1024,
    
};

int board_probe(struct device *dev,unsigned long driver_info)
{
    cxt_mgr_handle_t cxt_mgr=get_cxt_manager(dev);
    handle_t pci_handle=NULL;
    handle_t aver_xilinx_handle=NULL;
    handle_t trace_handle=NULL;
    handle_t i2c_mgr=NULL;
    handle_t gpio_mgr=NULL;
    handle_t adv7619_handle_1=NULL; 
    handle_t task_handle=NULL;
    handle_t mem_handle=NULL;
    int i;
    int ret;
    int card_id;
    
    enum
    {        
	    NO_ERROR=0,
        NO_PCI_HANDLE,
        ERROR_TRACE_HANDLE,
        ERROR_I2C_MGR,
        ERROR_GPIO_MGR,
        ERROR_TASK_HANDLE,
        ERROR_MEM_HANDLE,
        ERROR_AVER_XILINX,
        ERROR_BOARD_I2C_INIT,
        ERROR_BOARD_GPIO_INIT,
    }err=NO_ERROR;
    
        
    debug_msg("===board_probe start \n");
    do
    {
        pci_handle=pci_model_get_handle(cxt_mgr);
        if(!pci_handle)
        {
			debug_msg("pci_handle err\n");
            err=NO_PCI_HANDLE;
            break;
        } 
	    i2c_mgr=i2c_model_init(cxt_mgr);
	    if(!i2c_mgr)
	    {
		    debug_msg("i2c_model_init err\n");
            err=ERROR_I2C_MGR;
            break;
	    }
        gpio_mgr=gpio_model_init(cxt_mgr);
        if(!gpio_mgr)
        {
			debug_msg("gpio_model_init err\n");
            err=ERROR_GPIO_MGR;
            break;
        }
        mem_handle=mem_model_init(cxt_mgr);
        if(!mem_handle)
        {
			debug_msg("mem_model_init err\n");
            err=ERROR_MEM_HANDLE;
            break;
        }
        task_handle=task_model_init(cxt_mgr);
        if(!task_handle)
        {
			debug_msg("task_model_init err\n");
            err=ERROR_TASK_HANDLE;
            break;
        }
        aver_xilinx_handle=aver_xilinx_init(cxt_mgr,pci_handle,&aver_xilinx_cfg);
        aver_xilinx_init_registers(aver_xilinx_handle, &aver_xilinx_cfg);
        
        if(!aver_xilinx_handle)
        {
			debug_msg("aver_xilinx_init err\n");
            err=ERROR_AVER_XILINX;
            break;
        }
        
        aver_xilinx_get_cardid(aver_xilinx_handle, &card_id);
        //g_card_id = card_id;
        for(i=0;i<BOARD_I2C_BUS_NUM;i++)
        {
            i2c_model_bus_cfg_t i2c_bus_cfg;
            const char *i2c_bus_name;
            
            i2c_bus_name=board_get_i2c_bus_name(i);
            switch(i)
            {
                case I2C_BUS_COM:
                    aver_xilinx_get_i2c_bus_cfg(aver_xilinx_handle,AVER_XILINX_I2C_BUS_0,&i2c_bus_cfg);
                    i2c_model_new_bus(i2c_mgr,i2c_bus_name,&i2c_bus_cfg);
                    break;
                
            }
            
        }
        ret=board_gpio_init(cxt_mgr);
        if(ret!=0)
        {
			debug_msg("board_gpio_init\n");
            err=ERROR_BOARD_GPIO_INIT;
            break;
        }
        ret=board_i2c_init(cxt_mgr,driver_info);
        if(ret!=0)
	    {
		    debug_msg("board_i2c_init\n");
            err=ERROR_BOARD_I2C_INIT;
            break;
	    }   
       
     
        adv7619_handle_1=i2c_model_get_driver_handle(i2c_mgr,ADV7619_DRVNAME);
	    if(!adv7619_handle_1)
	    {
			debug_msg("adv7619_handle_1\n");
		    break;
	    }
	    adv7619_add_trace(adv7619_handle_1,trace_handle); 
	    
	    debug_msg("%s subsystem_id=%x\n",__func__,subsystem_id);
	    #ifdef CONFIG_CARD_ID_CUSTOMER
	    debug_msg("%s card_id=%x\n",__func__,card_id);
		
		if (card_id ==0)
	    {
			card_id =6; //
			mesg("%s forced card_id=%d\n",__func__,card_id);
		}
		
		if (card_id ==1)
	    {
			card_id =7; //
			mesg("%s forced card_id=%d\n",__func__,card_id);
		}
	    
	    #else
	    mesg("%s CONFIG_CARD_ID disable\n",__func__);
	    #endif
	    
        board_alsa_init(cxt_mgr,card_id); 
        //board_v4l2_init(cxt_mgr,card_id,board_enable_notification_image);
        board_v4l2_init(cxt_mgr,card_id);
        
        
    }while(0);
    if(err!=NO_ERROR)
    {
        debug_msg("%s err\n",__func__,err);
        debug_msg(">>>board_probe fail \n");
        switch(err)
        {
            case ERROR_AVER_XILINX:
                cxt_manager_unref_context(i2c_mgr);
            case ERROR_I2C_MGR:
                cxt_manager_unref_context(gpio_mgr);
            case ERROR_GPIO_MGR:    
                 cxt_manager_unref_context(trace_handle);
            case ERROR_TRACE_HANDLE:
                
            case NO_PCI_HANDLE:
                break;
            default:
                break;
        }
        return err;
    }
    
    
    return 0;
}

void board_suspend(struct device *dev)
{
    cxt_mgr_handle_t cxt_mgr;

    if (!dev)
    {
        debug_msg("Error: pointer is NULL");
        return;
    }

    cxt_mgr = get_cxt_manager(dev);
    if (!cxt_mgr)
    {
        debug_msg("Error: pointer is NULL");
        return;
    }

    board_v4l2_suspend(cxt_mgr);
}

void board_resume(struct device *dev)
{
    cxt_mgr_handle_t cxt_mgr;
    handle_t aver_xilinx_handle;

    if (!dev)
    {
        mesg("Error: pointer is NULL");
        return;
    }

    cxt_mgr = get_cxt_manager(dev);
    if (!cxt_mgr)
    {
        mesg("Error: pointer is NULL");
        return;
    }

    aver_xilinx_handle=cxt_manager_get_context(cxt_mgr,AVER_XILINX_CXT_ID,0);
    if (!aver_xilinx_handle)
    {
        mesg("Error: pointer is NULL");
        return;
    }

    aver_xilinx_init_registers(aver_xilinx_handle, &aver_xilinx_cfg);

    board_v4l2_resume(cxt_mgr);
}

void board_remove(struct device *dev)
{
    debug_msg("%s\n",__func__);
}


int board_init()
{
    pci_model_driver_setup_t pci_setup;
    
    pci_setup.name=BOARD_NAME;
    pci_setup.id_table=id_table;
    pci_setup.prob_func=board_probe;
    pci_setup.suspend_func=board_suspend;
    pci_setup.resume_func=board_resume;
    pci_setup.remove_func=board_remove;
    //pci_setup.flags= PCI_MODEL_DISABLE_MSI | PCI_MODEL_FORCE_DMA_32;
    pci_setup.flags=PCI_MODEL_FORCE_DMA_32;

    //board_enable_notification_image = 0;

    pci_model_driver_init(&pci_setup);

	return 0;
}

void board_exit()
{
    pci_model_driver_exit();
}
