#ifndef ADV7180_H_
#define ADV7180_H_

#ifdef __cplusplus
extern "C" {
#endif

#define ADV7180_ID adv7180
#define ADV7180_DRVNAME I2C_MODEL_MODULE_STR(adv7180)

#define	ADV_7180_BRIGHTNESS_MAX			        127
#define	ADV_7180_BRIGHTNESS_DEFAULT_NTSC		-30
#define	ADV_7180_BRIGHTNESS_DEFAULT_PAL 		0
#define	ADV_7180_BRIGHTNESS_MIN			        -128

#define	ADV_7180_CONTRAST_MAX			        255
#define	ADV_7180_CONTRAST_DEFAULT_NTSC		    128
#define	ADV_7180_CONTRAST_DEFAULT_PAL		    121
#define	ADV_7180_CONTRAST_MIN			        0

#define	ADV_7180_HUE_MAX				        127
#define	ADV_7180_HUE_DEFAULT_NTSC			    0
#define	ADV_7180_HUE_DEFAULT_PAL			    0
#define	ADV_7180_HUE_MIN				        -128

#define	ADV_7180_SATURATION_MAX			        255
#define	ADV_7180_SATURATION_DEFAULT_NTSC		143
#define	ADV_7180_SATURATION_DEFAULT_PAL		    135
#define	ADV_7180_SATURATION_MIN			        0

#define USER_MAP 0
#define USER_SUB_MAP 1

typedef void (*hw_reset_func_t)(void *);

typedef struct {
	hw_reset_func_t hw_reset_func;
} adv7180_cfg_t;

typedef enum
{
    ADV7180_CS_YUV,
    //CS_RGB_FULL,
    //CS_RGB_LIMIT,
}adv7180_colorspace_e;

typedef struct
{
	U32_T h_total;
    U32_T v_total;
    unsigned int width;
    unsigned int height;
    BOOL_T is_interlace;
    unsigned int framerate;
    unsigned int denominator;
    unsigned int framerate_ext; //extend
    unsigned int denominator_ext;
    U32_T    pixel_clock;
    int macrovision_level;
    adv7180_colorspace_e colorspace;
}adv7180_frameinfo_t;

typedef enum {
	ADV7180_LOCK,
	ADV7180_UNLOCK,
	ADV7180_MACROVISION,
} adv7180_event_e;

typedef enum {
	ADV7180_MACROVISION_DISABLE = 0,
	ADV7180_MACROVISION_LEVEL1,
	ADV7180_MACROVISION_LEVEL2,
	ADV7180_MACROVISION_LEVEL3,
} adv7180_macro_level_e;

typedef enum {
	ADV7180_NTSC = 0,
	ADV7180_PAL,
} adv7180_system_type_e;

typedef enum {
	ADV7180_NTSC_M = 0, //525
	ADV7180_NTSC_433, //525
	ADV7180_PAL_M, //525
	ADV7180_PAL_60, //525
	ADV7180_PAL_BGHID,  //PAL B/G/H/I/D
	ADV7180_SECAM_B,
	ADV7180_PAL_N_COMBO,
	ADV7180_SECAM,
	ADV7180_NONE,
} adv7180_system_standard_e;

typedef enum {
	ADV7180_INPUT_CVBS,
	ADV7180_INPUT_S_VIDEO,
} adv7180_input_e;

typedef void (*adv7180_callback_t)(void *cxt, adv7180_event_e event);
void adv7180_get_frameinfo(handle_t adv7180_handle,adv7180_frameinfo_t *frameinfo);
void adv7180_register_callback(handle_t adv7180_handle, adv7180_callback_t callback_func, void *cxt);
void adv7180_unregister_callback(handle_t adv7180_handle);
void adv7180_set_input(handle_t adv7180_handle, adv7180_input_e input);
adv7180_system_type_e adv7180_get_system_type(handle_t adv7180_handle);

#ifdef __cplusplus
}
#endif

#endif /* ADV7180_H_ */

