/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2018
 * All rights reserved
 * =================================================================
 * infoframe.h
 *
 *  Created on: July 09, 2018
 *      Author: 
 *      Version:
 * =================================================================
 */

#ifndef INFOFRAME_H
#define INFOFRAME_H

#ifdef __cplusplus
extern "C" {
#endif

//#include <stdbool.h>

//////////////////////////
#define	ENOMEM		12	/* Out of Memory */
#define	EINVAL		22	/* Invalid argument */
#define ENOSPC		28	/* No space left on device */

#define DB1 0
#define DB2 1
#define DB3 2
#define DB4 3
#define DB5 4
#define DB6 5
#define DB7 6
#define DB8 7
#define DB9 8
#define DB10 9
#define DB11 10
#define DB12 11
#define DB13 12
#define DB14 13
#define DB15 14
#define DB16 15
#define DB17 16
#define DB18 17
#define DB19 18
#define DB20 19
#define DB21 20
#define DB22 21
#define DB23 22
#define DB24 23
#define DB25 24
#define DB26 25
#define DB27 26

enum hdmi_infoframe_type {
    HDMI_INFOFRAME_TYPE_VENDOR = 0x81,
    HDMI_INFOFRAME_TYPE_AVI = 0x82,
    HDMI_INFOFRAME_TYPE_SPD = 0x83,
    HDMI_INFOFRAME_TYPE_AUDIO = 0x84,
    HDMI_INFOFRAME_TYPE_MPEG_SOURCE = 0x85,
    HDMI_INFOFRAME_TYPE_NTSC_VBI = 0x86,
    HDMI_INFOFRAME_TYPE_DRM = 0x87,
};

#define HDMI_IEEE_OUI 0x000c03
#define HDMI_FORUM_IEEE_OUI 0xc45dd8
#define HDMI_INFOFRAME_HEADER_SIZE  3
#define HDMI_AVI_INFOFRAME_SIZE    13
#define HDMI_VENDOR_INFOFRAME_SIZE    25
#define HDMI_AUDIO_INFOFRAME_SIZE  10

#define HDMI_INFOFRAME_SIZE(type)	\
	(HDMI_INFOFRAME_HEADER_SIZE + HDMI_ ## type ## _INFOFRAME_SIZE)

struct hdmi_any_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
};

enum hdmi_colorspace {
    HDMI_COLORSPACE_RGB,
    HDMI_COLORSPACE_YUV422,
    HDMI_COLORSPACE_YUV444,
    HDMI_COLORSPACE_YUV420,
    HDMI_COLORSPACE_RESERVED4,
    HDMI_COLORSPACE_RESERVED5,
    HDMI_COLORSPACE_RESERVED6,
    HDMI_COLORSPACE_IDO_DEFINED,
};

enum hdmi_bar_data_present {
    HDMI_BAR_DATA_NOT_PRESENT,
    HDMI_BAR_DATA_VERT_BAR_ONLY,
    HDMI_BAR_DATA_HORZ_BAR_ONLY,
    HDMI_BAR_DATA_BOTH_VERT_HORZ,
};

enum hdmi_scan_mode {
    HDMI_SCAN_MODE_NONE,
    HDMI_SCAN_MODE_OVERSCAN,
    HDMI_SCAN_MODE_UNDERSCAN,
    HDMI_SCAN_MODE_RESERVED,
};

enum hdmi_colorimetry {
    HDMI_COLORIMETRY_NONE,
    HDMI_COLORIMETRY_ITU_601,
    HDMI_COLORIMETRY_ITU_709,
    HDMI_COLORIMETRY_EXTENDED,
};

enum hdmi_picture_aspect {
    HDMI_PICTURE_ASPECT_NONE,
    HDMI_PICTURE_ASPECT_4_3,
    HDMI_PICTURE_ASPECT_16_9,
    HDMI_PICTURE_ASPECT_64_27,
    HDMI_PICTURE_ASPECT_256_135,
    HDMI_PICTURE_ASPECT_RESERVED,
};

enum hdmi_active_aspect {
    HDMI_ACTIVE_ASPECT_16_9_TOP = 2,
    HDMI_ACTIVE_ASPECT_14_9_TOP = 3,
    HDMI_ACTIVE_ASPECT_16_9_CENTER = 4,
    HDMI_ACTIVE_ASPECT_PICTURE = 8,
    HDMI_ACTIVE_ASPECT_4_3 = 9,
    HDMI_ACTIVE_ASPECT_16_9 = 10,
    HDMI_ACTIVE_ASPECT_14_9 = 11,
    HDMI_ACTIVE_ASPECT_4_3_SP_14_9 = 13,
    HDMI_ACTIVE_ASPECT_16_9_SP_14_9 = 14,
    HDMI_ACTIVE_ASPECT_16_9_SP_4_3 = 15,
};

enum hdmi_extended_colorimetry {
    HDMI_EXTENDED_COLORIMETRY_XV_YCC_601,
    HDMI_EXTENDED_COLORIMETRY_XV_YCC_709,
    HDMI_EXTENDED_COLORIMETRY_S_YCC_601,
    HDMI_EXTENDED_COLORIMETRY_ADOBE_YCC_601,
    HDMI_EXTENDED_COLORIMETRY_ADOBE_RGB,

    /* The following EC values are only defined in CEA-861-F. */
    HDMI_EXTENDED_COLORIMETRY_BT2020_CONST_LUM,
    HDMI_EXTENDED_COLORIMETRY_BT2020,
    HDMI_EXTENDED_COLORIMETRY_RESERVED,
};

enum hdmi_quantization_range {
    HDMI_QUANTIZATION_RANGE_DEFAULT,
    HDMI_QUANTIZATION_RANGE_LIMITED,
    HDMI_QUANTIZATION_RANGE_FULL,
    HDMI_QUANTIZATION_RANGE_RESERVED,
};

/* non-uniform picture scaling */
enum hdmi_nups {
    HDMI_NUPS_UNKNOWN,
    HDMI_NUPS_HORIZONTAL,
    HDMI_NUPS_VERTICAL,
    HDMI_NUPS_BOTH,
};

enum hdmi_ycc_quantization_range {
    HDMI_YCC_QUANTIZATION_RANGE_LIMITED,
    HDMI_YCC_QUANTIZATION_RANGE_FULL,
};

enum hdmi_content_type {
    HDMI_CONTENT_TYPE_GRAPHICS,
    HDMI_CONTENT_TYPE_PHOTO,
    HDMI_CONTENT_TYPE_CINEMA,
    HDMI_CONTENT_TYPE_GAME,
};

enum hdmi_additional_colorimetry_extension {
    HDMI_ACE_DCI_P3_RGB_D65,
    HDMI_ACE_DCI_P3_RGB_THEATER,
};

struct hdmi_avi_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    enum hdmi_colorspace colorspace;
    BOOL_T active_format_information_present;
    enum hdmi_bar_data_present bar_data_present;
    enum hdmi_scan_mode scan_mode;
    enum hdmi_colorimetry colorimetry;
    enum hdmi_picture_aspect picture_aspect;
    enum hdmi_active_aspect active_aspect;
    BOOL_T itc;
    enum hdmi_extended_colorimetry extended_colorimetry;
    enum hdmi_quantization_range quantization_range;
    enum hdmi_nups nups; //Non-Uniform Picture Scaling
    unsigned char video_code;
    enum hdmi_ycc_quantization_range ycc_quantization_range;
    enum hdmi_content_type content_type;
    enum hdmi_additional_colorimetry_extension ace;
    unsigned char pixel_repeat;
    unsigned short top_bar;
    unsigned short bottom_bar;
    unsigned short left_bar;
    unsigned short right_bar;
};

//int hdmi_avi_infoframe_init(struct hdmi_avi_infoframe *frame, void *buf);
//long hdmi_avi_infoframe_pack(struct hdmi_avi_infoframe *frame, void *buffer,long size);

enum hdmi_spd_sdi {
    HDMI_SPD_SDI_UNKNOWN,
    HDMI_SPD_SDI_DSTB,
    HDMI_SPD_SDI_DVDP,
    HDMI_SPD_SDI_DVHS,
    HDMI_SPD_SDI_HDDVR,
    HDMI_SPD_SDI_DVC,
    HDMI_SPD_SDI_DSC,
    HDMI_SPD_SDI_VCD,
    HDMI_SPD_SDI_GAME,
    HDMI_SPD_SDI_PC,
    HDMI_SPD_SDI_BD,
    HDMI_SPD_SDI_SACD,
    HDMI_SPD_SDI_HDDVD,
    HDMI_SPD_SDI_PMP,
};

struct hdmi_spd_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    char vendor[8];
    char product[16];
    enum hdmi_spd_sdi sdi;
};

//int hdmi_spd_infoframe_init(struct hdmi_spd_infoframe *frame, void *buf);
//long hdmi_spd_infoframe_pack(struct hdmi_spd_infoframe *frame, void *buffer,long size);

enum hdmi_mpeg_source_frame_type {
    HDMI_MPEG_SOURCE_FRAME_TYPE_UNKNOWN,
    HDMI_MPEG_SOURCE_FRAME_TYPE_I_PICTURE,
    HDMI_MPEG_SOURCE_FRAME_TYPE_B_PICTURE,
    HDMI_MPEG_SOURCE_FRAME_TYPE_P_PICTURE,
};

struct hdmi_mpeg_source_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    unsigned int mpeg_bit_rate;
    unsigned char field_repeat;
    enum hdmi_mpeg_source_frame_type mpeg_frame;
    //unsigned int mpeg_frame;
};
//int hdmi_mpeg_source_infoframe_init(struct hdmi_mpeg_source_infoframe *frame, void *buf);

struct hdmi_ntsc_vbi_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    unsigned char payload[27];
    unsigned char checksum;
};
int hdmi_ntsc_vbi_infoframe_init(struct hdmi_ntsc_vbi_infoframe *frame, void *buf);

enum hdmi_drm_eotf_id {
    HDMI_DRM_EOTF_SDR,
    HDMI_DRM_EOTF_HDR,
    HDMI_DRM_EOTF_ST_2084,
    HDMI_DRM_EOTF_HLG,
};

enum hdmi_drm_smd_id {
    HDMI_DRM_STATIC_METADATA_TYPE_1,
};

struct hdmi_drm_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    enum hdmi_drm_eotf_id eotf_id;
    enum hdmi_drm_smd_id smd_id;
    unsigned short display_prim_x[3];
    unsigned short display_prim_y[3];
    unsigned short white_point_x;
    unsigned short white_point_y;
    unsigned short max_display_mastering_luminance;
    unsigned short min_display_mastering_luminance;
    unsigned short max_content_light_level;
    unsigned short max_frame_average_light_level;

};
union hdmi_drm_any_infoframe {
    struct {
        enum hdmi_infoframe_type type;
        unsigned char version;
        unsigned char length;
        enum hdmi_drm_eotf_id eotf_id;
        enum hdmi_drm_smd_id smd_id;
    } any;
    struct hdmi_drm_infoframe type1;
};
//int hdmi_drm_infoframe_init(union hdmi_drm_any_infoframe *frame, void *buf);

enum hdmi_audio_coding_type {
    HDMI_AUDIO_CODING_TYPE_STREAM,
    HDMI_AUDIO_CODING_TYPE_PCM,
    HDMI_AUDIO_CODING_TYPE_AC3,
    HDMI_AUDIO_CODING_TYPE_MPEG1,
    HDMI_AUDIO_CODING_TYPE_MP3,
    HDMI_AUDIO_CODING_TYPE_MPEG2,
    HDMI_AUDIO_CODING_TYPE_AAC_LC,
    HDMI_AUDIO_CODING_TYPE_DTS,
    HDMI_AUDIO_CODING_TYPE_ATRAC,
    HDMI_AUDIO_CODING_TYPE_DSD,
    HDMI_AUDIO_CODING_TYPE_EAC3,
    HDMI_AUDIO_CODING_TYPE_DTS_HD,
    HDMI_AUDIO_CODING_TYPE_MLP,
    HDMI_AUDIO_CODING_TYPE_DST,
    HDMI_AUDIO_CODING_TYPE_WMA_PRO,
    HDMI_AUDIO_CODING_TYPE_CXT,
};

enum hdmi_audio_sample_size {
    HDMI_AUDIO_SAMPLE_SIZE_STREAM,
    HDMI_AUDIO_SAMPLE_SIZE_16,
    HDMI_AUDIO_SAMPLE_SIZE_20,
    HDMI_AUDIO_SAMPLE_SIZE_24,
};

enum hdmi_audio_sample_frequency {
    HDMI_AUDIO_SAMPLE_FREQUENCY_STREAM,
    HDMI_AUDIO_SAMPLE_FREQUENCY_32000,
    HDMI_AUDIO_SAMPLE_FREQUENCY_44100,
    HDMI_AUDIO_SAMPLE_FREQUENCY_48000,
    HDMI_AUDIO_SAMPLE_FREQUENCY_88200,
    HDMI_AUDIO_SAMPLE_FREQUENCY_96000,
    HDMI_AUDIO_SAMPLE_FREQUENCY_176400,
    HDMI_AUDIO_SAMPLE_FREQUENCY_192000,
};

enum hdmi_audio_coding_type_ext {
    /* Refer to Audio Coding Type (CT) field in Data Byte 1 */
    HDMI_AUDIO_CODING_TYPE_EXT_CT,

    /*
    * The next three CXT values are defined in CEA-861-E only.
    * They do not exist in older versions, and in CEA-861-F they are
    * defined as 'Not in use'.
    */
    HDMI_AUDIO_CODING_TYPE_EXT_HE_AAC,
    HDMI_AUDIO_CODING_TYPE_EXT_HE_AAC_V2,
    HDMI_AUDIO_CODING_TYPE_EXT_MPEG_SURROUND,

    /* The following CXT values are only defined in CEA-861-F. */
    HDMI_AUDIO_CODING_TYPE_EXT_MPEG4_HE_AAC,
    HDMI_AUDIO_CODING_TYPE_EXT_MPEG4_HE_AAC_V2,
    HDMI_AUDIO_CODING_TYPE_EXT_MPEG4_AAC_LC,
    HDMI_AUDIO_CODING_TYPE_EXT_DRA,
    HDMI_AUDIO_CODING_TYPE_EXT_MPEG4_HE_AAC_SURROUND,
    HDMI_AUDIO_CODING_TYPE_EXT_MPEG4_AAC_LC_SURROUND = 10,
};

enum hdmi_audio_lfe_playback_level {
    HDMI_AUDIO_LFEPBL_UNKNOWN,
    HDMI_AUDIO_LFEPBL_0db,
    HDMI_AUDIO_LFEPBL_10db,
};

struct hdmi_audio_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    unsigned char channels;
    enum hdmi_audio_coding_type coding_type;
    enum hdmi_audio_sample_size sample_size;
    enum hdmi_audio_sample_frequency sample_frequency;
    enum hdmi_audio_coding_type_ext coding_type_ext;
    unsigned char channel_allocation;
    enum hdmi_audio_lfe_playback_level lfe_playback_level;
    unsigned char level_shift_value;
    BOOL_T downmix_inhibit;

};

//int hdmi_audio_infoframe_init(struct hdmi_audio_infoframe *frame, void *buf);
//long hdmi_audio_infoframe_pack(struct hdmi_audio_infoframe *frame,void *buffer, long size);

enum hdmi_3d_structure {
    HDMI_3D_STRUCTURE_INVALID = -1,
    HDMI_3D_STRUCTURE_FRAME_PACKING = 0,
    HDMI_3D_STRUCTURE_FIELD_ALTERNATIVE,
    HDMI_3D_STRUCTURE_LINE_ALTERNATIVE,
    HDMI_3D_STRUCTURE_SIDE_BY_SIDE_FULL,
    HDMI_3D_STRUCTURE_L_DEPTH,
    HDMI_3D_STRUCTURE_L_DEPTH_GFX_GFX_DEPTH,
    HDMI_3D_STRUCTURE_TOP_AND_BOTTOM,
    HDMI_3D_STRUCTURE_SIDE_BY_SIDE_HALF = 8,
};

enum hdmi_3d_ext_data {
    HDMI_3D_EXT_DATA_HOR_ODD_ODD = 0,
    HDMI_3D_EXT_DATA_HOR_ODD_EVEN,
    HDMI_3D_EXT_DATA_HOR_EVEN_ODD,
    HDMI_3D_EXT_DATA_HOR_EVEN_EVEN,
    HDMI_3D_EXT_DATA_QUIN_ODD_ODD,
    HDMI_3D_EXT_DATA_QUIN_ODD_EVEN,
    HDMI_3D_EXT_DATA_QUIN_EVEN_ODD,
    HDMI_3D_EXT_DATA_QUIN_EVEN_EVEN,
};

enum hdmi_3d_meta_data_type {
    HDMI_3D_META_DATA_TYPE_ISO23002_3 = 0,
};

struct hdmi_vendor_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    unsigned int oui;
#if 1
    unsigned char vic;
    enum hdmi_3d_structure s3d_struct;
    enum hdmi_3d_ext_data s3d_ext_data;
    unsigned char is_3d_meta_present;
    enum hdmi_3d_meta_data_type s3d_meta_data_type;
    unsigned char n3d_meta_data_length;
    unsigned char u3d_meta_data[31];
#else
    unsigned payload[255];
#endif
};

struct hdmi_forum_vendor_infoframe {
    enum hdmi_infoframe_type type;
    unsigned char version;
    unsigned char length;
    unsigned int oui;
    //TODO:complete the data structure
    unsigned payload[255];
};

//int hdmi_vendor_any_infoframe_init(union hdmi_vendor_any_infoframe *frame, void *buf);
//int hdmi_vendor_infoframe_init(struct hdmi_vendor_infoframe *frame, void *buf);
//long hdmi_vendor_infoframe_pack(struct hdmi_vendor_infoframe *frame,void *buffer, long size);

//any and hdmi shares same memory
union hdmi_vendor_any_infoframe {
    struct {
        enum hdmi_infoframe_type type;
        unsigned char version;
        unsigned char length;
        unsigned int oui;
        unsigned char payload[255];
    } any;
    struct hdmi_vendor_infoframe hdmi;
    struct hdmi_forum_vendor_infoframe hdmi_forum;
};

union hdmi_infoframe {
    struct hdmi_any_infoframe any;
    struct hdmi_avi_infoframe avi;
    struct hdmi_spd_infoframe spd;
    union hdmi_vendor_any_infoframe vendor;
    struct hdmi_audio_infoframe audio;
    struct hdmi_mpeg_source_infoframe mpeg;
    struct hdmi_ntsc_vbi_infoframe vbi;
    union hdmi_drm_any_infoframe drm;
};

typedef union hdmi_infoframe sHdmiInfoFrame;




//int hdmi_infoframe_unpack(union hdmi_infoframe *frame, void *buffer);

//int UnpackHdmiInfoFrame(unsigned char *pBuf, sHdmiInfoFrame *pFrame, unsigned int nDataByte1Offset = 4);
//int PrintHdmiInfoFrame(sHdmiInfoFrame *pFrame);


#ifdef __cplusplus
}
#endif

#endif /* INFOFRAME_H */
