/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * v4l2_model_device.c
 *
 *  Created on: Apr 15, 2017
 *      Author: 
 *      Version:
 * =================================================================
 */
 
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/mutex.h>
#include <linux/slab.h>
#include <media/v4l2-device.h>
#include <media/v4l2-ioctl.h>
#include <media/videobuf2-vmalloc.h>
#include <linux/v4l2-dv-timings.h>
#include <linux/videodev2.h>
#include <linux/version.h>

#include <linux/vmalloc.h>

#include "v4l2_model.h"
#include "v4l2_model_videobuf2.h"
#include "v4l2_model_ioctl.h"
#include "v4l2_model_device.h"
#include "framegrabber.h"

#if defined (AVER_CARD_CN312S)
#define DEVICE_NAME "cn312s"
#elif defined (AVER_CARD_CN312M)
#define DEVICE_NAME "cn312m"
#elif defined (AVER_CARD_CM311H)
#define DEVICE_NAME "cm311h"
#elif defined (AVER_CARD_CL311MN)
#define DEVICE_NAME "cl311mn"
#elif defined (AVER_CARD_CL311M2)
#define DEVICE_NAME "cl311m2"
#endif

typedef struct v4l2_model_fh_s
{
    struct v4l2_fh fh;
}v4l2_model_fh_t;

#if 0
struct v4l2_dump_flash {
	__u32 pad;
	__u32 start_block;
	__u32 blocks;
	__u32 reserved[5];
	__u8  *flash;
};
#endif

static int v4l2_model_fops_open(struct file *);
static int v4l2_model_fops_release(struct file *);
static int v4l2_model_fops_mmap (struct file *, struct vm_area_struct *);
static unsigned int v4l2_model_fops_poll (struct file *, struct poll_table_struct *);
static ssize_t v4l2_model_fops_read (struct file *, char __user *, size_t, loff_t *);
static ssize_t v4l2_model_fops_write (struct file *, const char __user *, size_t, loff_t *);
static long v4l2_model_ioctl(struct file *file, unsigned int cmd, unsigned long arg);
static void v4l2_model_notify(struct v4l2_subdev *sd,unsigned int notification, void *arg);
static void *v4l2_model_alloc(void);
static void v4l2_model_release(void *context);

//void v4l2_board_register_subdev(struct sagitta_dev *sdev, struct v4l2_device *v4l2_dev);

#if 1
static struct v4l2_file_operations v4l2_model_fops = {
	.owner			= THIS_MODULE,
	.open			= v4l2_model_fops_open,
	.release		= v4l2_model_fops_release,
	.unlocked_ioctl	= v4l2_model_ioctl,
	.mmap			= v4l2_model_fops_mmap,
	.poll			= v4l2_model_fops_poll,
	.read			= v4l2_model_fops_read,
	.write			= v4l2_model_fops_write,
};
#else
static const struct v4l2_file_operations v4l2_model_fops = {
	.owner		= THIS_MODULE,
	.open           = v4l2_model_fops_open,
	.release        = v4l2_model_fops_release,
	.read           = vb2_fop_read,
	.write          = vb2_fop_write,
	.poll		= vb2_fop_poll,
	.unlocked_ioctl = video_ioctl2,
	.mmap           = vb2_fop_mmap,
};
#endif

static struct v4l2_ioctl_ops v4l2_model_ioctl_ops = {
	.vidioc_reqbufs             = vb2_ioctl_reqbufs,
	.vidioc_create_bufs         = vb2_ioctl_create_bufs,
	.vidioc_prepare_buf         = vb2_ioctl_prepare_buf,
	.vidioc_querybuf            = vb2_ioctl_querybuf,
	.vidioc_qbuf                = vb2_ioctl_qbuf,
	.vidioc_dqbuf               = vb2_ioctl_dqbuf,
	.vidioc_streamon 			= vb2_ioctl_streamon,
	.vidioc_streamoff 			= vb2_ioctl_streamoff,

	.vidioc_querycap      		= v4l2_model_ioctl_querycap,
	.vidioc_enum_fmt_vid_cap	= v4l2_model_ioctl_enum_fmt_vid_cap,
	.vidioc_g_fmt_vid_cap		= v4l2_model_ioctl_g_fmt_vid_cap,
	.vidioc_try_fmt_vid_cap  	= v4l2_model_ioctl_try_fmt_vid_cap,
	.vidioc_s_fmt_vid_cap     	= v4l2_model_ioctl_s_fmt_vid_cap,
	.vidioc_enum_framesizes   	= v4l2_model_ioctl_enum_framesizes,
	.vidioc_enum_input    		= v4l2_model_ioctl_enum_input,
	.vidioc_g_input       		= v4l2_model_ioctl_g_input,
	.vidioc_s_input       		= v4l2_model_ioctl_s_input,
	.vidioc_enum_frameintervals = v4l2_model_ioctl_enum_frameintervals,
	.vidioc_g_parm        		= v4l2_model_ioctl_g_parm,
	.vidioc_s_parm        		= v4l2_model_ioctl_s_parm,
	.vidioc_g_ctrl        		= v4l2_model_ioctl_g_ctrl,
	.vidioc_s_ctrl        		= v4l2_model_ioctl_s_ctrl,
	.vidioc_queryctrl           = v4l2_model_ioctl_queryctrl,
	//.vidioc_g_edid				= v4l2_model_ioctl_g_edid,
	//.vidioc_s_edid				= v4l2_model_ioctl_s_edid,
	
	//.vidioc_cropcap				= v4l2_model_ioctl_cropcap,
    //.vidioc_s_dv_timings        = v4l2_model_ioctl_s_dv_timings,
    //.vidioc_g_dv_timings        = v4l2_model_ioctl_g_dv_timings,
#if 0
    .vidioc_s_std               = v4l2_model_ioctl_s_std,
    .vidioc_g_std               = v4l2_model_ioctl_g_std,
    .vidioc_querystd            = v4l2_model_ioctl_querystd,
    .vidioc_g_tuner             = v4l2_model_g_tuner,
    .vidioc_s_tuner             = v4l2_model_s_tuner,
#endif
};

v4l2_model_handle_t v4l2_model_init(cxt_mgr_handle_t cxt_mgr,v4l2_model_device_setup_t *device_info,framegrabber_handle_t framegrabber_handle,unsigned long card_id)
{
	v4l2_model_context_t *context=NULL;
	if(cxt_mgr)
	{
		struct video_device *vdev;
		int ret;
		int devicetype = 0;
		int i;
		struct device *dev;
		v4l2_model_error_e err=V4L2_MODEL_OK;
		int card_id_low=-1;

		do
		{
			dev=cxt_manager_get_dev(cxt_mgr);
			if(!dev)
			{
				err=V4L2_MODEL_ERROR_NO_DEV;
				break;
			}
			context=cxt_manager_add_cxt(cxt_mgr,V4L2_CXT_ID,v4l2_model_alloc,v4l2_model_release);
			if(!context)
			{
				err=V4L2_MODEL_ERROR_ALLOC;
				break;
			}
                        context->dev=dev;
			mutex_init(&context->lock);
			for(i=0;i<HOOK_IOCTL_TYPE_NUM;i++)
				INIT_LIST_HEAD(&context->ioctl_hook_list[i]);
		    memcpy(&context->device_info,device_info, sizeof(v4l2_model_device_setup_t));
		    switch(context->device_info.type)
		    {
		    	case DEVICE_TYPE_GRABBER:
		    	    devicetype = VFL_TYPE_GRABBER;
		    		break;
		    	case DEVICE_TYPE_VBI:
		    		devicetype = VFL_TYPE_VBI;
		    		break;
		    	case DEVICE_TYPE_RADIO:
		            devicetype = VFL_TYPE_RADIO;
		    		break;
		        case DEVICE_TYPE_SUBDEV:
		            devicetype = VFL_TYPE_SUBDEV;
		    		break;
		        default:
		            printk("%s no supported device type 0x%x\n",__func__, context->device_info.type);
		            break;
		    }
		    ret = v4l2_device_register(dev,&context->v4l2_dev);
		    if(ret)
		    {
		    	err=V4L2_MODEL_ERROR_REGISTER_V4L2;
		    	break;
		    }
		    context->framegrabber_handle=framegrabber_handle;
		   	//framegrabber_associate_v4l2dev(framegrabber_handle,&context->v4l2_dev);
		    context->v4l2_dev.notify=v4l2_model_notify;
		    /* initialize video buffer 2 and get vb2 context */
			context->vb2_context = v4l2_model_vb2_init(&context->queue, devicetype,device_info->buffer_type,dev, context);
		    if(!context->vb2_context)
		    {
		    	err=V4L2_MODEL_ERROR_VIDEO_BUF;
		    	break;
		    }
		    context->pic_bmp_handle = cxt_manager_get_context(cxt_mgr, PCI_BMP_CXT_ID, 0); 

		    vdev = &context->vdev;
            snprintf(vdev->name, sizeof(vdev->name), "%s", DEVICE_NAME);
            vdev->device_caps = V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_STREAMING;
            //vdev->device_caps = V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VIDEO_OVERLAY | V4L2_CAP_STREAMING | V4L2_CAP_READWRITE;
	        vdev->release = video_device_release_empty;
	    	vdev->v4l2_dev = &context->v4l2_dev;
	        vdev->vfl_dir = VFL_DIR_RX;
	    	vdev->fops = &v4l2_model_fops;
	    	vdev->ioctl_ops = &v4l2_model_ioctl_ops;
	    	vdev->lock = &context->lock;
	        vdev->queue = &context->queue;
	        /* debug message level */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,0,0)
	        vdev->dev_debug=0;
#else
	        vdev->debug = 0;
#endif
	        video_set_drvdata(vdev, context);
            #ifdef CONFIG_CARD_ID_CUSTOMER
            card_id_low = (int)(card_id & 0x0000ffff);
            #endif
            if ((device_info->board_id == 0x3115) || (device_info->board_id == 0x3117))
            {
	            ret = video_register_device(vdev, devicetype, card_id);
	            printk("%s device_id = %ld\n",__func__, card_id);
			}
	        else
	        {
	            ret = video_register_device(vdev, devicetype, /*-1*/card_id_low);
	            printk("%s default device_id\n",__func__);
			}
	        
#if 1	            
	        if(ret !=0) //auto setting
	        {
				ret = video_register_device(vdev, devicetype, -1);
				printk("%s reset\n",__func__);
			}
#endif	        
	        if(ret)
	        {
	        	err=V4L2_MODEL_ERROR_REGISTER_VIDEO;
	        	break;
	        }

		}while(0);
		if(err!=V4L2_MODEL_OK)
		{
			switch(err)
			{
				case V4L2_MODEL_ERROR_REGISTER_VIDEO:
					v4l2_model_vb2_release(context->vb2_context);
				case V4L2_MODEL_ERROR_VIDEO_BUF:
					v4l2_device_unregister(&context->v4l2_dev);
				case V4L2_MODEL_ERROR_REGISTER_V4L2:
					kfree(context);
				case V4L2_MODEL_ERROR_ALLOC:
				case V4L2_MODEL_ERROR_NO_DEV:
					break;

				default:
					break;
			}
		}
	}

	return (v4l2_model_handle_t )context;

}

static void *v4l2_model_alloc()
{
	v4l2_model_context_t *context;

	context = kzalloc(sizeof(*context), GFP_KERNEL);

	return context;
}

static void v4l2_model_release(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;
	int i;
	struct list_head *hook_list_head=NULL;
	int release_cnt=0;

	if(!v4l2m_context) {
        return;
    }
    //printk("%s\n", __func__);
    for(i=0;i<HOOK_IOCTL_TYPE_NUM;i++)
    {
    	v4l2_model_ioctl_hook_item_t *item;
    	hook_list_head=&v4l2m_context->ioctl_hook_list[i];
    	
    	do
    	{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
    		item =list_first_entry_or_null(hook_list_head,v4l2_model_ioctl_hook_item_t,list);
#else
		item=(!list_empty(hook_list_head) ? list_first_entry(hook_list_head,v4l2_model_ioctl_hook_item_t,list) : NULL);
#endif

    		if(item)
    		{
    			list_del(&item->list);
    			kfree(item);
    		}
	   	}while(item!=NULL);	
     }

    video_unregister_device(&v4l2m_context->vdev);
    release_cnt++;
    v4l2_model_vb2_release(v4l2m_context->vb2_context);
    release_cnt++;
    v4l2_device_unregister(&v4l2m_context->v4l2_dev);
    release_cnt++;
    kfree(v4l2m_context);
    release_cnt++;
    if (release_cnt ==4)
        printk("%s...\n", __func__);
}

void v4l2_model_disconnect(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	if(v4l2m_context)
	{
		mutex_lock(v4l2m_context->queue.lock);
		mutex_lock(&v4l2m_context->lock);

		v4l2_device_disconnect(&v4l2m_context->v4l2_dev);
		mutex_unlock(&v4l2m_context->lock);
		mutex_unlock(v4l2m_context->queue.lock);
	}
}

void v4l2_model_reference(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	if(v4l2m_context)
	{
		v4l2_device_get(&v4l2m_context->v4l2_dev);
	}
}

void v4l2_model_unreference(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	if(v4l2m_context)
	{
		v4l2_device_put(&v4l2m_context->v4l2_dev);
	}
}

int v4l2_model_register_callback(v4l2_model_handle_t context,
        v4l2_model_callback_e callback_no,
        v4l2_model_callback_t callback,
        void *asso_data)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	int ret=V4L2_MODEL_OK;
	int done=0;
	do
	{
		v4l2_model_callback_item_t *callback_item;
		if(v4l2m_context==NULL)
		{
			ret=V4L2_MODEL_ERROR_INVALID_CONTEXT;
			break;
		}
		if(callback_no <0 || callback_no >=V4L2_MODEL_CALLBACK_NUM)
		{
			ret=V4L2_MODEL_ERROR_INVALID_CALLBACK_NO;
			break;
		}
		callback_item=&v4l2m_context->callbacks[callback_no];
		if(callback_item->callback)
		{
			ret=V4L2_MODEL_ERROR_DUPLICATE_CALLBACK;
			break;
		}
		callback_item->callback=callback;
		callback_item->data.asso_data=asso_data;
                switch(callback_no)
                {
                    case V4L2_MODEL_CALLBACK_STREAMON:
                    case V4L2_MODEL_CALLBACK_STREAMOFF:
                    case V4L2_MODEL_CALLBACK_BUFFER_INIT:
                    case V4L2_MODEL_CALLBACK_BUFFER_QUEUE:
                    case V4L2_MODEL_CALLBACK_BUFFER_FINISH:
                    case V4L2_MODEL_CALLBACK_BUFFER_CLEANUP:
                    #if defined (AVER_CARD_CN312S)
                    case V4L2_MODEL_CALLBACK_LINE_BUFFER:
                    #endif
                    default:
                        callback_item->data.type=BASIC_TYPE;
                        break;
                    case V4L2_MODEL_CALLBACK_BUFFER_PREPARE:
                        callback_item->data.type=BUFFER_PREPARE_TYPE;
                        break;
                    case V4L2_MODEL_CALLBACK_QUEUE_SETUP:
                        callback_item->data.type=QUEUE_SETUP_TYPE;
                        break;
                }
		done=1;
	}while(0);
	if(!done)
		return ret;
	return ret;
}

int  v4l2_model_unregister_callback(v4l2_model_handle_t context,v4l2_model_callback_e callback_no)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	int ret=V4L2_MODEL_OK;
	int done=0;
	v4l2_model_callback_item_t *callback_item;
	do
	{
		if(v4l2m_context==NULL)
                {
                    ret=V4L2_MODEL_ERROR_INVALID_CONTEXT;
                    break;
                }
		if(callback_no <0 || callback_no >=V4L2_MODEL_CALLBACK_NUM)
		{
			ret=V4L2_MODEL_ERROR_INVALID_CALLBACK_NO;
			break;
		}
		callback_item=&v4l2m_context->callbacks[callback_no];
		callback_item->callback=NULL;
		callback_item->data.asso_data=NULL;
                callback_item->data.type=BASIC_TYPE;
		done=1;
	}while(0);
	if(!done)
		return ret;

	return ret;
}

static int v4l2_model_fops_open(struct file *fp)
{
    v4l2_model_fh_t *v4l2m_fh;
    v4l2_model_context_t *v4l2m_context =
            (v4l2_model_context_t *) video_drvdata(fp);

    v4l2m_fh = kzalloc(sizeof (*v4l2m_fh), GFP_KERNEL);

    //    printk("%s v4l2m_fh  %p\n", __func__, v4l2m_fh);

    if (v4l2m_fh)
    {
        v4l2_fh_init(&v4l2m_fh->fh, &v4l2m_context->vdev);
        fp->private_data = &v4l2m_fh->fh;
        v4l2_fh_add(&v4l2m_fh->fh);
    }

    return 0;
}

static int v4l2_model_fops_release(struct file *fp)
{
	struct v4l2_fh *fh = fp->private_data;
	v4l2_model_fh_t *v4l2m_fh = container_of(fh,v4l2_model_fh_t,fh);
    //v4l2_model_context_t *v4l2m_context =  (v4l2_model_context_t *) video_drvdata(fp);


	struct video_device *vdev = video_devdata(fp);

	if (fp->private_data == vdev->queue->owner) {
		vb2_queue_release(vdev->queue);
		vdev->queue->owner = NULL;
	}
        
	v4l2_fh_del(&v4l2m_fh->fh);
	v4l2_fh_exit(&v4l2m_fh->fh);
	kfree(v4l2m_fh);

	return 0;
}

static int v4l2_model_fops_mmap (struct file *fp, struct vm_area_struct *vm)
{
	return vb2_fop_mmap(fp,vm);
}

static unsigned int v4l2_model_fops_poll (struct file *fp, struct poll_table_struct *poll)
{
	return vb2_fop_poll(fp,poll);
}

static ssize_t v4l2_model_fops_write (struct file *file, const char __user *buf,size_t count, loff_t *ppos)
{
	return vb2_fop_write(file,(char __user *)buf,count,ppos);
}

static ssize_t v4l2_model_fops_read (struct file *file, char __user *buf,size_t count, loff_t *ppos)
{
	return vb2_fop_read(file,buf,count,ppos);
}

#if defined (AVER_CARD_CN312S)
#define AVER_FLASH_IOCTL_RX_ENABLE  _IOWR('F', 205, int)
#define AVER_FLASH_IOCTL_FW_RESULT  _IOWR('F', 206, int)
#define AVER_FLASH_IOCTL_FW_PMPID  _IOWR('F', 207, int)
#define AVER_FLASH_IOCTL_FW_VERSION  _IOWR('F', 208, int)
#define AVER_FLASH_IOCTL_FW_BLOCK  _IOWR('F', 209, int)
#define AVER_FLASH_IOCTL_FW_DUMP  _IOWR('F', 210, unsigned char)
#define AVER_FLASH_IOCTL_FW_UPDATE _IOWR('F', 211, unsigned char)
#define AVER_STRM_IOCTL_BUFFER_INFO  _IOWR('S', 245, int)
#define AVER_STRM_IOCTL_LINE_NUM  _IOWR('S', 246, int)
#define AVER_STRM_IOCTL_BLOCK_NUM  _IOWR('S', 247, int) 
#define AVER_STRM_IOCTL_LAST_BLOCKSIZE  _IOWR('S', 248, int)
#elif defined (AVER_CARD_CN312M)
#define AVER_FLASH_IOCTL_FW_DUMP  _IOR('F', 210, struct v4l2_dump_flash)
#define AVER_FLASH_IOCTL_FW_UPDATE _IOWR('F', 211, struct v4l2_dump_flash)
#elif defined (AVER_CARD_CM311H)
#define AVER_FLASH_IOCTL_FW_RESULT  _IOWR('F', 206, int)
#define AVER_FLASH_IOCTL_FW_PMPID  _IOWR('F', 207, int)
#define AVER_FLASH_IOCTL_FW_VERSION  _IOWR('F', 208, int)

#define AVER_FLASH_IOCTL_FW_BLOCK  _IOWR('F', 209, int)
#define AVER_FLASH_IOCTL_FW_DUMP  _IOWR('F', 210, unsigned char)
#define AVER_FLASH_IOCTL_FW_UPDATE _IOWR('F', 211, unsigned char)
#endif

#define AVER_SET_CUSTOM_EDID	_IOWR('F', 212, struct v4l2_custom_edid)
#define AVER_GET_CUSTOM_EDID	_IOWR('F', 213, struct edid_buf)
#define AVER_SET_EDID_FT_VAL	_IOWR('F', 214, struct edid_ft_val)		//Set the edid Fault-tolerant value
#define AVER_HDMI_IOCTL_AVI_DUMP  _IOR('H', 240, unsigned char)
#define AVER_HDMI_IOCTL_VENDOR_DUMP  _IOR('H', 250, unsigned char)
#define AVER_HDMI_IOCTL_AUDIOINFO_DUMP  _IOR('H', 260, unsigned char)
#define AVER_STRM_IOCTL_BYPASS_DISABLE  _IOWR('S', 241, int)
#define AVER_STRM_IOCTL_HDCP_FLAG  _IOR('S', 242, int)
#define AVER_STRM_IOCTL_PIXEL_YUYV  _IOWR('S', 243, int)
#define AVER_STRM_IOCTL_AUDIO_RATE  _IOWR('S', 244, int)

static long v4l2_model_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{
	long ret = 0;
	unsigned char arg_tmp[30];
	int hdcp_flag=0;
	int stream_bypass_disable=0;
	int pixelformat=0;
	int audio_rate=0;
	
	#if defined (AVER_CARD_CN312S)
	int buffer_info=0xffff;
	int line_num=0;
	int block_num=0;
	int last_blocksize=0;
	
	int flash_access_block=0;
	int rx_update_enable=0;
	int flash_fpga_pmpid=0;
	int flash_fpga_version=0;
	int flash_update_result=0;
	//struct v4l2_dump_flash flash_data_info;
	unsigned char flash_data_info[256];
	unsigned char flash_data_dump[256];
	#elif defined (AVER_CARD_CN312M)
	struct v4l2_dump_flash flash_data_info;
#elif defined (AVER_CARD_CM311H)
	int flash_access_block=0;
	int flash_fpga_pmpid=0;
	int flash_fpga_version=0;
	int flash_update_result=0;

	//struct v4l2_dump_flash flash_data_info;
	unsigned char flash_data_info[256];
	unsigned char flash_data_dump[256];
#endif

	struct edid_ft_val edid_ft_data;
	struct v4l2_custom_edid edid_custom_data;
	struct edid_buf edid_data_info;
	
	//memset(&edid_ft_data, 0, sizeof(struct edid_ft_val));
	//memset(&edid_custom_data, 0, sizeof(struct v4l2_custom_edid));
	//memset(&edid_data_info, 0, sizeof(struct edid_buf));
	#if defined (AVER_CARD_CN312S)
	memset(&buffer_info, 0, sizeof(int));
	memset(&line_num, 0, sizeof(int));
	memset(&block_num, 0, sizeof(int));
	memset(&last_blocksize, 0, sizeof(int));
	#endif
	
#if 0
	hook_list_head=&v4l2m_context->ioctl_hook_list[HOOK_PREV_IOCTL];
	if(!list_empty(hook_list_head))
	{	
		list_for_each(pos,hook_list_head)
		{
			v4l2_model_ioctl_hook_item_t *ioctl_hook_item;
			ioctl_hook_item=list_entry(pos,v4l2_model_ioctl_hook_item_t,list);
			try=video_usercopy(ioctl_hook_item->data,cmd,arg,(v4l2_kioctl)ioctl_hook_item->ioctl_hook_func);
			if(try!=-EINVAL)
			{
				return try;
				
			}
		}
	}
	//printk("%s..cmd=%d\n",__func__,cmd);
#endif

	switch (cmd)
	{
	#if defined (AVER_CARD_CN312S)
		case AVER_FLASH_IOCTL_FW_RESULT:
		    printk("----->AVER_FLASH_IOCTL_FW_RESULT\n");	
		    
		    ret = v4l2_model_ioctl_g_flash_result(file,&flash_update_result);
		    ret = copy_to_user ((char *) arg, &flash_update_result, sizeof(flash_update_result));
		    	
			break;
		case AVER_FLASH_IOCTL_FW_PMPID:
		    printk("----->AVER_FLASH_IOCTL_FW_PMPID\n");	
		    
		    ret = v4l2_model_ioctl_g_flash_pmpid(file,&flash_fpga_pmpid);
		    ret = copy_to_user ((char *) arg, &flash_fpga_pmpid, sizeof(flash_fpga_pmpid));
		    	
			break;
		case AVER_FLASH_IOCTL_FW_VERSION:
		    printk("----->AVER_FLASH_IOCTL_FW_VERSION\n");	
		    
		    ret = v4l2_model_ioctl_g_flash_version(file,&flash_fpga_version);
		    ret = copy_to_user ((char *) arg, &flash_fpga_version, sizeof(flash_fpga_version));
		    	
			break;	
		
		case AVER_FLASH_IOCTL_FW_BLOCK:
		    //printk("----->AVER_FLASH_IOCTL_FW_BLOCK\n");	
		    
		    ret = copy_from_user (&flash_access_block,(char *) arg, sizeof(flash_access_block));	
		    ret = v4l2_model_ioctl_s_flash_accessblock(file,&flash_access_block);
			break;
			
		case AVER_FLASH_IOCTL_RX_ENABLE:
		    
		    ret = copy_from_user (&rx_update_enable,(char *) arg, sizeof(rx_update_enable));	
		    printk("----->AVER_FLASH_IOCTL_RX_ENABLE(%d)\n",rx_update_enable);	
		    
		    ret = v4l2_model_ioctl_s_flash_rx(file,&rx_update_enable);
			break;	
			
		case AVER_FLASH_IOCTL_FW_DUMP:
		    //printk("----->AVER_FLASH_IOCTL_FW_DUMP\n");
		    //ret = copy_from_user (&flash_data_info,(struct v4l2_dump_flash *) arg, sizeof(struct v4l2_dump_flash));
			ret = v4l2_model_ioctl_g_flash(file,&flash_data_dump[0]);

			//flash_data_dump = &flash_data_info;
			ret = copy_to_user((unsigned char *)arg, flash_data_dump, sizeof(flash_data_dump));
			break;	
			
		case AVER_FLASH_IOCTL_FW_UPDATE:
		    //printk("----->AVER_FLASH_IOCTL_FW_UPDATE\n");
		    
		    //flash_data_info = (unsigned char *)vmalloc(8192 * 256);
		    ret = copy_from_user (&flash_data_info[0],(unsigned char *) arg, sizeof(flash_data_info));
		    
			ret = v4l2_model_ioctl_s_flash(file,&flash_data_info[0]);
			
			//vfree(flash_data_info);
			break;	
			
		case AVER_STRM_IOCTL_BUFFER_INFO:
		    //if (int_cnt <10)
		    //    printk("----->AVER_STRM_IOCTL_BUFFER_INFO(%d)\n",buffer_info);	
		    ret = v4l2_model_ioctl_g_bufferinfo(file,&buffer_info);
		    ret = copy_to_user ((int *) arg, &buffer_info, sizeof(int));
		    //printk("----->AVER_STRM_IOCTL_BUFFER_INFO(%d)\n",buffer_info);	
		    
			break;
			
		case AVER_STRM_IOCTL_LINE_NUM:
		    //if (int_cnt <10)
		   
		    ret = copy_from_user(&line_num,(int *) arg, sizeof(int));
		    ret = v4l2_model_ioctl_s_linenum(file,&line_num);
		    printk("----->AVER_STRM_IOCTL_LINE_NUM(%d)\n",line_num);	    
			break;	
			
		case AVER_STRM_IOCTL_BLOCK_NUM:
		    //if (int_cnt <10)
		    //    printk("----->AVER_STRM_IOCTL_BLOCK_NUM(%d)\n",block_num);	
		    ret = v4l2_model_ioctl_g_blocknum(file,&block_num);
		    ret = copy_to_user ((int *) arg, &block_num, sizeof(int));
		    
			break;	
			
		case AVER_STRM_IOCTL_LAST_BLOCKSIZE:
		    //if (int_cnt <10)
		    //    printk("----->AVER_STRM_IOCTL_LAST_BLOCKSIZE(%d)\n",last_blocksize);	
		    ret = v4l2_model_ioctl_g_last_blocksize(file,&last_blocksize);
		    ret = copy_to_user ((int *) arg, &last_blocksize, sizeof(int));
		    
			break;	
		#elif defined (AVER_CARD_CN312M)
		case AVER_FLASH_IOCTL_FW_DUMP:
		    printk("----->AVER_FLASH_IOCTL_FW_DUMP\n");
			ret = v4l2_model_ioctl_g_flash(file,&flash_data_info);
			ret = copy_to_user((struct v4l2_dump_flash*)arg, &flash_data_info, sizeof(struct v4l2_dump_flash));
			break;

		case AVER_FLASH_IOCTL_FW_UPDATE:
		    printk("----->AVER_FLASH_IOCTL_FW_UPDATE\n");
		    ret = copy_from_user (&flash_data_info,(struct v4l2_dump_flash *) arg, sizeof(struct v4l2_dump_flash));
			ret = v4l2_model_ioctl_s_flash(file,&flash_data_info);
			break;
	#elif defined (AVER_CARD_CM311H)
		case AVER_FLASH_IOCTL_FW_RESULT:
		    printk("----->AVER_FLASH_IOCTL_FW_RESULT\n");	
		    
		    ret = v4l2_model_ioctl_g_flash_result(file,&flash_update_result);
		    ret = copy_to_user ((char *) arg, &flash_update_result, sizeof(flash_update_result));
		    	
			break;
		case AVER_FLASH_IOCTL_FW_PMPID:
		    printk("----->AVER_FLASH_IOCTL_FW_PMPID\n");	
		    
		    ret = v4l2_model_ioctl_g_flash_pmpid(file,&flash_fpga_pmpid);
		    ret = copy_to_user ((char *) arg, &flash_fpga_pmpid, sizeof(flash_fpga_pmpid));
		    	
			break;
		case AVER_FLASH_IOCTL_FW_VERSION:
		    printk("----->AVER_FLASH_IOCTL_FW_VERSION\n");	
		    
		    ret = v4l2_model_ioctl_g_flash_version(file,&flash_fpga_version);
		    ret = copy_to_user ((char *) arg, &flash_fpga_version, sizeof(flash_fpga_version));
		    	
			break;	
		case AVER_FLASH_IOCTL_FW_BLOCK:
		    printk("----->AVER_FLASH_IOCTL_FW_BLOCK\n");	
		    
		    ret = copy_from_user (&flash_access_block,(char *) arg, sizeof(flash_access_block));	
		    ret = v4l2_model_ioctl_s_flash_accessblock(file,&flash_access_block);
			break;
		case AVER_FLASH_IOCTL_FW_DUMP:
		    printk("----->AVER_FLASH_IOCTL_FW_DUMP\n");
		    //ret = copy_from_user (&flash_data_info,(struct v4l2_dump_flash *) arg, sizeof(struct v4l2_dump_flash));
			ret = v4l2_model_ioctl_g_flash(file,&flash_data_dump[0]);

			//flash_data_dump = &flash_data_info;
			ret = copy_to_user((unsigned char *)arg, flash_data_dump, sizeof(flash_data_dump));
			break;

		case AVER_FLASH_IOCTL_FW_UPDATE:
		    printk("----->AVER_FLASH_IOCTL_FW_UPDATE\n");
		    
		    //flash_data_info = (unsigned char *)vmalloc(8192 * 256);
		    ret = copy_from_user (&flash_data_info[0],(unsigned char *) arg, sizeof(flash_data_info));
			ret = v4l2_model_ioctl_s_flash(file,&flash_data_info[0]);
			
			//vfree(flash_data_info);
			break;
		#endif
		case AVER_STRM_IOCTL_AUDIO_RATE:
		    printk("----->AVER_STRM_IOCTL_AUDIO_RATE\n");	
		    
		    ret = v4l2_model_ioctl_g_audiorate(file,&audio_rate);
		    ret = copy_to_user ((char *) arg, &audio_rate, sizeof(audio_rate));
			break;		
			
		case AVER_STRM_IOCTL_BYPASS_DISABLE:
		    printk("----->AVER_STRM_IOCTL_BYPASS_DISABLE\n");	
		    
		    ret = copy_from_user (&stream_bypass_disable,(char *) arg, sizeof(stream_bypass_disable));	
		    ret = v4l2_model_ioctl_s_bypassmode(file,&stream_bypass_disable);
			break;
			
		case AVER_STRM_IOCTL_PIXEL_YUYV:
		    printk("----->AVER_STRM_IOCTL_PIXEL_YUYV/UYVY\n");	
		    
		    ret = copy_from_user (&pixelformat,(char *) arg, sizeof(pixelformat));
		    ret = v4l2_model_ioctl_s_pixelformat(file,&pixelformat);
			break;	
			
		case AVER_STRM_IOCTL_HDCP_FLAG:
		    printk("----->AVER_STRM_IOCTL_HDCP_FLAG\n");	
		    ret = v4l2_model_ioctl_g_hdcp(file,&hdcp_flag);
		    ret = copy_to_user ((char *) arg, &hdcp_flag, sizeof(hdcp_flag));
			break;	
			
		case AVER_HDMI_IOCTL_AVI_DUMP:
		    printk("----->AVER_HDMI_IOCTL_AVI_DUMP\n");
		       
            ret = v4l2_model_ioctl_g_aviinfoframe(file,arg_tmp);
            ret = copy_to_user ((char *) arg, arg_tmp, 30);	//sizeof(arg_tmp)	

			break;	
			
		case AVER_HDMI_IOCTL_VENDOR_DUMP:
		    printk("----->AVER_HDMI_IOCTL_VENDOR_DUMP\n");
		       
            ret = v4l2_model_ioctl_g_vendorinfoframe(file,arg_tmp);
            ret = copy_to_user ((char *) arg, arg_tmp, 30);	
            break;
            
        case AVER_HDMI_IOCTL_AUDIOINFO_DUMP:
		    printk("----->AVER_HDMI_IOCTL_AUDIOINFO_DUMP\n");
		       
            ret = v4l2_model_ioctl_g_audioinfoframe(file,arg_tmp);
            ret = copy_to_user ((char *) arg, arg_tmp, 30);		    	
			break;

#if defined (AVER_CARD_CM311H)
		case AVER_SET_CUSTOM_EDID:
			printk("----->VIDIOC_SET_CUSTOM_EDID\n");
			ret = copy_from_user (&edid_custom_data,(struct v4l2_custom_edid *) arg, sizeof(struct v4l2_custom_edid));
			//ret = v4l2_model_ioctl_s_customedid(file,(struct v4l2_custom_edid *)arg);
			ret = v4l2_model_ioctl_s_customedid(file,&edid_custom_data);
			break;

		case AVER_GET_CUSTOM_EDID:
			printk("----->VIDIOC_GET_CUSTOM_EDID\n");
			ret = v4l2_model_ioctl_g_customedid(file,&edid_data_info);
			ret = copy_to_user((struct edid_buf*)arg, &edid_data_info, sizeof(struct edid_buf));
			break;

		case AVER_SET_EDID_FT_VAL:
			printk("----->AVER_SET_EDID_FT_VAL\n");
			ret = copy_from_user (&edid_ft_data,(struct edid_ft_val *) arg, sizeof(struct edid_ft_val));
			ret = v4l2_model_ioctl_s_edid_ft_val(file, &edid_ft_data);
			break;
#endif
		default:
			ret = video_ioctl2(file, cmd, arg);
			break;
	}
#if 0
	//ret = video_ioctl2(file, cmd, arg);
	hook_list_head=&v4l2m_context->ioctl_hook_list[HOOK_POST_IOCTL];
	if(!list_empty(hook_list_head))
	{	
		list_for_each(pos,hook_list_head)
		{
			v4l2_model_ioctl_hook_item_t *ioctl_hook_item;
			ioctl_hook_item=list_entry(pos,v4l2_model_ioctl_hook_item_t,list);
			try=video_usercopy(ioctl_hook_item->data,cmd,arg,(v4l2_kioctl)ioctl_hook_item->ioctl_hook_func);
			if(try!=-EINVAL)
			{
				return try;
			}	
			
		}
	}
#endif 
	return ret;
}

int v4l2_model_hook_ioctl(v4l2_model_handle_t v4l2_context,v4l2_model_ioctl_hook_type_e hooktype,v4l2_model_ioctl_hook_func_t ioctl_hook,void *data)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)v4l2_context;
	int ret=0;

	if(v4l2m_context)
	{
		struct list_head *hook_list_head=NULL;
		v4l2_model_ioctl_hook_item_t *ioctl_hook_item;
		do
		{
			if(hooktype <0 || hooktype >HOOK_IOCTL_TYPE_NUM)
			{
				ret=-1;
				break;
			}	
			ioctl_hook_item=(v4l2_model_ioctl_hook_item_t *)kzalloc(sizeof(*ioctl_hook_item), GFP_KERNEL);
			if(ioctl_hook_item==NULL)
			{
				ret=-1;
				break;
			}
			INIT_LIST_HEAD(&ioctl_hook_item->list);
			ioctl_hook_item->ioctl_hook_func=ioctl_hook;
			ioctl_hook_item->data=data;	
			hook_list_head=&v4l2m_context->ioctl_hook_list[hooktype];
			list_add_tail(&ioctl_hook_item->list,hook_list_head);

		}while(0);

	}
	return ret;

}
struct v4l2_device *v4l2_model_get_v4l2_device(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	return &v4l2m_context->v4l2_dev;
}

static void v4l2_model_notify(struct v4l2_subdev *sd,unsigned int notification, void *arg)
{
	v4l2_model_context_t *v4l2m_context=container_of(sd->v4l2_dev,v4l2_model_context_t,v4l2_dev);

	printk("%s sd name %s owner %s\n",__func__,sd->name,sd->owner->name);
	framegrabber_notify(v4l2m_context->framegrabber_handle,sd->name,notification, arg);
//	printk("%s %s notification %d \n",__func__,sd->name,notification);
//	if(notification==2)
//	{
//		struct v4l2_dv_timings detected_timings;
//		v4l2_model_output_fmt_t *input_fmt;
//		int status;
//		v4l2_subdev_call(sd,video, query_dv_timings, &detected_timings);
//		v4l2_subdev_call(sd,video, g_input_status, &status);
//		printk("%s input status %x\n",__func__,status);
//		printk("%s detected %dx%d%c\n", __func__,detected_timings.bt.width,	detected_timings.bt.height,detected_timings.bt.interlaced == V4L2_DV_INTERLACED ? 'i' : 'p');
//		v4l2_subdev_call(sd,video, s_dv_timings, &detected_timings);
//
//
//		input_fmt=&v4l2m_context->input_fmt;
//		input_fmt->width = detected_timings.bt.width;
//		input_fmt->height = detected_timings.bt.height;
//		input_fmt->is_interlace = detected_timings.bt.interlaced == V4L2_DV_INTERLACED;
//
//
//	}

}

int v4l2_model_get_current_frame_size(v4l2_model_handle_t context,struct frame_size *frame_size)
{
	v4l2_model_context_t *v4l2m_context=(v4l2_model_context_t *)context;
	framegrabber_g_input_framesize(v4l2m_context->framegrabber_handle,&frame_size->width,&frame_size->height);

	return 0;
}

//void *v4l2_model_get_context(struct sagitta_dev *sdev)
//{
//	struct sagitta_module *mod,*tmp;
//
//	list_for_each_entry_safe(mod,tmp,&sdev->root->modules,list)
//	{
//		if(strncmp(mod->name,V4L2_MODULE_ID_STR,strlen(mod->name))==0)
//		{
//			struct v4l2_module *this=(struct v4l2_module *)mod;
//
//			return this->v4l2_context;
//		}
//	}
//
//	return NULL;
//
//}

