/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * v4l2_model_table.c
 *
 *  Created on: Apr 15, 2017
 *      Author: 
 *      Version:
 * =================================================================
 */

#include <linux/kernel.h>
#include <media/videobuf2-core.h>
#include <media/v4l2-device.h>
//#include "sagitta.h"
#include "v4l2_model.h"
#include "v4l2_model_table.h"

#if defined (AVER_CARD_CN312S)
static const struct frame_size v4l2_model_frame_size[] = {
	{ .width = 640, .height = 480 },
	{ .width = 720, .height = 480 },
	{ .width = 720, .height = 576 },
	{ .width = 800, .height = 600 },
	{ .width = 1024, .height = 768 },
	{ .width = 1280, .height = 720 },
	{ .width = 1280, .height = 800 },
	{ .width = 1280, .height = 1024 },
	{ .width = 1360, .height = 768 },
	{ .width = 1440, .height = 900 },
	{ .width = 1680, .height = 1050 },
	{ .width = 1920, .height = 1080 },
};

static const v4l2_model_timing_t support_videofmt[]= {
	[V4L2_MODEL_VIDEOFORMAT_720x480I60]		= V4L2_MODEL_TIMING(720,480,60,1),
	[V4L2_MODEL_VIDEOFORMAT_720X480P60]		= V4L2_MODEL_TIMING(720,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_720X576P50]		= V4L2_MODEL_TIMING(720,480,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P25]	= V4L2_MODEL_TIMING(1280,720,25,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P30]	= V4L2_MODEL_TIMING(1280,720,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P50]	= V4L2_MODEL_TIMING(1280,720,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P60]	= V4L2_MODEL_TIMING(1280,720,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P24]	= V4L2_MODEL_TIMING(1920,1080,24,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P25]	= V4L2_MODEL_TIMING(1920,1080,25,0),
    [V4L2_MODEL_VIDEOFORMAT_1920X1080P30]	= V4L2_MODEL_TIMING(1920,1080,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P50]	= V4L2_MODEL_TIMING(1920,1080,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P60]	= V4L2_MODEL_TIMING(1920,1080,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P60]		= V4L2_MODEL_TIMING(640,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P72]		= V4L2_MODEL_TIMING(640,480,72,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P75]		= V4L2_MODEL_TIMING(640,480,75,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P85]		= V4L2_MODEL_TIMING(640,480,85,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P56]		= V4L2_MODEL_TIMING(800,600,56,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P60]		= V4L2_MODEL_TIMING(800,600,60,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P72]		= V4L2_MODEL_TIMING(800,600,72,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P75]		= V4L2_MODEL_TIMING(800,600,75,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P85]		= V4L2_MODEL_TIMING(800,600,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P60]	= V4L2_MODEL_TIMING(1024,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P70]	= V4L2_MODEL_TIMING(1024,768,70,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P75]	= V4L2_MODEL_TIMING(1024,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P85]	= V4L2_MODEL_TIMING(1024,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P60]	= V4L2_MODEL_TIMING(1280,800,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P60]	= V4L2_MODEL_TIMING(1280,1024,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P75]	= V4L2_MODEL_TIMING(1280,1024,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P60]	= V4L2_MODEL_TIMING(1360,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P60]	= V4L2_MODEL_TIMING(1440,900,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1680X1050P60]	= V4L2_MODEL_TIMING(1680,1050,60,0),
};
#elif defined (AVER_CARD_CN312M)
static const struct frame_size v4l2_model_frame_size[] = {
	{ .width = 640, .height = 480 },
	{ .width = 720, .height = 480 },
	{ .width = 720, .height = 576 },
	{ .width = 800, .height = 600 },
	{ .width = 1024, .height = 768 },
	{ .width = 1152, .height = 864 },
	{ .width = 1280, .height = 720 },
	{ .width = 1280, .height = 768 },
	{ .width = 1280, .height = 800 },
	{ .width = 1280, .height = 960 },
	{ .width = 1280, .height = 1024 },
	{ .width = 1360, .height = 768 },
	{ .width = 1366, .height = 768 },
	{ .width = 1440, .height = 900 },
	{ .width = 1600, .height = 1200 },
	{ .width = 1680, .height = 1050 },
	{ .width = 1920, .height = 1080 },
};

static const v4l2_model_timing_t support_videofmt[]= {
	[V4L2_MODEL_VIDEOFORMAT_640X480P60]	= V4L2_MODEL_TIMING(640,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P72]	= V4L2_MODEL_TIMING(640,480,72,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P75]	= V4L2_MODEL_TIMING(640,480,75,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P85]	= V4L2_MODEL_TIMING(640,480,85,0),
	[V4L2_MODEL_VIDEOFORMAT_720x480I60]	= V4L2_MODEL_TIMING(720,480,60,1),
	[V4L2_MODEL_VIDEOFORMAT_720X480P60]	= V4L2_MODEL_TIMING(720,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_720X576P50]	= V4L2_MODEL_TIMING(720,480,50,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P56]	= V4L2_MODEL_TIMING(800,600,56,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P60]	= V4L2_MODEL_TIMING(800,600,60,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P72]	= V4L2_MODEL_TIMING(800,600,72,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P75]	= V4L2_MODEL_TIMING(800,600,75,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P85]	= V4L2_MODEL_TIMING(800,600,85,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P120]	= V4L2_MODEL_TIMING(800,600,120,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P60]	= V4L2_MODEL_TIMING(1024,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P70]	= V4L2_MODEL_TIMING(1024,768,70,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P75]	= V4L2_MODEL_TIMING(1024,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P85]	= V4L2_MODEL_TIMING(1024,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P120]	= V4L2_MODEL_TIMING(1024,768,120,0),
	[V4L2_MODEL_VIDEOFORMAT_1152X864P75]	= V4L2_MODEL_TIMING(1152,864,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P50]	= V4L2_MODEL_TIMING(1280,720,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P60]	= V4L2_MODEL_TIMING(1280,720,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P100]	= V4L2_MODEL_TIMING(1280,720,100,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P120]	= V4L2_MODEL_TIMING(1280,720,120,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X768P60]	= V4L2_MODEL_TIMING(1280,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X768P75]	= V4L2_MODEL_TIMING(1280,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X768P85]	= V4L2_MODEL_TIMING(1280,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X768P120]	= V4L2_MODEL_TIMING(1280,768,120,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P60]	= V4L2_MODEL_TIMING(1280,800,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P75]	= V4L2_MODEL_TIMING(1280,800,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P85]	= V4L2_MODEL_TIMING(1280,800,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P120]	= V4L2_MODEL_TIMING(1280,800,120,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X960P60]	= V4L2_MODEL_TIMING(1280,960,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X960P85]	= V4L2_MODEL_TIMING(1280,960,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P60]	= V4L2_MODEL_TIMING(1280,1024,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P75]	= V4L2_MODEL_TIMING(1280,1024,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P85]	= V4L2_MODEL_TIMING(1280,1024,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P60]	= V4L2_MODEL_TIMING(1360,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P120]	= V4L2_MODEL_TIMING(1360,768,120,0),
	[V4L2_MODEL_VIDEOFORMAT_1366X768P60]	= V4L2_MODEL_TIMING(1366,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P60]	= V4L2_MODEL_TIMING(1440,900,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P75]	= V4L2_MODEL_TIMING(1440,900,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P85]	= V4L2_MODEL_TIMING(1440,900,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1600X1200P60]	= V4L2_MODEL_TIMING(1600,1200,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1680X1050P60]	= V4L2_MODEL_TIMING(1680,1050,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P24]	= V4L2_MODEL_TIMING(1920,1080,24,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P25]	= V4L2_MODEL_TIMING(1920,1080,25,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P30]	= V4L2_MODEL_TIMING(1920,1080,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P50]	= V4L2_MODEL_TIMING(1920,1080,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P60]	= V4L2_MODEL_TIMING(1920,1080,60,0),
};
#elif defined (AVER_CARD_CM311H)
static const struct frame_size v4l2_model_frame_size[] = {
	{ .width = 640, .height = 480 },
	{ .width = 720, .height = 480 },
	{ .width = 720, .height = 576 },
	{ .width = 800, .height = 600 },
	{ .width = 1024, .height = 768 },
	{ .width = 1280, .height = 720 },
	{ .width = 1280, .height = 800 },
	{ .width = 1280, .height = 1024 },
	{ .width = 1360, .height = 768 },
	{ .width = 1440, .height = 900 },
	{ .width = 1680, .height = 1050 },
	{ .width = 1920, .height = 1080 },
};

static const v4l2_model_timing_t support_videofmt[]= {
	[V4L2_MODEL_VIDEOFORMAT_720x480I60]		= V4L2_MODEL_TIMING(720,480,60,1),
	[V4L2_MODEL_VIDEOFORMAT_720X480P60]		= V4L2_MODEL_TIMING(720,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_720X576P50]		= V4L2_MODEL_TIMING(720,480,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P25]	= V4L2_MODEL_TIMING(1280,720,25,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P30]	= V4L2_MODEL_TIMING(1280,720,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P50]	= V4L2_MODEL_TIMING(1280,720,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P60]	= V4L2_MODEL_TIMING(1280,720,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P24]	= V4L2_MODEL_TIMING(1920,1080,24,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P25]	= V4L2_MODEL_TIMING(1920,1080,25,0),
    [V4L2_MODEL_VIDEOFORMAT_1920X1080P30]	= V4L2_MODEL_TIMING(1920,1080,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P50]	= V4L2_MODEL_TIMING(1920,1080,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P60]	= V4L2_MODEL_TIMING(1920,1080,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P60]		= V4L2_MODEL_TIMING(640,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P72]		= V4L2_MODEL_TIMING(640,480,72,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P75]		= V4L2_MODEL_TIMING(640,480,75,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P85]		= V4L2_MODEL_TIMING(640,480,85,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P56]		= V4L2_MODEL_TIMING(800,600,56,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P60]		= V4L2_MODEL_TIMING(800,600,60,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P72]		= V4L2_MODEL_TIMING(800,600,72,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P75]		= V4L2_MODEL_TIMING(800,600,75,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P85]		= V4L2_MODEL_TIMING(800,600,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P60]	= V4L2_MODEL_TIMING(1024,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P70]	= V4L2_MODEL_TIMING(1024,768,70,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P75]	= V4L2_MODEL_TIMING(1024,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P85]	= V4L2_MODEL_TIMING(1024,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P60]	= V4L2_MODEL_TIMING(1280,800,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P60]	= V4L2_MODEL_TIMING(1280,1024,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P75]	= V4L2_MODEL_TIMING(1280,1024,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P60]	= V4L2_MODEL_TIMING(1360,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P60]	= V4L2_MODEL_TIMING(1440,900,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1680X1050P60]	= V4L2_MODEL_TIMING(1680,1050,60,0),
};
#elif defined (AVER_CARD_CL311M2)
static const struct frame_size v4l2_model_frame_size[] = {
	{ .width = 640, .height = 480 },
	{ .width = 720, .height = 480 },
	{ .width = 800, .height = 600 },
	{ .width = 1024, .height = 768 },
	{ .width = 1280, .height = 720 },
	{ .width = 1280, .height = 800 },
	{ .width = 1280, .height = 1024 },
	{ .width = 1360, .height = 768 },
	{ .width = 1440, .height = 900 },
	{ .width = 1680, .height = 1050 },
	{ .width = 1920, .height = 1080 },
};

static const v4l2_model_timing_t support_videofmt[]= {
	[V4L2_MODEL_VIDEOFORMAT_720x480I60]		= V4L2_MODEL_TIMING(720,480,60,1),
	[V4L2_MODEL_VIDEOFORMAT_720X480P60]		= V4L2_MODEL_TIMING(720,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_720X576P50]		= V4L2_MODEL_TIMING(720,480,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P50]	= V4L2_MODEL_TIMING(1280,720,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P60]	= V4L2_MODEL_TIMING(1280,720,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P24]	= V4L2_MODEL_TIMING(1920,1080,24,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P25]	= V4L2_MODEL_TIMING(1920,1080,25,0),
    [V4L2_MODEL_VIDEOFORMAT_1920X1080P30]	= V4L2_MODEL_TIMING(1920,1080,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P50]	= V4L2_MODEL_TIMING(1920,1080,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P60]	= V4L2_MODEL_TIMING(1920,1080,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P60]		= V4L2_MODEL_TIMING(640,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P72]		= V4L2_MODEL_TIMING(640,480,72,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P75]		= V4L2_MODEL_TIMING(640,480,75,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P85]		= V4L2_MODEL_TIMING(640,480,85,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P56]		= V4L2_MODEL_TIMING(800,600,56,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P60]		= V4L2_MODEL_TIMING(800,600,60,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P72]		= V4L2_MODEL_TIMING(800,600,72,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P75]		= V4L2_MODEL_TIMING(800,600,75,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P85]		= V4L2_MODEL_TIMING(800,600,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P60]	= V4L2_MODEL_TIMING(1024,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P70]	= V4L2_MODEL_TIMING(1024,768,70,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P75]	= V4L2_MODEL_TIMING(1024,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P85]	= V4L2_MODEL_TIMING(1024,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P60]	= V4L2_MODEL_TIMING(1280,800,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P60]	= V4L2_MODEL_TIMING(1280,1024,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P75]	= V4L2_MODEL_TIMING(1280,1024,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P60]	= V4L2_MODEL_TIMING(1360,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P60]	= V4L2_MODEL_TIMING(1440,900,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1680X1050P60]	= V4L2_MODEL_TIMING(1680,1050,60,0),
};
#elif defined (AVER_CARD_CL311MN)
static const struct frame_size v4l2_model_frame_size[] = {
	{ .width = 640, .height = 480 },
	{ .width = 720, .height = 480 },
	{ .width = 720, .height = 576 },
	{ .width = 800, .height = 600 },
	{ .width = 1024, .height = 768 },
	{ .width = 1280, .height = 720 },
	{ .width = 1280, .height = 800 },
	{ .width = 1280, .height = 1024 },
	{ .width = 1360, .height = 768 },
	{ .width = 1440, .height = 900 },
	{ .width = 1680, .height = 1050 },
	{ .width = 1920, .height = 1080 },
};

static const v4l2_model_timing_t support_videofmt[]= {
	[V4L2_MODEL_VIDEOFORMAT_720x480I60]		= V4L2_MODEL_TIMING(720,480,60,1),
	[V4L2_MODEL_VIDEOFORMAT_720X480P60]		= V4L2_MODEL_TIMING(720,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_720X576P50]		= V4L2_MODEL_TIMING(720,480,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P25]	= V4L2_MODEL_TIMING(1280,720,25,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P30]	= V4L2_MODEL_TIMING(1280,720,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P50]	= V4L2_MODEL_TIMING(1280,720,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X720P60]	= V4L2_MODEL_TIMING(1280,720,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P24]	= V4L2_MODEL_TIMING(1920,1080,24,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P25]	= V4L2_MODEL_TIMING(1920,1080,25,0),
    [V4L2_MODEL_VIDEOFORMAT_1920X1080P30]	= V4L2_MODEL_TIMING(1920,1080,30,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P50]	= V4L2_MODEL_TIMING(1920,1080,50,0),
	[V4L2_MODEL_VIDEOFORMAT_1920X1080P60]	= V4L2_MODEL_TIMING(1920,1080,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P60]		= V4L2_MODEL_TIMING(640,480,60,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P72]		= V4L2_MODEL_TIMING(640,480,72,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P75]		= V4L2_MODEL_TIMING(640,480,75,0),
	[V4L2_MODEL_VIDEOFORMAT_640X480P85]		= V4L2_MODEL_TIMING(640,480,85,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P56]		= V4L2_MODEL_TIMING(800,600,56,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P60]		= V4L2_MODEL_TIMING(800,600,60,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P72]		= V4L2_MODEL_TIMING(800,600,72,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P75]		= V4L2_MODEL_TIMING(800,600,75,0),
	[V4L2_MODEL_VIDEOFORMAT_800X600P85]		= V4L2_MODEL_TIMING(800,600,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P60]	= V4L2_MODEL_TIMING(1024,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P70]	= V4L2_MODEL_TIMING(1024,768,70,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P75]	= V4L2_MODEL_TIMING(1024,768,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1024X768P85]	= V4L2_MODEL_TIMING(1024,768,85,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X800P60]	= V4L2_MODEL_TIMING(1280,800,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P60]	= V4L2_MODEL_TIMING(1280,1024,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1280X1024P75]	= V4L2_MODEL_TIMING(1280,1024,75,0),
	[V4L2_MODEL_VIDEOFORMAT_1360X768P60]	= V4L2_MODEL_TIMING(1360,768,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1440X900P60]	= V4L2_MODEL_TIMING(1440,900,60,0),
	[V4L2_MODEL_VIDEOFORMAT_1680X1050P60]	= V4L2_MODEL_TIMING(1680,1050,60,0),
};
#endif

struct frame_size *v4l2_model_get_frame_size(int index)
{
	if(index < 0 || index >= ARRAY_SIZE(v4l2_model_frame_size))
		return NULL;

	return (struct frame_size *)(v4l2_model_frame_size + index);
}


v4l2_model_timing_t *v4l2_model_get_support_videoformat(int index)
{
	if(index <0 ||index >V4L2_MODEL_VIDEOFORMAT_NUM)
			return NULL;

	return (v4l2_model_timing_t *)&support_videofmt[index];
}




