/*===================================================================
 * Copyright(c) AVerMedia TECHNOLOGIES, Inc. 2017
 * All rights reserved
 * =================================================================
 * v4l2_model_videobuf2.c
 *
 *  Created on: Apr 15, 2017
 *      Author: 
 *      Version:
 * =================================================================
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/mutex.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/version.h>
#include <media/v4l2-device.h>
#include <media/videobuf2-vmalloc.h>
#include <media/videobuf2-dma-sg.h>
#include <media/videobuf2-dma-contig.h>
#include <media/v4l2-common.h>
#include <linux/videodev2.h>
#include <linux/version.h>

#include "queue.h"
#include "typedef.h"
#include "sys.h"
#include "v4l2_model.h"
#include "v4l2_model_device.h"
#include "v4l2_model_videobuf2.h"
//#include "images.h"
#include "pic_bmp.h"

//#include <asm/fpu/api.h>


//static void v4l2_model_setup_buffer_cb_context(struct vb2_buffer *vb,v4l2_model_buffer_info_t *buffer_cb_context);

typedef enum _v4l2_model_vb2_image_type_e
{
    V4L2_MODEL_VB2_IMAGE_NOT_LOADED = 0,
    V4L2_MODEL_VB2_IMAGE_NO_SIGNAL,
    V4L2_MODEL_VB2_IMAGE_OUT_OF_RANGE,
    V4L2_MODEL_VB2_IMAGE_COPY_PROTECTION,
} v4l2_model_vb2_image_type_e;

typedef struct {
	struct mutex qlock;
	struct list_head buffer_list;
	u32 frames;
	v4l2_model_buffer_type_e queue_type;
	void *alloc_ctx;
	spinlock_t queuelock;
	u8 *buffer_interlace;
	u8 *buffer_deinterlace;
	u32 buff_cnt;
	u32 enable_notification_image;
	//u8 *image_no_signal;
	//u8 *image_hdcp_protection;
	u8 *image_data;
	v4l2_model_vb2_image_type_e image_type;
} v4l2_model_vb2_context_t;

typedef struct {
	/* common v4l buffer stuff -- must be first */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,4,0)
	struct vb2_v4l2_buffer vb;
	//struct vb2_v4l2_buffer linevb;
#else
	struct vb2_buffer vb;
#endif
	struct list_head list;

	unsigned int length;
	unsigned int bytesused;

	u32 pts;

	v4l2_model_buffer_info_t buffer_info;
	v4l2_model_buf_desc_t *buf_descs[VIDEO_MAX_PLANES];

} v4l2_model_vb2_buffer_t;

int print_cnt=0;

//extern int int_cnt;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,8,0)
static int v4l2_model_qops_queue_setup(struct vb2_queue *q,
                                       unsigned int *num_buffers, unsigned int *num_planes,
                                       unsigned int sizes[], struct device *alloc_ctxs[])
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4,5,0)
static int v4l2_model_qops_queue_setup(struct vb2_queue *q,
				       unsigned int *num_buffers,
				       unsigned int *num_planes,
				       unsigned int sizes[], void *alloc_ctxs[])
#elif LINUX_VERSION_CODE>=KERNEL_VERSION(4,4,0)
static int v4l2_model_qops_queue_setup(struct vb2_queue *q, const void *parg,
				       unsigned int *num_buffers,
				       unsigned int *num_planes,
				       unsigned int sizes[], void *alloc_ctxs[])
#else

static int v4l2_model_qops_queue_setup(struct vb2_queue *q,
				       const struct v4l2_format *fmt,
				       unsigned int *num_buffers,
				       unsigned int *num_planes,
				       unsigned int sizes[], void *alloc_ctxs[])
#endif
{
	v4l2_model_context_t *v4l2m_context = vb2q_to_v4l2context(q);
	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_QUEUE_SETUP];
//    const framegrabber_pixfmt_t *pixfmt;
	unsigned framebufsize;
	int width, height;
	unsigned bytesperline;

//    pixfmt = framegrabber_g_out_pixelfmt(v4l2m_context->framegrabber_handle);
//    if (width == 0 || height == 0)
//    {
//        return -EINVAL;
//    }
	framebufsize =
	    framegrabber_g_max_framebuffersize(v4l2m_context->
					       framegrabber_handle);
	framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);	//test
	printk("%s framebufsize %u, width %d, height %d\n", __func__,
	       framebufsize, width, height);
	if (1) {
	    if (v4l2m_context->pic_bmp_handle)
	        {
		        if (vb2_context->image_data)
		        {
			        vfree(vb2_context->image_data);
			        vb2_context->image_data = NULL;
		        }
		        vb2_context->image_data = vmalloc(framebufsize);

		        pic_bmp_prepare(v4l2m_context->pic_bmp_handle, 640, 480, width, height);
		        load_no_signal_image(v4l2m_context->pic_bmp_handle, vb2_context->image_data, width, height);
		        vb2_context->image_type = V4L2_MODEL_VB2_IMAGE_NO_SIGNAL;
	        }
	}
	       
	if (width ==0)
	{
		width = 1920;
		height = 1080;
	}       
	       
	//if (framebufsize == 0)
	{
		//int width, height;
		//unsigned bytesperline;

		//framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle,
		//			     &width, &height);
		bytesperline =
		    framegrabber_g_out_bytesperline(v4l2m_context->
						    framegrabber_handle);
		framebufsize = bytesperline * height;
		printk("%s >>> framebufsize %u, width %d, height %d\n",
		       __func__, framebufsize, width, height);
	}

	sizes[0] = framebufsize;
	*num_planes = 1;
	if (vb2_context->alloc_ctx)
		alloc_ctxs[0] = vb2_context->alloc_ctx;

	if (callback_item->callback) {
		callback_item->data.u.queue_setup_info.buffer_num =
		    *num_buffers;
		callback_item->callback(&callback_item->data);
	}
	vb2_context->buff_cnt=0;
	printk("%s >>>>>\n", __func__);
	return 0;
}

static void v4l2_model_qops_wait_prepare(struct vb2_queue *q)
{
	return vb2_ops_wait_prepare(q);
}

static void v4l2_model_qops_wait_finish(struct vb2_queue *q)
{
	return vb2_ops_wait_finish(q);
}

//invoked when each new buffer after it has been allocated

static int v4l2_model_qops_buf_init(struct vb2_buffer *vb)
{
	v4l2_model_context_t *v4l2m_context = vb2b_to_v4l2context(vb);
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_BUFFER_INIT];
	
	v4l2m_context->inibuffer_create++;
	
	//printk("==%s vb %p ; buffer create++=%d\n", __func__, vb, v4l2m_context->inibuffer_create);
	
	if (callback_item->callback) {
		v4l2_model_callback_parameter_t *parm = &callback_item->data;
		callback_item->callback(parm);
	}
        v4l2m_context->inibuffer_index =v4l2m_context->inibuffer_create;
	return 0;
}

//invoked when user space queues the buffer

static int v4l2_model_qops_buf_prepare(struct vb2_buffer *vb)
{
	int i;
	v4l2_model_context_t *v4l2m_context = vb2b_to_v4l2context(vb);
	v4l2_model_vb2_context_t *vb2_cxt = v4l2m_context->vb2_context;
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_BUFFER_PREPARE];
#if LINUX_VERSION_CODE >=KERNEL_VERSION(4,4,0)
	v4l2_model_vb2_buffer_t *v4l2_buf =
	    container_of(to_vb2_v4l2_buffer(vb), v4l2_model_vb2_buffer_t, vb);
#else
	v4l2_model_vb2_buffer_t *v4l2_buf =
	    container_of(vb, v4l2_model_vb2_buffer_t, vb);
#endif
	const framegrabber_pixfmt_t *pixfmt;
	int width, height;
	framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width,
				     &height);
				     
	pixfmt =
	    framegrabber_g_out_pixelfmt(v4l2m_context->framegrabber_handle);

	v4l2_buf->buffer_info.planes = vb->num_planes;
	v4l2_buf->buffer_info.buf_type = vb2_cxt->queue_type;
	for (i = 0; i < vb->num_planes; i++) {
		v4l2_model_buffer_info_t *buffer_info = &v4l2_buf->buffer_info;
		switch (vb2_cxt->queue_type) {
		case V4L2_MODEL_BUF_TYPE_VMALLOC:
			buffer_info->buf_count[i] = 1;
			v4l2_buf->buf_descs[i] =
			    kzalloc(sizeof(v4l2_model_buf_desc_t) *
				    buffer_info->buf_count[i], GFP_KERNEL);
			if (v4l2_buf->buf_descs[i]) {
				v4l2_model_buf_desc_t *buf_descs =
				    v4l2_buf->buf_descs[i];
				buf_descs[0].addr =
				    (unsigned long)vb2_plane_vaddr(vb, i);
				buf_descs[0].size = vb2_plane_size(vb, i);
			}
			buffer_info->buf_info[i] = v4l2_buf->buf_descs[i];
			break;
		case V4L2_MODEL_BUF_TYPE_DMA_SG:
			{

				struct sg_table *sgt =
				    vb2_dma_sg_plane_desc(vb, i);
				struct scatterlist *sg;
				int j;

				buffer_info->buf_count[i] = sgt->nents;
				
				if (v4l2m_context->inibuffer_index != 0)
				{
					//printk("%s >>>>>vb %p v4l2_buf %p\n",__func__,vb,v4l2_buf);
					v4l2_buf->buf_descs[i] =
				    kzalloc(sizeof(v4l2_model_buf_desc_t) *
					    buffer_info->buf_count[i],GFP_KERNEL);
                                    v4l2m_context->inibuffer_index--;
				}
				if (v4l2_buf->buf_descs[i]) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,19,0)
					if (!dma_map_sg
					    (v4l2m_context->dev, sgt->sgl,
					     sgt->nents, DMA_FROM_DEVICE))
						printk("error dma_map_sg\n");
#endif

					for_each_sg(sgt->sgl, sg, sgt->nents, j) {
						v4l2_model_buf_desc_t *buf_descs
						    = v4l2_buf->buf_descs[i];

						buf_descs[j].addr =
						    sg_dma_address(sg);
						buf_descs[j].size =
						    sg_dma_len(sg);
					}
					buffer_info->buf_info[i] =
					    v4l2_buf->buf_descs[i];
				}
			}
			break;
		case V4L2_MODEL_BUF_TYPE_DMA_CONT:
			v4l2_buf->buffer_info.buf_count[i] = 1;
			v4l2_buf->buf_descs[i] =
			    kzalloc(sizeof(v4l2_model_buf_desc_t) *
				    buffer_info->buf_count[i], GFP_KERNEL);
			if (v4l2_buf->buf_descs[i]) {
				v4l2_model_buf_desc_t *buf_descs =
				    v4l2_buf->buf_descs[i];
				buf_descs[0].addr =
				    vb2_dma_contig_plane_dma_addr(vb, i);
				buf_descs[0].size = vb2_plane_size(vb, i);
			}
			buffer_info->buf_info[i] = v4l2_buf->buf_descs[i];

			break;
		}
		/* set vb2 plane bytesused size */
		vb2_set_plane_payload(vb, i,
				      width * height * (pixfmt->depth >> 3)); //rrr
		v4l2_buf->length += vb2_plane_size(vb, i);

	}

	if (callback_item->callback) {
		v4l2_model_callback_parameter_t *parm = &callback_item->data;
		parm->u.buffer_prepare_info.buffer_info =
		    &v4l2_buf->buffer_info;
		callback_item->callback(parm);
	}

	return 0;
}

static void v4l2_model_qops_buf_cleanup(struct vb2_buffer *vb)
{
	v4l2_model_context_t *v4l2m_context = vb2b_to_v4l2context(vb);
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_BUFFER_CLEANUP];
#if LINUX_VERSION_CODE >=KERNEL_VERSION(4,4,0)
	v4l2_model_vb2_buffer_t *v4l2_buf =
	    container_of(to_vb2_v4l2_buffer(vb), v4l2_model_vb2_buffer_t, vb);
#else
	v4l2_model_vb2_buffer_t *v4l2_buf =
	    container_of(vb, v4l2_model_vb2_buffer_t, vb);
#endif
	int i;
	printk("=====================%s vb %p \n", __func__, vb);

	for (i = 0; i < vb->num_planes; i++) {
		switch (v4l2_buf->buffer_info.buf_type) {
		case V4L2_MODEL_BUF_TYPE_VMALLOC:
			break;
		case V4L2_MODEL_BUF_TYPE_DMA_SG:
			{
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,19,0)
				struct sg_table *sgt =
				    vb2_dma_sg_plane_desc(vb, i);

				dma_unmap_sg(v4l2m_context->dev, sgt->sgl,
					     sgt->nents, DMA_FROM_DEVICE);

#endif
			}
			break;
		case V4L2_MODEL_BUF_TYPE_DMA_CONT:
			break;
		}
		kfree(v4l2_buf->buf_descs[i]);
		v4l2_buf->buf_descs[i] = NULL;
	}

	if (callback_item->callback) {
		callback_item->callback(&callback_item->data);
	}
}

// invoked just before the buffer is passed back to user space (process context)
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,15,0)
static int v4l2_model_qops_buf_finish(struct vb2_buffer *vb)
#else

static void v4l2_model_qops_buf_finish(struct vb2_buffer *vb)
#endif
{
	v4l2_model_context_t *v4l2m_context = vb2b_to_v4l2context(vb);
	//v4l2_model_vb2_context_t *vb2_context=(v4l2_model_vb2_context_t *)v4l2m_context->vb2_context;
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_BUFFER_FINISH];
	//printk("%s vb %p \n", __func__, vb);

	if (callback_item->callback) {
		callback_item->callback(&callback_item->data);
	}
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,15,0)
	return 0;
#else
	return;
#endif
}

static int v4l2_model_qops_start_streaming(struct vb2_queue *q,
					   unsigned int count)
{
	v4l2_model_context_t *v4l2m_context = vb2q_to_v4l2context(q);
	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_STREAMON];
	printk("%s...\n", __func__);
	if (callback_item->callback) {
		callback_item->callback(&callback_item->data);
	}
	vb2_context->frames = 0;
	framegrabber_start_streaming(v4l2m_context->framegrabber_handle, AVER_TRUE);
	//v4l2m_context->current_framebuf_info.rcv_size=0;
	v4l2_model_reset_video_data(v4l2m_context);

        v4l2m_context->inibuffer_index =v4l2m_context->inibuffer_create;

	return 0;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,16,0)
static int v4l2_model_qops_stop_streaming(struct vb2_queue *q)
#else

static void v4l2_model_qops_stop_streaming(struct vb2_queue *q)
#endif
{
	v4l2_model_context_t *v4l2m_context = vb2q_to_v4l2context(q);
	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_STREAMOFF];
	//v4l2_model_vb2_buffer_t *buf,*next_buf;

	if (callback_item->callback) {
		callback_item->callback(&callback_item->data);
	}
	if (list_empty(&vb2_context->buffer_list)) {
		printk("%s vb2_context->buffer_list empty\n", __func__);
	} else {
		v4l2_model_queue_cancel(v4l2m_context);
		
		printk("%s...\n", __func__);
	}
	//  v4l2m_context->current_framebuf_info.vbuf_info=NULL;
	//  v4l2m_context->current_framebuf_info.rcv_size=0;

	v4l2_model_reset_video_data(v4l2m_context);	

	framegrabber_start_streaming(v4l2m_context->framegrabber_handle, AVER_FALSE);
	//framegrabber_s_out_framesize(v4l2m_context->framegrabber_handle, 0, 0);
    v4l2m_context->inibuffer_index=0;
	v4l2m_context->inibuffer_create=0;
	print_cnt=0;
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,16,0)
	return 0;
#endif
}

//invoked when pass actual ownership of the buffer to the driver

static void v4l2_model_qops_buf_queue(struct vb2_buffer *vb)
{
	unsigned long flags = 0;

	v4l2_model_context_t *v4l2m_context = vb2b_to_v4l2context(vb);
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_BUFFER_QUEUE];
#if LINUX_VERSION_CODE >=KERNEL_VERSION(4,4,0)
	v4l2_model_vb2_buffer_t *v4l2_model_buffer =
	    container_of(to_vb2_v4l2_buffer(vb), v4l2_model_vb2_buffer_t, vb);
#else
	v4l2_model_vb2_buffer_t *v4l2_model_buffer =
	    container_of(vb, v4l2_model_vb2_buffer_t, vb);
#endif
	v4l2_model_vb2_context_t *vb2_context = v4l2m_context->vb2_context;

	//printk("%s vb %p\n",__func__,vb);
	spin_lock_irqsave(&vb2_context->queuelock, flags);
	list_add_tail(&v4l2_model_buffer->list, &vb2_context->buffer_list);
	spin_unlock_irqrestore(&vb2_context->queuelock, flags);

	if (callback_item->callback) {
		callback_item->callback(&callback_item->data);
	}
}

const struct vb2_ops v4l2_model_qops = {
	.queue_setup = v4l2_model_qops_queue_setup,

	/* do nothing */
	.wait_prepare = v4l2_model_qops_wait_prepare,
	.wait_finish = v4l2_model_qops_wait_finish,
	.buf_init = v4l2_model_qops_buf_init,

	.buf_prepare = v4l2_model_qops_buf_prepare,
	.buf_cleanup = v4l2_model_qops_buf_cleanup,
	.buf_finish = v4l2_model_qops_buf_finish,
	.start_streaming = v4l2_model_qops_start_streaming,
	.stop_streaming = v4l2_model_qops_stop_streaming,
	.buf_queue = v4l2_model_qops_buf_queue,
};

void *v4l2_model_vb2_init(struct vb2_queue *q, v4l2_model_devicetype_t dev_type,
			  v4l2_model_buffer_type_e queue_type,
			  struct device *dev, void *priv)
{
	int rc;
	v4l2_model_vb2_context_t *vb2_context = NULL;

	if (dev_type == DEVICE_TYPE_VBI)
		q->type = V4L2_BUF_TYPE_VBI_CAPTURE;
	else
		q->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	vb2_context = kzalloc(sizeof(v4l2_model_vb2_context_t), GFP_KERNEL);
	if (!vb2_context)
		return NULL;
		
	#if defined (AVER_CARD_CN312S) || defined (AVER_CARD_CN312M)
	v4l2_model_context_t *v4l2m_context = vb2q_to_v4l2context(q);
	unsigned framebufsize;
	framebufsize=framegrabber_g_max_framebuffersize(v4l2m_context->framegrabber_handle);
	if (framebufsize == 0) 
	{
		int width, height;
		unsigned bytesperline;
		
		framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);
		bytesperline = framegrabber_g_out_bytesperline(v4l2m_context->framegrabber_handle);
		framebufsize = bytesperline * height;
		
		//if (framebufsize ==0)
        //    framebufsize = 1920 * 1080 *2;
	}
	
    vb2_context->buffer_deinterlace = kmalloc(framebufsize, GFP_KERNEL);
    vb2_context->buffer_interlace = kmalloc(framebufsize, GFP_KERNEL);
    
    if (!vb2_context->buffer_deinterlace)
    {
		printk(".................vb2_dma buffer_deinterlace fail\n");
        kfree(vb2_context->buffer_deinterlace);
		return NULL;
	}
	
	if (!vb2_context->buffer_interlace)
    {
		printk(".................vb2_dma buffer_interlace fail\n");
        kfree(vb2_context->buffer_interlace);
		return NULL;
	}
	#endif
       
	INIT_LIST_HEAD(&vb2_context->buffer_list);
	mutex_init(&vb2_context->qlock);
	spin_lock_init(&vb2_context->queuelock);
	vb2_context->queue_type = queue_type;
	vb2_context->enable_notification_image = 0;		//chenhaiwei

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,15,0)
	q->timestamp_flags = V4L2_BUF_FLAG_TIMESTAMP_MONOTONIC;
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
	q->timestamp_type = V4L2_BUF_FLAG_TIMESTAMP_MONOTONIC;
#endif
	q->drv_priv = priv;
	q->buf_struct_size = sizeof(v4l2_model_vb2_buffer_t);
	q->ops = &v4l2_model_qops;
	q->lock = &vb2_context->qlock;
	switch (queue_type) {
	case V4L2_MODEL_BUF_TYPE_VMALLOC:
		q->mem_ops = &vb2_vmalloc_memops;
		q->io_modes = VB2_READ | VB2_MMAP | VB2_USERPTR;
		break;
	case V4L2_MODEL_BUF_TYPE_DMA_CONT:
		q->mem_ops = &vb2_dma_contig_memops;
		q->io_modes = VB2_READ | VB2_MMAP | VB2_DMABUF;
		q->gfp_flags = GFP_DMA32;
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,8,0)
        vb2_context->alloc_ctx = vb2_dma_contig_init_ctx(dev);
        if (!vb2_context->alloc_ctx)
        {
            printk("vb2_dma_contig_init_ctx fail\n");
            kfree(vb2_context);
            return NULL;
        }
#else
        vb2_context->alloc_ctx = dev;
#endif
		break;
	case V4L2_MODEL_BUF_TYPE_DMA_SG:
		q->mem_ops = &vb2_dma_sg_memops;
		q->io_modes = VB2_READ | VB2_USERPTR | VB2_MMAP | VB2_DMABUF;
		q->gfp_flags = GFP_DMA32;
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,8,0)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)
        vb2_context->alloc_ctx = vb2_dma_sg_init_ctx(dev);
        if (!vb2_context->alloc_ctx)
        { 
            printk("vb2_dma_sg_init_ctx fail\n");
            kfree(vb2_context);
            return NULL;
        }
#endif
#else
        vb2_context->alloc_ctx = dev;
#endif
		break;
	}
    vb2_context->image_data = NULL;
    vb2_context->image_type = V4L2_MODEL_VB2_IMAGE_NOT_LOADED;
	rc = vb2_queue_init(q);
	if (rc) {
		printk("%s vb2 queue init failed\n", __func__);
		kfree(vb2_context);
		return NULL;
	}

	return (void *)vb2_context;
}

void v4l2_model_vb2_release(void *context)
{
    v4l2_model_vb2_context_t *vb2_context = (v4l2_model_vb2_context_t *) context;

    if (vb2_context)
    {
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,8,0)
        if (vb2_context->alloc_ctx)
        {
            switch (vb2_context->queue_type)
            {
            case V4L2_MODEL_BUF_TYPE_VMALLOC:
                printk("%s vmalloc should not have alloc_cxt \n", __func__);
                break;
            case V4L2_MODEL_BUF_TYPE_DMA_CONT:
                vb2_dma_contig_cleanup_ctx(vb2_context->alloc_ctx);
                break;
            case V4L2_MODEL_BUF_TYPE_DMA_SG:
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)                                
                vb2_dma_sg_cleanup_ctx(vb2_context->alloc_ctx);
#endif                    
                break;
            }

        }
#endif
		#if defined (AVER_CARD_CN312S) || defined (AVER_CARD_CN312M)
        if (vb2_context->buffer_deinterlace)
            kfree(vb2_context->buffer_deinterlace);
        if (vb2_context->buffer_interlace)    
            kfree(vb2_context->buffer_interlace);
		#endif
        #if 0
        if (vb2_context->enable_notification_image) {
            vfree(vb2_context->image_hdcp_protection);
            vfree(vb2_context->image_no_signal);
        }
        #endif
        if (1) {
             if (vb2_context->image_data) vfree(vb2_context->image_data);
		 }
        kfree(vb2_context);
        printk("%s ... \n",__func__);
    }
}

void v4l2_model_queue_cancel(void *context)
{
	unsigned long flags;
	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;
	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_vb2_buffer_t *buf;
	framegrabber_status_bitmask_e framegrabber_status;
	printk("%s\n", __func__);
	spin_lock_irqsave(&vb2_context->queuelock, flags);
	while (!list_empty(&vb2_context->buffer_list)) {
		struct vb2_buffer *vb;

		buf =
		    list_first_entry(&vb2_context->buffer_list,
				     v4l2_model_vb2_buffer_t, list);
#if LINUX_VERSION_CODE >=KERNEL_VERSION(4,4,0)
		vb = &buf->vb.vb2_buf;
#else
		vb = &buf->vb;
#endif
		//printk("%s vb %p done\n",__func__,vb);
		list_del_init(&buf->list);
		framegrabber_status =
		    framegrabber_g_status(v4l2m_context->framegrabber_handle);

		if ((framegrabber_status &
		     (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
		      FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) ==
		    (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
		     FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) {

			vb2_buffer_done(vb, VB2_BUF_STATE_DONE);
		} else {
			printk("%s framegrabber_status %08x\n", __func__,
			       framegrabber_status);
			vb2_buffer_done(vb, VB2_BUF_STATE_ERROR);
		}

	}
	spin_unlock_irqrestore(&vb2_context->queuelock, flags);
	printk("%s done\n", __func__);
}

void v4l2_model_next_buffer(v4l2_model_handle_t context,
			    v4l2_model_buffer_info_t ** buffer_info)
{
	//struct vb2_buffer *vb;
	v4l2_model_vb2_buffer_t *next_buffer;
	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;
	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	unsigned long flags;

	spin_lock_irqsave(&vb2_context->queuelock, flags);
	if (list_empty(&vb2_context->buffer_list)) {
		//printk("%s vb2 queue is empty\n", __func__);
		*buffer_info = NULL;
		spin_unlock_irqrestore(&vb2_context->queuelock, flags);
		return;
	}

	next_buffer =
	    list_first_entry(&vb2_context->buffer_list, v4l2_model_vb2_buffer_t,
			     list);
	spin_unlock_irqrestore(&vb2_context->queuelock, flags);

	
	//if (int_cnt<50)
	//    printk("%s next_buffer %p\n",__func__,next_buffer);
	*buffer_info = &next_buffer->buffer_info;
}

#if defined (AVER_CARD_CN312S)
void v4l2_model_buffer_done(v4l2_model_handle_t context,int buffer_modes,int buffer_index, int line_buffer_index, int line_buffer_size)
{
	unsigned long flags;
	struct vb2_buffer *vb,*linevb;

	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;
	
	v4l2_model_callback_item_t *callback_item =
	    &v4l2m_context->callbacks[V4L2_MODEL_CALLBACK_BUFFER_PREPARE];

	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_vb2_buffer_t *buf,*linebuf;//
	framegrabber_status_bitmask_e framegrabber_status;
	int width, height;
	framegrabber_framemode_e framemode;
	int buffer_index_info;
	
	if (/*(line_buffer_index !=0) &&*/ (buffer_modes ==1))
	{
		buffer_index_info = line_buffer_size << 16 | buffer_index << 12 | line_buffer_index; 
		//printk("\n..buffer info=%d\n",1000*buffer_index + line_buffer_index);
	    //framegrabber_s_bufferinfo(v4l2m_context->framegrabber_handle, 1000*buffer_index + line_buffer_index);  
	    framegrabber_s_bufferinfo(v4l2m_context->framegrabber_handle, buffer_index_info);  
	    return; 
	}
	
	spin_lock_irqsave(&vb2_context->queuelock, flags);
	if (!list_empty(&vb2_context->buffer_list)) {
		buf = list_first_entry(&vb2_context->buffer_list,
				       v4l2_model_vb2_buffer_t, list);	       
		list_del_init(&buf->list);
	} else {
		//printk("......%s pre buffer to serve(%p)\n", __func__,&buf->vb);	
		buf = NULL;
	}
	spin_unlock_irqrestore(&vb2_context->queuelock, flags);

	if (!buf) {
		return;
	}
    

#if   LINUX_VERSION_CODE >= KERNEL_VERSION(4,5,0)
	vb = &buf->vb.vb2_buf;
	buf->vb.field = V4L2_FIELD_NONE;
	buf->vb.sequence = vb2_context->frames++;
	vb->timestamp = ktime_get_ns();
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4,4,0)
	vb = &buf->vb.vb2_buf;
	buf->vb.field = V4L2_FIELD_NONE;
	buf->vb.sequence = vb2_context->frames++;
	v4l2_get_timestamp(&buf->vb.timestamp);
#else
	vb = &buf->vb;
	vb->v4l2_buf.field = V4L2_FIELD_NONE;
	vb->v4l2_buf.sequence = vb2_context->frames++;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
	v4l2_get_timestamp(&vb->v4l2_buf.timestamp);
#else
	{
		struct timespec ts;
		ktime_get_ts(&ts);
		vb->v4l2_buf.timestamp.tv_sec = ts.tv_sec;
		vb->v4l2_buf.timestamp.tv_usec = ts.tv_nsec / NSEC_PER_USEC;
	}
#endif
#endif

#ifdef CONFIG_DEINTERLACE_ENABLE
{
    u32 input_status;
    input_status = framegrabber_g_input_status(v4l2m_context->framegrabber_handle);

    if (input_status == FRAMEGRABBER_INPUT_STATUS_OK)
    { 

        print_cnt++;
    
        framemode = framegrabber_g_input_framemode(v4l2m_context->framegrabber_handle);

        if (framemode == FRAMEGRABBER_FRAMEMODE_INTERLACED)
        {
		    framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);
		
		    if (width ==0)
		    {
			    width =1920;
			    height = 1080;
		    }
            if (print_cnt ==1)
		        printk("...software deinterlace w:%d h:%d\n",width,height);
	        {
			    u8 *ptr_in;
			    u8 *ptr_out;
			    struct sg_table *sgt = vb2_dma_sg_plane_desc(vb, 0);
			    
			
			    ptr_in = vb2_context->buffer_interlace;
			    ptr_out = vb2_context->buffer_deinterlace;
			    
			    sg_copy_to_buffer(sgt->sgl, sgt->nents, ptr_in, width * height * 2);
			    //printk("...software deinterlace w:%d h:%d\n",width,height);
                //mmcpy_sse(ptr_out, ptr_in, width, height, width * 2);
                
                #if 1 //reorder the line data
                {
                    int h;
                    for (h=0; h<height/2; h++)
                    {	
	                    memcpy(ptr_out + width*2*h*2, ptr_in+width*2*h, width * 2); //odd field 
                    }

                    for (h=height/2; h<height; h++)
                    {	
	                    memcpy(ptr_out + width*2*(h-height/2)*2 + width*2, ptr_in+width*2*h, width * 2); //even field	
                    }
			    }
                #endif
                
                
                sg_copy_from_buffer(sgt->sgl, sgt->nents, ptr_out, width * height * 2);
           }
        }
    }
}
#endif

	framegrabber_status =
	    framegrabber_g_status(v4l2m_context->framegrabber_handle);

	if ((framegrabber_status &
	     (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
	      FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) ==
	    (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
	     FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) {
		vb2_buffer_done(vb, VB2_BUF_STATE_DONE);
		
		if (vb2_context->buff_cnt==0)
		{
			printk("%s=========================\n", __func__);
			vb2_context->buff_cnt =1;
		}
		
		
	} else {
		//printk("%s framegrabber_status %08x\n", __func__, framegrabber_status);
		vb2_buffer_done(vb, VB2_BUF_STATE_ERROR);
	}
}
#else
void v4l2_model_buffer_done(v4l2_model_handle_t context)
{
	unsigned long flags;
	struct vb2_buffer *vb;

	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;

	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_vb2_buffer_t *buf;//
	framegrabber_status_bitmask_e framegrabber_status;
	
	//const framegrabber_pixfmt_t *pixfmt=framegrabber_g_out_pixelfmt(v4l2m_context->framegrabber_handle);
	//framegrabber_g_input_framesize(v4l2m_context->framegrabber_handle,
	//			       &width, &height);

	/* get next v4l2_model_buffer from vb2 context buffer_list */
	spin_lock_irqsave(&vb2_context->queuelock, flags);
	if (!list_empty(&vb2_context->buffer_list)) {
		buf = list_first_entry(&vb2_context->buffer_list,
				       v4l2_model_vb2_buffer_t, list);
		list_del_init(&buf->list);
	} else {
		buf = NULL;
	}
	spin_unlock_irqrestore(&vb2_context->queuelock, flags);

	if (!buf) {
		printk("%s no buffer to serve\n", __func__);
		return;
	}
#if   LINUX_VERSION_CODE >= KERNEL_VERSION(4,5,0)
	vb = &buf->vb.vb2_buf;
	buf->vb.field = V4L2_FIELD_NONE;
	buf->vb.sequence = vb2_context->frames++;
	vb->timestamp = ktime_get_ns();
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4,4,0)
	vb = &buf->vb.vb2_buf;
	buf->vb.field = V4L2_FIELD_NONE;
	buf->vb.sequence = vb2_context->frames++;
	v4l2_get_timestamp(&buf->vb.timestamp);
#else
	vb = &buf->vb;
	vb->v4l2_buf.field = V4L2_FIELD_NONE;
	vb->v4l2_buf.sequence = vb2_context->frames++;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
	v4l2_get_timestamp(&vb->v4l2_buf.timestamp);
#else
	{
		struct timespec ts;
		ktime_get_ts(&ts);
		vb->v4l2_buf.timestamp.tv_sec = ts.tv_sec;
		vb->v4l2_buf.timestamp.tv_usec = ts.tv_nsec / NSEC_PER_USEC;
	}
#endif
#endif

#ifdef CONFIG_DEINTERLACE_ENABLE
{
	int width, height;
	framegrabber_framemode_e framemode;
    u32 input_status;
    input_status = framegrabber_g_input_status(v4l2m_context->framegrabber_handle);

    if (input_status == FRAMEGRABBER_INPUT_STATUS_OK)
    { 

        print_cnt++;
    
        framemode = framegrabber_g_input_framemode(v4l2m_context->framegrabber_handle);

        if (framemode == FRAMEGRABBER_FRAMEMODE_INTERLACED)
        {
		    framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);
		
		    if (width ==0)
		    {
			    width =1920;
			    height = 1080;
		    }
            if (print_cnt ==1)
		        printk("...software deinterlace w:%d h:%d\n",width,height);
	        {
			    u8 *ptr_in;
			    u8 *ptr_out;
			    struct sg_table *sgt = vb2_dma_sg_plane_desc(vb, 0);
			    
			
			    ptr_in = vb2_context->buffer_interlace;
			    ptr_out = vb2_context->buffer_deinterlace;
			    
			    sg_copy_to_buffer(sgt->sgl, sgt->nents, ptr_in, width * height * 2);
			    //printk("...software deinterlace w:%d h:%d\n",width,height);
                //mmcpy_sse(ptr_out, ptr_in, width, height, width * 2);
                
                #if 1 //reorder the line data
                {
                    int h;
                    for (h=0; h<height/2; h++)
                    {	
	                    memcpy(ptr_out + width*2*h*2, ptr_in+width*2*h, width * 2); //odd field 
                    }

                    for (h=height/2; h<height; h++)
                    {	
	                    memcpy(ptr_out + width*2*(h-height/2)*2 + width*2, ptr_in+width*2*h, width * 2); //even field	
                    }
			    }
                #endif
                
                
                sg_copy_from_buffer(sgt->sgl, sgt->nents, ptr_out, width * height * 2);
           }
        }
    }
}
#endif
#if 0
	if (vb2_context->enable_notification_image) {
		u32 input_status;
		int width, height;
		framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);
		//printk("%s %d %d\n",__func__,width,height);
		input_status = framegrabber_g_input_status(v4l2m_context->framegrabber_handle);
		//printk("%s input_status: 0x%x\n",__func__,input_status);
		if (input_status != FRAMEGRABBER_INPUT_STATUS_OK_BIT) {
			u8 *ptr;
			if (input_status & FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT) {
				ptr = vb2_context->image_no_signal;
				printk("%s no signal: 0x%x 0x%x\n",__func__,input_status, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			} else {
				ptr = vb2_context->image_hdcp_protection;
				printk("%s hdcp: 0x%x 0x%x\n",__func__, input_status, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			}
			//printk(".\n");
			switch (vb2_context->queue_type)
			{
				case V4L2_MODEL_BUF_TYPE_VMALLOC:
					break;
				case V4L2_MODEL_BUF_TYPE_DMA_SG:
				{
					struct sg_table *sgt = vb2_dma_sg_plane_desc(vb, 0);

					sg_copy_from_buffer(sgt->sgl, sgt->nents, ptr, width * height * 2);
				}
					break;
				case V4L2_MODEL_BUF_TYPE_DMA_CONT:
					break;
			}
		}
	}
#endif

    if (1)
    {
        u32 input_status, hdcp_flag;
        u8 *ptr = NULL;
        input_status = framegrabber_g_input_status(v4l2m_context->framegrabber_handle);
        hdcp_flag = framegrabber_g_hdcp_flag(v4l2m_context->framegrabber_handle);
        if (input_status != FRAMEGRABBER_INPUT_STATUS_OK || hdcp_flag)
        {
            v4l2_model_vb2_image_type_e type =
                    (input_status == FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL) ? V4L2_MODEL_VB2_IMAGE_NO_SIGNAL
                  : (hdcp_flag) ? V4L2_MODEL_VB2_IMAGE_COPY_PROTECTION
                  : V4L2_MODEL_VB2_IMAGE_NO_SIGNAL;
            //printk(".1 type=%d(%d)\n",input_status,vb2_context->image_type);
            if (type != vb2_context->image_type)
            {
				//printk(".2\n");
                if (type == V4L2_MODEL_VB2_IMAGE_NO_SIGNAL)
                {
                    int width, height;
                    framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);
                    load_no_signal_image(v4l2m_context->pic_bmp_handle, vb2_context->image_data, width, height);
                    printk("load no signal done\n");
                }
                else if (type == V4L2_MODEL_VB2_IMAGE_COPY_PROTECTION)
                {
                    int width, height;
                    framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);
                    load_copy_protection_image(v4l2m_context->pic_bmp_handle, vb2_context->image_data, width, height);
                    printk("load copy protection done\n");
                }
                vb2_context->image_type = type;
            }

            ptr = vb2_context->image_data;
            //printk(".\n");
        }

        if (ptr)
        {
			//printk(".3\n");
            switch (vb2_context->queue_type)
            {
                case V4L2_MODEL_BUF_TYPE_VMALLOC:
                    break;
                case V4L2_MODEL_BUF_TYPE_DMA_SG:
                {
                    struct sg_table *sgt = vb2_dma_sg_plane_desc(vb, 0);
                    int width, height;
                    framegrabber_g_out_framesize(v4l2m_context->framegrabber_handle, &width, &height);

                    sg_copy_from_buffer(sgt->sgl, sgt->nents, ptr, width * height * 2);
                }
                    break;
                case V4L2_MODEL_BUF_TYPE_DMA_CONT:
                    break;
            }
        }
    }

	framegrabber_status =
	    framegrabber_g_status(v4l2m_context->framegrabber_handle);

	if ((framegrabber_status &
	     (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
	      FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) ==
	    (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
	     FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) {
		vb2_buffer_done(vb, VB2_BUF_STATE_DONE);
		
		if (vb2_context->buff_cnt==0)
		{
			printk("%s=========================\n", __func__);
			vb2_context->buff_cnt =1;
		}
		
		
	} else {
		//printk("%s framegrabber_status %08x\n", __func__, framegrabber_status);
		vb2_buffer_done(vb, VB2_BUF_STATE_ERROR);
	}
}
#endif

void v4l2_model_linebuffer_done(v4l2_model_handle_t context,int buffer_index, int line_buffer_index)
{
	unsigned long flags;
	struct vb2_buffer *vb;

	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;

	v4l2_model_vb2_context_t *vb2_context =
	    (v4l2_model_vb2_context_t *) v4l2m_context->vb2_context;
	v4l2_model_vb2_buffer_t *buf;//
	framegrabber_status_bitmask_e framegrabber_status;

	return;
	
#if 1
	/* get next v4l2_model_buffer from vb2 context buffer_list */
	spin_lock_irqsave(&vb2_context->queuelock, flags);
	if (!list_empty(&vb2_context->buffer_list)) {
		buf = list_first_entry(&vb2_context->buffer_list,
				       v4l2_model_vb2_buffer_t, list);
		list_del_init(&buf->list);
	} else {
		buf = NULL;
	}
	spin_unlock_irqrestore(&vb2_context->queuelock, flags);
#endif

#if 1
    //buf = v4l2m_context->linebuf;
	if (!buf) {
		printk("no buffer to serve\n");
		return;
	}
#endif
	
#if   LINUX_VERSION_CODE >= KERNEL_VERSION(4,5,0)
	vb = &buf->vb.vb2_buf;
	buf->vb.field = V4L2_FIELD_NONE;
	buf->vb.sequence = vb2_context->frames++;
	buf->vb.flags = 1000*buffer_index + line_buffer_index;
	vb->timestamp = ktime_get_ns();
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4,4,0)
	vb = &buf->vb.vb2_buf;
	buf->vb.field = V4L2_FIELD_NONE;
	buf->vb.sequence = vb2_context->frames++;
	v4l2_get_timestamp(&buf->vb.timestamp);
#else
	vb = &buf->vb;
	vb->v4l2_buf.field = V4L2_FIELD_NONE;
	vb->v4l2_buf.sequence = vb2_context->frames++;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
	v4l2_get_timestamp(&vb->v4l2_buf.timestamp);
#else
	{
		struct timespec ts;
		ktime_get_ts(&ts);
		vb->v4l2_buf.timestamp.tv_sec = ts.tv_sec;
		vb->v4l2_buf.timestamp.tv_usec = ts.tv_nsec / NSEC_PER_USEC;
	}
#endif
#endif

	framegrabber_status =
	    framegrabber_g_status(v4l2m_context->framegrabber_handle);

	if ((framegrabber_status &
	     (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
	      FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) ==
	    (FRAMEGRABBER_STATUS_V4L_START_STREAMING_BIT |
	     FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT)) {
		vb2_buffer_done(vb, VB2_BUF_STATE_DONE);
		
		if (vb2_context->buff_cnt==0)
		{
			printk("%s=========================\n", __func__);
			vb2_context->buff_cnt =1;
		}
		
		
	} else {
		printk("%s VB2_BUF_STATE_ERROR\n", __func__);
		vb2_buffer_done(vb, VB2_BUF_STATE_ERROR);
	}
}

void v4l2_model_reset_video_data(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_cxt = (v4l2_model_context_t *) context;

	#if defined (AVER_CARD_CN312S)
	int buffer_modes,buffer_index,line_buffer_index,line_buffer_size;
	
	if (v4l2m_cxt->current_framebuf_info.vbuf_info)
		v4l2_model_buffer_done(v4l2m_cxt,buffer_modes,buffer_index,line_buffer_index,line_buffer_size);
	#else
	if (v4l2m_cxt->current_framebuf_info.vbuf_info)
		v4l2_model_buffer_done(v4l2m_cxt);
	#endif
		//v4l2_model_buffer_done(v4l2m_cxt);
	v4l2m_cxt->current_framebuf_info.vbuf_info = NULL;
	v4l2m_cxt->current_framebuf_info.rcv_size = 0;
}

void v4l2_model_streamoff(v4l2_model_handle_t context)
{
	v4l2_model_context_t *v4l2m_context = (v4l2_model_context_t *) context;

	printk("%s\n", __func__);

	vb2_streamoff(&v4l2m_context->queue, V4L2_BUF_TYPE_VIDEO_CAPTURE);

}
